/**
 * Dasu Project
 *
 * File:         NetInterfaceClassifier.java
 * RCS:          $Id$
 * Description:  NetInterfaceClassifier class (see below)
 * Author:       Zachary Bischof
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Feb 22, 2011 at 1:01:51 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.NetInterface
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2011, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.NetInterface;

import java.util.HashSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.gudy.azureus2.plugins.utils.Utilities;

import com.azureus.plugins.aznetmon.main.RuntimeExec;

import edu.northwestern.dasu.DasuData;
import edu.northwestern.dasu.Main;

/**
 * @author Zachary Bischof &lt;Zachary.Bischof@eecs.northwestern.edu&gt;
 *
 * The UPNPDeviceHandler class ...
 */
public class NetInterfaceClassifier {
	public enum InterfaceType{
		WIRED,
		WIRELESS,
		UNKNOWN;
	}
	private InterfaceType state;


	public NetInterfaceClassifier(String interfaceName){
		state = InterfaceType.UNKNOWN;
		Main.getPluginInterface().getUtilities();
		Utilities utils = Main.getPluginInterface().getUtilities();
		
		if (utils.isWindows()) {
			windowsClassifer(interfaceName);
		}
		else if (utils.isOSX()) {
			maxosxClassifier(interfaceName);
		}
		else if (utils.isLinux()) {
			linuxClassifier(interfaceName);
		} else if (utils.isFreeBSD()){
			
		} else if (utils.isSolaris()){
			
		} else if (utils.isUnix()){
			
		}
	}
	
	public String getInterfaceTypeString() {
		if(state == InterfaceType.WIRED){
			return "wired";
		} else if (state == InterfaceType.WIRELESS){
			return "wireless";
		} else if (state == InterfaceType.UNKNOWN){
			return "unknown";
		}
		return null;
	}

	private void windowsClassifer(String interfaceName){
		/*///*Other command: netsh wlan show all
		String[] cmd = {"ipconfig","/all"};
		RuntimeExec re = new RuntimeExec(cmd);
		re.exec();

		try{Thread.sleep(1000);}catch(Exception e){}

		String output = re.getStdOut();
		String err = re.getStdErr();
		 */
		String lower = interfaceName.toLowerCase();
		
		HashSet<String> wiredKeywords = new HashSet<String>();
		wiredKeywords.add("ethernet");
		for(String str : wiredKeywords){
			if(lower.contains(str))
				this.state = InterfaceType.WIRED;
		}
		
		HashSet<String> wirelessKeywords = new HashSet<String>();
		wirelessKeywords.add("wireless");
		wirelessKeywords.add("wifi");
		wirelessKeywords.add("wi-fi");
		wirelessKeywords.add("80211");
		wirelessKeywords.add("802.11");
		for(String str : wirelessKeywords){
			if(lower.contains(str))
				this.state = InterfaceType.WIRELESS;
			
		}
		

	}

	private void maxosxClassifier(String interfaceName){
		String[] cmd = {"system_profiler","SPNetworkDataType"};
		RuntimeExec re = new RuntimeExec(cmd);
		re.exec();
		String output = re.getStdOut();

		HashSet<String> ethernets = new HashSet<String>();
		{
			String ethExp = "Type: Ethernet\n.*Hardware: Ethernet\n.*Device Name: (.*)";
			Pattern patManufacturer = Pattern.compile(ethExp);
			Matcher m = patManufacturer.matcher(output);
			while(m.find()){
				if(m.groupCount() >= 1){
					ethernets.add(m.group(1));
				}
			}
		}

		HashSet<String> wirelesses = new HashSet<String>();
		{
			String wifiExp = "Type: AirPort\n.*Hardware: AirPort\n.*Device Name: (.*)";
			Pattern patManufacturer = Pattern.compile(wifiExp);
			Matcher m = patManufacturer.matcher(output);
			while(m.find()){
				if(m.groupCount() >= 1){
					wirelesses.add(m.group(1));
				}
			}
		}
		
		if(ethernets.contains(interfaceName)){
			state = InterfaceType.WIRED;
		} else if (wirelesses.contains(interfaceName)){
			state = InterfaceType.WIRELESS;
		} else {
			state = InterfaceType.UNKNOWN;
		}
	}	

	private void linuxClassifier(String interfaceName){
		String[] cmd = {"/sbin/iwconfig"};
		RuntimeExec re = new RuntimeExec(cmd);
		re.exec();

		String err = re.getStdErr();
		String[] toks = err.split("\n");
		for(String token : toks){
			if(token.startsWith(interfaceName)){
				if(token.contains("802.11") || token.contains("80211")){
					state = InterfaceType.WIRELESS;
				}
				if(token.toLowerCase().contains("no wireless extensions")){
					state = InterfaceType.WIRED;
				}
			}
		}
	}

	public boolean isWirelessConnection(){
		if(state == InterfaceType.WIRELESS){
			return true;
		} else {
			return false;
		}
	}

	public boolean isWiredConnection(){
		if(state == InterfaceType.WIRED){
			return true;
		} else {
			return false;
		}
	}

	public InterfaceType getInterfaceType(){
		return state;
	}

}
