/**
 * Dasu Project
 *
 * File:         NullReporter.java
 * RCS:          $Id$
 * Description:  NullReporter class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 8, 2007 at 10:00:02 AM
 * Language:     Java
 * Package:      edu.northwestern.news.database
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.database;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.measurement.TraceEntry;

/**
 * @author Mario Sanchez &lt;msanchezs@eeccs.northwestern.edu&gt;
 *
 * The CallbackReporter class has all the methods of 
 * a database reporter, but stores all the values in the disk. Useful for 
 * periods of network disconnection. 
 */
public class CallbackReporter implements DatabaseReporter {

	private static CallbackReporter instance = new CallbackReporter();
	protected TreeMap<Long, List<Object>> dbQueue; 
	//sessionStartTime, Map<table, values>>
	private static boolean DEBUG = false;
	
	public synchronized static DatabaseReporter getInstance() {

		if (instance == null
				|| (!(instance instanceof CallbackReporter) && !Main
						.isShuttingDown())) {
			instance = new CallbackReporter();
		}

		return instance;
	}

	
	public CallbackReporter() {
		dbQueue = DasuManager.getPersistentDataManager().getDbQueue();
	}
	
	public boolean canConnect() {
		return PhpReporter.getInstance().canConnect();		
	}

	public boolean connect() {
		return PhpReporter.getInstance().connect();
	}

	public void disconnect() {
		PhpReporter.getInstance().disconnect();
	}

	public boolean insertDownloadStatDynamic(int selfId, int downloadId,
			long timeRecorded, int numPeers, float availability, int completed,
			int discarded, int downloadRate, int uploadRate, int shareRatio,
			int numKnownNonSeeds, int numKnownSeeds, int state, int priority,
			long downloaded, long uploaded, int position, 
			int totalNumKnownNonSeeds, int totalNumKnownSeeds, long hashFails,
			int connectedSeeds, int connectedLeechers, long swarmPeerSpeed) {
		
		return pushDbCall("insertDownloadStatDynamic", new Object[]{selfId, downloadId, timeRecorded, numPeers, 
				availability, completed, discarded, downloadRate, uploadRate, shareRatio,
				numKnownNonSeeds, numKnownSeeds, state, priority, downloaded, uploaded, position, 
				totalNumKnownNonSeeds, totalNumKnownSeeds, hashFails, connectedSeeds, connectedLeechers, swarmPeerSpeed});
	}

	public boolean insertDownloadStatSummary(int selfId, int downloadId,
			long currentTime, long downloadingTime, long seedingTime,
			int maxDownloadRateLimit, int maxUploadLimit, long totalUploaded,
			long totalDownloaded, long size, long pieceSize) {

		return pushDbCall("insertDownloadStatSummary", new Object[]{selfId, downloadId, currentTime, downloadingTime, seedingTime,
			maxDownloadRateLimit, maxUploadLimit, totalUploaded, totalDownloaded, size, pieceSize});

	}

	public boolean insertLocalPeerStatDynamic(int selfId,
			long currentTime, int uploadRate, int downloadRate, 
			int maxDownloadRate, int maxUploadRate, boolean isAutoSpeed, int rst, 
			int numConnections, int passiveOpens, int activeOpens, int failedConns,
			int numTorrents, int numSeedingTorrents, int numLeechingTorents, 
			int numConnectedSeeds, int numConnectedLeechers,
			long bytesSentLpsd, long bytesReceivedLpsd, long bytesSentNetstat, long bytesReceivedNetstat, long swarmPeerSpeed,
			long numberOfActiveConnections, long totalBytesSent, long totalBytesReceived, long totalPacketsSent,
			long totalPacketsReceived, long uptime) {
		
		return pushDbCall("insertLocalPeerStatDynamic", new Object[]{selfId, currentTime, uploadRate, downloadRate, 
			maxDownloadRate, maxUploadRate, isAutoSpeed, rst, 
			numConnections, passiveOpens, activeOpens, failedConns,
			numTorrents, numSeedingTorrents, numLeechingTorents, 
			numConnectedSeeds, numConnectedLeechers,
			bytesSentLpsd, bytesReceivedLpsd, bytesSentNetstat, bytesReceivedNetstat, swarmPeerSpeed,
			numberOfActiveConnections, totalBytesSent, totalBytesReceived, totalPacketsSent,
			totalPacketsReceived, uptime});
		
	}


	public boolean insertRemotePeerStatDynamic(int selfId, int peerId,
			long currentTime, int bytesReceivedFromPeer, int bytesSentToPeer,
			int currentReceiveRate, int currentSendRate, boolean isSeed,
			double ping, boolean isInterested, boolean isInteresting) {

		return pushDbCall("insertRemotePeerStatDynamic", new Object[]{ selfId, peerId,
				currentTime, bytesReceivedFromPeer, bytesSentToPeer,
				currentReceiveRate, currentSendRate, isSeed,
				ping, isInterested, isInteresting});	
	}

	public boolean insertRemotePeerStatStatic(int selfId, int peerId,
			int peerClient, long uptime, int tcpPort, int udpPort,
			boolean sameCluster, int type, String encryption, long pieceSize) {

		return pushDbCall("insertRemotePeerStatStatic", new Object[]{ selfId, peerId,
			peerClient, uptime, tcpPort, udpPort,
			sameCluster, type, encryption, pieceSize});

	}

	public long insertTraceEntry(int sourceId, int destId, long time, int entries, int taskId) {
		
		pushDbCall("insertTraceEntry", new Object[]{sourceId, destId, time, entries, taskId});		
		return 0;
		
	}

	public boolean insertTraceEntryData(long entryId, int[] myRouterIds,
			TraceEntry te) {
		
		return pushDbCall("insertTraceEntryData", new Object[]{entryId, myRouterIds, te});

	}

	public int makeId(String tableName, String colName, String value) {
		return PhpReporter.getInstance().makeId(tableName, colName, value);
	}

	public boolean registerUsage(int myId, long GMTTime, String uuid, long currentTime,
			String timeZone, boolean failed) {

		return pushDbCall("registerUsage",new Object[]{myId, GMTTime, uuid, currentTime,
			timeZone, failed});

	}

	public void reportError(Exception e) {

		pushDbCall("reportError", new Object[]{e.toString()});

	}

	public void setExtendedConnection(boolean extend) {
		PhpReporter.getInstance().setExtendedConnection(extend);
	}

	public void stopReporting() {
		PhpReporter.getInstance().stopReporting();

	}

	public void flush() {
		PhpReporter.getInstance().flush();		
	}

	public Map<String, Integer> makeIdBatch(String tableName,
			String columnName, List<String> values) {
		return PhpReporter.getInstance().makeIdBatch(tableName, columnName, values);		
	}
	
	public void updateConfig(){
		
	}

	public boolean pushDbCall(String methodName, Object[] args) {		
		
		long time = System.nanoTime();
		ArrayList<Object> call = new ArrayList<Object>();
		call.add(methodName);
		for ( int i=0; i < args.length; i++ )
			call.add(args[i]);
		
		if (DEBUG)
			System.out.println(call);
		
		synchronized(dbQueue) {
			try {
				dbQueue.put(time, call);				
			}
			catch (Exception e) {
				e.printStackTrace();
				return false;
			}
		}
		return true;
		
	}
	
	public void reportIpChange(int oldIp, int newIp, int localIp,
			long currentGMTTime) {

		pushDbCall("reportIpChange", new Object[]{ oldIp, newIp, localIp,
			currentGMTTime});
		
	}

	public boolean insertPingResultDasu(int destId, int sourceId, float rtt,
			long time, int destType, int taskId) {

		return pushDbCall("insertPingResultDasu", new Object[]{destId, sourceId, rtt,
			time, destType, taskId});
		
	}

	public boolean insertHttpResultDasu(int sourceId, int urlId,  
    		double downloadDuration, double downloadSpeed, long time, 
    		int noBytes, int taskId) {

		return pushDbCall("insertHttpResultDasu", new Object[]{ sourceId, urlId,  
    		downloadDuration, downloadSpeed, time, 
    		noBytes, taskId});

	}
	
	public boolean insertUpnpTestResult(int sourceId, long testTime, int port,
			String protocol, int success) {

		return pushDbCall("insertUpnpTestResult", new Object[]{sourceId, testTime, port,
			protocol, success});
		
	}

	public String getIspInfo(String myIp) {
		return PhpReporter.getInstance().getIspInfo(myIp);
	}

	public boolean insertUserReports(int sourceId, String downloadIds,
			String comment) {
		
		return pushDbCall("insertUserReports", new Object[]{sourceId, downloadIds,
			comment});
	}

	public boolean insertInterference(int sourceId, String type, long timestamp) {
		
		return pushDbCall("insertInterference", new Object[]{sourceId, type, timestamp});
	}

	public boolean insertIspInfo(int sourceId, String prefix, String ispname, long timeRecorded) {
		
		return pushDbCall("insertIspInfo", new Object[]{sourceId, prefix,ispname, timeRecorded});
		
	}

	public boolean insertTask(int sourceId, long timeStamp, int taskId) {
		
		System.out.println("insertTask: " + sourceId + " " + timeStamp + " " + taskId);

		return pushDbCall("insertTask", new Object[]{sourceId, timeStamp, taskId});
	}

	public boolean insertRSTStat(int sourceId, int destId, long downloadId,
			int numRst, long timeRecorded, int type, int port,
			String bitTorrentClient, String source, int isSeed, int amSeed,
			int numSeeds, int numPeers, String encryption) {

		return pushDbCall("insertRSTStat", new Object[]{sourceId, destId, downloadId,
			numRst, timeRecorded, type, port,
			bitTorrentClient, source, isSeed, amSeed,
			numSeeds, numPeers, encryption});

	}

	public boolean insertInterferenceEvent(int sourceId, String type,
			long timestamp, long downloadId, long droppedTo, float mean) {
		
		return pushDbCall("insertInterferenceEvent", new Object[]{ sourceId, type,
			timestamp, downloadId, droppedTo, mean});
		
	}

	public boolean insertLocalPeerStatStatic(int selfId, long uptime,
			int maxUploadRate, int maxDownloadRate,
			int maxPeerConnectionsGlobal, int maxPeerConnectionsPerTorrent,
			int maxPeerConnectionsPerTorrentSeeding, int maxUploadRateSeeding,
			int maxSeedsPerTorrent, int lazyBitfield, int requireEncryption,
			String encryptionMinLevel, int allowNonEncryptedOUT,
			int allowNonEncryptedIn, int useCryptoPortExtension, int tcpPortIn,
			int udpPortIn) {
		
		return pushDbCall("insertLocalPeerStatStatic", new Object[]{selfId, uptime,
			maxUploadRate, maxDownloadRate,
			maxPeerConnectionsGlobal, maxPeerConnectionsPerTorrent,
		    maxPeerConnectionsPerTorrentSeeding, maxUploadRateSeeding,
			maxSeedsPerTorrent, lazyBitfield, requireEncryption,
			encryptionMinLevel, allowNonEncryptedOUT,
			allowNonEncryptedIn, useCryptoPortExtension, tcpPortIn,
			udpPortIn});
		
	}

	public boolean insertRemotePeerConnectionStat(int sourceId, int destId,
			long timeRecorded, int type, int port, String bitTorrentClient,
			int isSeed, int amSeed, int numSeeds, int numPeers,
			String encryptionLevel) {

		return pushDbCall("insertRemotePeerConnectionStat", new Object[]{sourceId, destId,
			timeRecorded, type, port, bitTorrentClient,
			isSeed, amSeed, numSeeds, numPeers,
			encryptionLevel});

	}

	public boolean getCanSendData() {
		return PhpReporter.getInstance().getCanSendData();
	}

	public String getCurrentUrl() {
		return PhpReporter.getInstance().getCurrentUrl();
	}

	public Charset getDefaultCharSet() {
		return PhpReporter.getInstance().getDefaultCharSet();
	}

	public String getDefaultServer() {
		return PhpReporter.getInstance().getDefaultServer();
	}

	public boolean isUseGzip() {
		return PhpReporter.getInstance().isUseGzip();
	}

	public void setLastFailTime(long l) {
		PhpReporter.getInstance().setLastFailTime(l);		
	}

	public void updateLastFailTime() {
		PhpReporter.getInstance().updateLastFailTime();
	}

	public boolean insertSpeedTestResult(int sourceId, long testTime,
			int downloadSpeed, int uploadSpeed, int encryption, char type,
			int downloadSpeedNetstat, int uploadSpeedNetstat) {
		
		return pushDbCall("insertSpeedTestResult", new Object[]{sourceId, testTime,
			downloadSpeed, uploadSpeed, encryption, type,
			downloadSpeedNetstat, uploadSpeedNetstat});
	}

	public long insertScrapeEntry(String type, int peerId, String url, String hash, int entries,
			int numSeeds, int numLeechers, int taskId, long time) {

		pushDbCall("insertScrapeEntry", new Object[]{type, peerId, url, hash, entries,
				numSeeds, numLeechers, taskId, time});
		return 0;
	}

	public boolean insertScrapeEntryData(long entryId, int peerid, int tcp,
			int udp) {
		 
		return pushDbCall("insertScrapeEntryData", new Object[]{entryId, peerid, tcp,
			udp});
		
	}

	public boolean insertDnsResult(int myId, int dnsId, long requestTime,
			long responseTime, String query, String type, String Options,
			String response, boolean failed, long taskId) {
		
		return pushDbCall("insertDnsResult", new Object[]{ myId, dnsId, requestTime,
			responseTime, query, type, Options,
			response, failed, taskId});
	}

	public boolean insertDnsMapping(int myIp, int publicDns, int privateDns,
			String type, int order, long timeRecorded) {

		return pushDbCall("insertDnsMapping", new Object[]{ myIp, publicDns, privateDns,
			type, order, timeRecorded});

	}

	public boolean insertNdtResultDasu(int myId, String osName, String osArch,
			String osVer, String interfaceName, String javaVer, String server,
			double rate, double rate2, double mylink, int mismatch,
			double avgrtt, int packetSize, int connectionStalledTimes,
			double connectionIdlePercentage, int packetQueuing,
			int packetQueuing2,
			double throughputPacketQueueingDetectedPercentage,
			double connectionReceiverLimitedPercentage,
			double connectionSenderLimitedPercentage,
			double connectionNetworkLimitedPercentage, int RFC2018, int RFC896,
			int RFC3168, int RFC1323TimeStamping, int RFC1323WindowScaling,
			int serverScalingFactor, int clientScalingFactor,
			int serverBehindFirewall, int clientBehindFirewall,
			double theoreticalNetworkLimit, double serverBuffer,
			double serverThroughputLimit, double clientBuffer,
			double clientThroughputLimit, long time, int failed,
			String errorMsg, double rateNetstat, double rateNetstat2,
			double networkFlowControlThroughputLimit, int congestion,
			int congestionSignals, int fastRetran, int timeouts,
			double outOfOrderPercentage, int pktsRetrans, int dupAcksIn,
			int ksRcvd, double packetLoss) {

		return pushDbCall("insertNdtResultDasu", new Object[]{myId, osName, osArch,
			osVer, interfaceName, javaVer, server,
			rate, rate2, mylink, mismatch,
			avgrtt, packetSize, connectionStalledTimes,
			connectionIdlePercentage, packetQueuing,
			packetQueuing2,
			throughputPacketQueueingDetectedPercentage,
			connectionReceiverLimitedPercentage,
			connectionSenderLimitedPercentage,
			connectionNetworkLimitedPercentage, RFC2018, RFC896,
			RFC3168, RFC1323TimeStamping, RFC1323WindowScaling,
			serverScalingFactor, clientScalingFactor,
			serverBehindFirewall, clientBehindFirewall,
			theoreticalNetworkLimit, serverBuffer,
			serverThroughputLimit, clientBuffer,
			clientThroughputLimit, time, failed,
			errorMsg, rateNetstat, rateNetstat2,
			networkFlowControlThroughputLimit, congestion,
			congestionSignals, fastRetran, timeouts,
			outOfOrderPercentage, pktsRetrans, dupAcksIn,
			ksRcvd, packetLoss});
	}

	public boolean insertStaticUPnPInfo(int myId, String UpstreamMaxBitRate,
			String DownstreamMaxBitRate, String NATEnabled,
			String Layer1UpstreamMaxBitRate, String Layer1DownstreamMaxBitRate,
			String PortMappingEnabled, String WANAccessType,
			String MaximumActiveConnections, String ExternalIPAddress) {

		return pushDbCall("insertStaticUPnPInfo", new Object[]{ myId, UpstreamMaxBitRate,
			DownstreamMaxBitRate, NATEnabled,
			Layer1UpstreamMaxBitRate, Layer1DownstreamMaxBitRate,
			PortMappingEnabled, WANAccessType,
			MaximumActiveConnections, ExternalIPAddress});
	}

	public boolean insertConfigEventDasu(int sourceId, String event, long timestamp) {

		return PhpReporter.getInstance().insertConfigEventDasu(sourceId, event, timestamp);
		
	}

	public boolean insertFeedback(String type, String name, String email, String message, String dump,
			long systime, long dasuGmtTime) {

		return pushDbCall("insertFeedback", new Object[]{type, name, email,
			message, dump, systime, dasuGmtTime});
	}

	public boolean insertDnsTimeout(int myDnsId, int myId, long startTime,
			long endTime, int giveUp) {
		
		System.out.println("insertDnsTimeout: " + myDnsId + " " + myId + " " + startTime + " " + endTime + " " + giveUp);
				
		return pushDbCall("insertDnsTimeout", new Object[]{myDnsId, myId, startTime,
			endTime, giveUp});
	}


	public void stop() {
		
	}

}
