package edu.northwestern.dasu.database;

import java.nio.charset.Charset;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import edu.northwestern.dasu.measurement.TraceEntry;

public interface DatabaseReporter {

	public abstract void setExtendedConnection(boolean extend);

	/**
	 * Establishes a connection to the databse.
	 * 
	 * @return false if the connection was not made; true otherwise
	 */
	public abstract boolean connect();

	public abstract boolean canConnect();

	/**
	 * @param edgeServer
	 * @return
	 * @throws SQLException
	 */
	public abstract int makeId(String tableName, String colName, String value);

	/**
	 * 
	 */
	public abstract void disconnect();

	/**
	 * @param edgeId
	 * @param myId
	 * @param destId
	 * @param middleId
	 * @param rtt
	 * @param time
	 */
	public abstract boolean insertPingResultDasu(int destId, int sourceId,float rtt,  
			long time, int destType, int taskId);
	
	public abstract boolean insertHttpResultDasu(int sourceId, int url,  
    		double downloadDuration, double downloadSpeed, long time, 
    		int noBytes, int taskId);

	public abstract boolean insertSpeedTestResult(int sourceId, long testTime, 
			int downloadSpeed, int uploadSpeed, int encryption, char type, 
			int downloadSpeedNetstat, int uploadSpeedNetstat);
	
	public abstract boolean insertUpnpTestResult(int sourceId, long testTime, 
        	int port, String protocol, int success);
	
	public boolean insertNdtResultDasu(int myId, String osName, String osArch,
			String osVer, String interfaceName, String javaVer, String server,
			double rate, double rate2, double mylink, int mismatch,
			double avgrtt, int packetSize, 
			int connectionStalledTimes,
			double connectionIdlePercentage, int packetQueuing,
			int packetQueuing2,
			double throughputPacketQueueingDetectedPercentage,
			double connectionReceiverLimitedPercentage,
			double connectionSenderLimitedPercentage,
			double connectionNetworkLimitedPercentage, int RFC2018,
			int RFC896, int RFC3168, int RFC1323TimeStamping,
			int RFC1323WindowScaling, int serverScalingFactor,
			int clientScalingFactor, int serverBehindFirewall,
			int clientBehindFirewall, double theoreticalNetworkLimit,
			double serverBuffer, double serverThroughputLimit,
			double clientBuffer, double clientThroughputLimit, long time,
			int failed, String errorMsg, double c2sRateNetstat, double s2cRateNetstat,
			double networkFlowControlThroughputLimit, int congestion, int congestionSignals,
			int fastRetran, int timeouts, double outOfOrderPercentage, 
			int pktsRetrans, int dupAcksIn, int sACKsRcvd, double packetLoss);

	
	public abstract boolean insertRSTStat(int sourceId, int destId, 
			long downloadId, int numRst, long timeRecorded, int type,
			int port, String bitTorrentClient, String source, int isSeed, 
			int amSeed, int numSeeds, int numPeers, String encryptionLevel);
	
	public abstract boolean insertRemotePeerConnectionStat(int sourceId, int destId, 
			long timeRecorded, int type,
			int port, String bitTorrentClient, int isSeed, 
			int amSeed, int numSeeds, int numPeers, String encryptionLevel);
	
	/**
	 * returns the id for the entry description
	 * 
	 * @param myId
	 * @param customerId
	 * @return
	 */

	public abstract void reportError(Exception e);

	public abstract long insertTraceEntry(int sourceId, int destId, long time, 
			int entries, int taskId);

	public abstract boolean insertTraceEntryData(long entryId, int[] myRouterIds,
			TraceEntry te);
	
	public abstract long insertScrapeEntry(String type, int peerId, String url,
			String hash, int entries, int numSeeds, int numLeechers, int taskId, long time);
	
	public abstract boolean insertScrapeEntryData(long entryId,
			int peerId, int tcp, int udp);

	public abstract boolean insertDnsResult(int myId, int dnsId, 
			long requestTime, long responseTime, String query,
			String type, String Options, String response, boolean failed,
			long taskId);


	/**
	 * Inserts local peer stats.
	 * 
	 * @param selfId
	 *            local peer id
	 * @param currentTime
	 *            the time of the stat being reported
	 * @param uploadRate
	 *            the upload rate at that time
	 * @param downloadRate
	 *            the download rate at that time
	 * @param b 
	 * @param j 
	 * @param i 
	 * @return
	 */

	public abstract boolean insertLocalPeerStatDynamic(int selfId,
			long currentTime, int uploadRate, int downloadRate, 
			int maxDownloadRate, int maxUploadRate, boolean isAutoSpeed, int rst, 
			int numConnections, int passiveOpens, int activeOpens, int failedConns,
			int numTorrents, int numSeedingTorrents, int numLeechingTorents, 
			int numConnectedSeeds, int numConnectedLeechers,
			long bytesSentLpsd, long bytesReceivedLpsd, long bytesSentNetstat, long bytesReceivedNetstat, long swarmPeerSpeed,
			long numberOfActiveConnections, long totalBytesSent, long totalBytesReceived, long totalPacketsSent, 
			long totalPacketsReceived, long uptime);

	/**
	 * Inserts once-per-session peer stats
	 * 
	 * @param selfId
	 *            the peer id
	 * @param uptime
	 *            the peer session lifetime
	 * @param maxUploadRate
	 *            the max upload rate
	 * @param maxDownloadRate
	 *            the max download rate
	 */

	public abstract boolean insertLocalPeerStatStatic(int selfId, long uptime,
			int maxUploadRate, int maxDownloadRate, int maxPeerConnectionsGlobal, 
			int maxPeerConnectionsPerTorrent, int maxPeerConnectionsPerTorrentSeeding, 
			int maxUploadRateSeeding, int maxSeedsPerTorrent, int lazyBitfield, int requireEncryption,
			String encryptionMinLevel, int allowNonEncryptedOUT, int allowNonEncryptedIn,
			int useCryptoPortExtension, int tcpPortIn, int udpPortIn);

	/**
	 * Reports once-per-session peer stats.
	 * 
	 * @param selfId
	 *            the id of the local peer
	 * @param peerId
	 *            the id of the peer being reported on
	 * @param peerClient
	 *            the id of the client software being used
	 * @param uptime
	 *            the duration of the connection
	 * @param tcpPort
	 *            the tcp port being used
	 * @param udpPort
	 *            the udp port being used
	 * @param sameCluster
	 *            true if node is in same cluster
	 */
	public abstract boolean insertRemotePeerStatStatic(int selfId, int peerId,
			int peerClient, long uptime, int tcpPort, int udpPort,
			boolean sameCluster, int type, String encryption, long pieceSize);

	/**
	 * Reports periodic data from remote peers.
	 * 
	 * @param selfId
	 *            local peer
	 * @param peerId
	 *            remote peer (being measured)
	 * @param currentTime
	 *            time of measurement
	 * @param bytesReceivedFromPeer
	 *            bytes received at that point
	 * @param bytesSentToPeer
	 *            bytes sent at that point
	 * @param currentReceiveRate
	 *            current download rate
	 * @param currentSendRate
	 *            current upload rate
	 * @param isSeed
	 *            whether it is a seed
	 * @param ping
	 *            ping latency
	 * @param isInteresting
	 * @param isInterested
	 * @return
	 */
	public abstract boolean insertRemotePeerStatDynamic(int selfId, int peerId,
			long currentTime, int bytesReceivedFromPeer, int bytesSentToPeer,
			int currentReceiveRate, int currentSendRate, boolean isSeed,
			double ping, boolean isInterested, boolean isInteresting);

	/**
	 * 
	 * @param selfId
	 *            peer id
	 * @param downloadId
	 *            the id for this download for this session
	 * @param currentTime
	 *            current time in ms
	 * @param downloadingTime
	 *            time spent downloading (s)
	 * @param seedingTime
	 *            time spent seeding (s)
	 * @param maxDownloadRateLimit
	 *            dl rate limit
	 * @param maxUploadLimit
	 *            upload rate limit
	 * @param totalUploaded
	 *            total bytes uploaded for this download
	 * @param totalDownloaded
	 *            total bytes downloaded for this download
	 * @param size
	 *            size of the download
	 * @param pieceSize
	 *            piece size for dl
	 */
	public abstract boolean insertDownloadStatSummary(int selfId, int downloadId,
			long currentTime, long downloadingTime, long seedingTime,
			int maxDownloadRateLimit, int maxUploadLimit, long totalUploaded,
			long totalDownloaded, long size, long pieceSize);

	/**
	 * 
	 * @param selfId
	 *            peer id
	 * @param downloadId
	 *            download id
	 * @param timeRecorded
	 *            time recorded
	 * @param numPeers
	 *            num peers connected
	 * @param availability
	 *            data availability
	 * @param completed
	 *            completion in thousands
	 * @param discarded
	 *            num bytes discarded
	 * @param downloadRate
	 *            download rate
	 * @param uploadRate
	 *            upload rate
	 * @param shareRatio
	 *            share ratio in thousandths
	 * @param numKnownNonSeeds
	 *            total non seeds known about
	 * @param numKnownSeeds
	 *            total seeds known about
	 * @param state
	 *            download state
	 * @param priority
	 * @param uploaded 
	 * @param downloaded 
	 * @param position 
	 * @param numOnoPeers 
	 * @param totalNumKnownSeeds 
	 * @param totalNumKnownNonSeeds 
	 * @param hashFails 
	 * @return success
	 */
	public abstract boolean insertDownloadStatDynamic(int selfId,
			int downloadId, long timeRecorded, int numPeers,
			float availability, int completed, int discarded, int downloadRate,
			int uploadRate, int shareRatio, int numKnownNonSeeds,
			int numKnownSeeds, int state, int priority, long downloaded, 
			long uploaded, int position,
			int totalNumKnownNonSeeds, int totalNumKnownSeeds, long hashFails,
			int connectedSeeds, int connectedLeechers, long swarmPeerSpeed);

	public abstract boolean registerUsage(int myId, long GMTTime, String uuid, long currentTime,
			String timeZone, boolean failed);

	public abstract void stopReporting();
	
	public abstract void flush();
	
	public abstract void stop();
	
	public abstract Map<String, Integer> makeIdBatch(String tableName,
			String columnName, List<String> values);
	
	public abstract void updateConfig();

	public abstract void reportIpChange(int oldIp, int newIp,
			int localIp, long currentGMTTime);

	public abstract String getIspInfo(String myIp);

	public abstract boolean insertUserReports(int sourceId, String downloadIds, String comment);
	
	public abstract boolean insertIspInfo(int sourceId, String prefix, String ispname, long timeRecorded);
	
	public abstract boolean insertInterference(int sourceId, String type, long timestamp);
	
	public abstract boolean insertStaticUPnPInfo(int myId, String UpstreamMaxBitRate, String DownstreamMaxBitRate, String NATEnabled,
			String Layer1UpstreamMaxBitRate, String Layer1DownstreamMaxBitRate, String PortMappingEnabled, String WANAccessType, 
			String MaximumActiveConnections, String ExternalIPAddress);

	public abstract boolean insertInterferenceEvent(int sourceId, String type, long timestamp, long downloadId, long droppedTo, float mean);

	public abstract boolean insertTask(int sourceId, long timeStamp, int taskId);
	
	public abstract boolean insertDnsMapping(int myIp, int publicDns, int privateDns, String type, int order, long timeRecorded);

	public abstract boolean insertDnsTimeout(int myDnsId, int myId, long startTime, long endTime, int giveUp);
	
	public abstract void updateLastFailTime();

	public abstract void setLastFailTime(long l);

	public abstract boolean getCanSendData();

	public abstract String getDefaultServer();

	public abstract String getCurrentUrl();

	public abstract Charset getDefaultCharSet();

	public abstract boolean isUseGzip();

	public abstract boolean insertConfigEventDasu(int sourceId, String event, long timestamp);
	
	public abstract boolean insertFeedback(String type, String name, String email, String message, String dump,
			long systime, long dasuGmtTime);

	
}