/**
 * Dasu Project
 *
 * File:         sendData.java
 * RCS:          $Id$
 * Description:  sendData class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 9, 2010 at 12:15:40 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.database
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.database;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.Map.Entry;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import edu.northwestern.dasu.Main;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The sendData class ...
 */
public class SendData extends Thread {

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	private TreeMap<Double, String> urls = new TreeMap<Double, String>();

	private boolean canSendData = PhpReporter.getInstance().getCanSendData();
	private static final boolean TESTING = false;
	private static final boolean TESTING_GZIP = false;
	private static boolean useGzip = PhpReporter.getInstance().isUseGzip(); 
	private String currentUrl = PhpReporter.getInstance().getCurrentUrl();
	
	private static final String DEFAULT_SERVER = PhpReporter.getInstance().getDefaultServer();
	private static final String TEST_GZIP_SERVER = DEFAULT_SERVER;
	private static Charset DEFAULT_CHARSET = PhpReporter.getInstance().getDefaultCharSet();
	
	private String header;
	private ByteArrayOutputStream data;
	
	public SendData(String header, ByteArrayOutputStream data) {
		
		this.header = header;
		this.data = data;
		
	}
	
	public void run() {
	
		if (TESTING)
			System.out.println("\nSenData Thread: Create! " + header);
		
		sendData(header, data);		
	}

	private long sendData(String header, ByteArrayOutputStream data) {
		
		try {
			if (!canSendData)
				return -1;
			if (data == null)
				data = new ByteArrayOutputStream();
			String response = getResponse(header, data);
						
			if (TESTING)
				System.out.println("SenData Thread: Response: " + header + ": " + response);
			
			
			if (response == null || response.length()==0)
				return -1;
			if (response.contains("Error")) {
				data.reset();
				return -1;
			} else if (response.contains("Fail")) {
				data.reset();
				PhpReporter.getInstance().updateLastFailTime();
				return -1;
			} else if (response.contains("Success")) {
				data.reset();
				return 0;
			} else if (response.length() > 0) {
				if (header.equals("type=config")) {
					//parseConfig(response);
					PhpReporter.getInstance().updateConfig();
					return 0;
				} else {
					try {
						if (data != null && data.size() > 0)
							data.reset();
						// must be encoded as <length>:<number>
						String pair[] = response.split("\\:");
						if (pair.length != 2)
							return -1;
						int size = Integer.parseInt(pair[0]);
						if (pair[1].length() == size)
							return Integer.parseInt(pair[1]);
						else
							return -1;
					} catch (NumberFormatException e) {
						return -1;
					}
				}
			} else {
				return -1;
			}

		} catch (MalformedURLException e1) {
			if (TESTING)
			e1.printStackTrace();
		} catch (IOException e) {
			if (TESTING)
				e.printStackTrace();
			
			PhpReporter.getInstance().updateLastFailTime();
		} finally {
			if (data!=null) data.reset();
		}
		return -1;
	}
	

	private String getNewRandomUrl(String oldUrl) {
		if (TESTING_GZIP) return TEST_GZIP_SERVER;
		useGzip = false;
		String thisUrl = oldUrl;
		if (urls.size() <= 1)
			return thisUrl;
		else {
			do {
				double val = Main.getRandom().nextDouble();
				for (Entry<Double, String> ent : urls.entrySet()) {
					if (val <= ent.getKey())
						thisUrl = ent.getValue();
				}
			} while (!thisUrl.equals(oldUrl));
		}
		return thisUrl;
	}
	
	private String getResponse(String header, ByteArrayOutputStream data)
	throws IOException {

		if (!PhpReporter.getInstance().canConnect()) 
			return null;
		
		URL url = null;
		URLConnection urlConn = null;
		InputStream is = null;
		
		int count = 0;
		while (true) {
			try {
				url = new URL(currentUrl);
				if (TESTING) {
					String h = header;
					if (header.contains("\n"))
						h = header.split("\\n")[0];
					if (h == null || h.length() == 0) {
						h = data.toString();
						if (h.contains("\n"))
							h = h.split("\\n")[0];
					}
					System.out.print("["
							+ DateFormat.getDateTimeInstance().format(
									new Date()) + "]" + " Connecting to: "
							+ url + "(" + h + ")");
				}
			} catch (MalformedURLException e2) {
				currentUrl = getNewRandomUrl(currentUrl);
				count++;
				if (count >= urls.size()) {
					PhpReporter.getInstance().updateLastFailTime();
					return null;
				}
				continue;
			}
		
			// URL connection channel.
			try {
				urlConn = url.openConnection();
				break;
			} catch (IOException e2) {
				currentUrl = getNewRandomUrl(currentUrl);
				count++;
				if (count >= urls.size()) {
					PhpReporter.getInstance().updateLastFailTime();
					return null;
				}
			}
		}
		
		PhpReporter.getInstance().setLastFailTime(-1L); // reset failure time since we are successful
		
		try {
			byte headerB[] = DEFAULT_CHARSET.encode(header).array();
			byte dataB[] = data.toByteArray();
			int rawLength = headerB.length+dataB.length;
			// Let the run-time system (RTS) know that we want input.
			urlConn.setDoInput(true);
			// Let the RTS know that we want to do output.
			urlConn.setDoOutput(true);
			// No caching, we want the real thing.
			urlConn.setUseCaches(false);
			// Specify the content type.
			urlConn.setRequestProperty("Content-Type",
					"application/x-www-form-urlencoded");
			urlConn.setRequestProperty("accept-encoding",
		            "gzip");
			// java gzip implementation wastes more space than it saves if size 
			// is less than 150
			if (rawLength>=150 && useGzip)urlConn.setRequestProperty("Content-Encoding",
		    "gzip");
			
			// Send POST output.
			OutputStream os = urlConn.getOutputStream();
			
			if (useGzip){
				if (rawLength<150){
		//			os = new NewsGZIPOutputStream(os, 512, Deflater.BEST_COMPRESSION);
				}
				else os = new GZIPOutputStream(os);
			}
			
			if (TESTING) System.out.print("[Sending "+(headerB.length+dataB.length)+" bytes] ");
			os.write(headerB);
			if (data.size() > 0)
				os.write(dataB);
		
			if (useGzip && os instanceof GZIPOutputStream) ((GZIPOutputStream)os).finish();
			os.flush();		
			os.close();
			
			// Get response data.
			for (Entry<String, List<String>> ent : urlConn.getHeaderFields().entrySet() ){
				for (String s : ent.getValue()){
					if (s.equals("gzip")) {
						useGzip = true;
						break;
					}
					if (useGzip) break;
				}
			}
			is = urlConn.getInputStream();
			if (useGzip) is = new GZIPInputStream(is);
			StringBuffer sb = new StringBuffer();
			byte b[] = new byte[1];
			boolean eof = false;
			int readCount = 0;
			while (!eof) {
				int c = is.read(b);
				if (c < 0)
					break;
				sb.append((char) b[0]);
				readCount++;
				if (readCount > 10000) {
					System.out.println("Something wrong!?");
					break;
				}
			}
			is.close();
			return sb.toString();
		} finally {
			if (TESTING) System.out.println(" [Done]");
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
}
