/**
 * Dasu Project
 *
 * File:         ActiveMeasurementEngine.java
 * RCS:          $Id$
 * Description:  ActiveMeasurementEngine class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:28:49 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.drools;

import java.io.DataInputStream;
import java.net.URL;
import java.util.logging.Logger;

import org.drools.logger.KnowledgeRuntimeLoggerFactory;

import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.download.DownloadMonitor;
import edu.northwestern.dasu.measurement.AMCoordinatorV2;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.DigitalSignatureManager;
import edu.northwestern.dasu.util.Util;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The ActiveMeasurementEngine class ...
 */
public class ActiveMeasurementEngine extends RuleBasedEngine {

	private final static Logger LOGGER = Logger.getLogger(ActiveMeasurementEngine.class.getName());
	
	public ActiveMeasurementEngine(String filename, String sigFilename, Integer taskId) throws Exception {
		super("ActiveMeasurementEngine", filename, sigFilename, taskId);
	}
	
	@Override
	public boolean loadRules() {
		
		LOGGER.info("ActiveMeasurementEngine: Verify signature of downloaded file...");
		
		boolean verified = DigitalSignatureManager.verify(filename, sigFilename);
		Integer inVer = extractTaskId(filename);
		
		if (!inVer.equals(taskId)) {
			verified = false;

			LOGGER.warning("Invalid file! TaskId differ! " + taskId + " " + inVer);
			if(Main.DEBUG) {
				LOGGER.info("ignoring because of DEBUG mode");
				verified = true;
				inVer = taskId;
			}
		}
		
		if ((verified) && ( inVer.equals(taskId))) {
			
			kbase = loadRules(filename);
			ksession = kbase.newStatefulKnowledgeSession();
			FactHalt halt= new FactHalt(false);
			ksession.setGlobal("halt", halt);
			
			if (Main.DEBUG)
				logger = KnowledgeRuntimeLoggerFactory.newFileLogger(ksession, "active");
			
			return true;
		}
		
		return false;
	}
	
	@Override
	protected void destroySession() {
		
		if (ksession != null)
			ksession.dispose();
		
		if (taskId != -1) {
			DasuManager.getPersistentDataManager().getTaskLogHistory().put(taskId, Util.currentGMTTime());
			Statistics.getInstance().commitTask(taskId);
		}

	}
	
	protected Integer extractTaskId(String drlFileName) {

		drlFileName = "file:" + drlFileName;
		Integer version = -1;

		try {
			URL url = new URL(drlFileName);
			if (url != null) {
	
				DataInputStream in = new DataInputStream(url.openStream());
				String line;
	
				while ( (line = in.readLine()) != null ) {
					
					if ( line.contains("_v_") ) {
						String[] f = line.split("_");
						try {
							version = Integer.parseInt(f[2]);
						}
						catch (Exception e) {
						}
					}
				}
				
			}
		} catch (Exception e) {
			
		}
		
		return version;
	}

	
}
