/**
 * Dasu Project
 *
 * File:         FactMeasurementDsd.java
 * RCS:          $Id$
 * Description:  FactMeasurementDsd class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 21, 2010 at 10:06:41 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.drools;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.HashSet;

import edu.northwestern.dasu.stats.DownloadStatsDynamic;
import edu.northwestern.dasu.stats.DynamicLocalPeerStat;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The FactMeasurementDsd class ...
 * For an member to be visible to Drools (as part of the fact)  
 * it must have it's own public setter and getter functions
 */
public class FactMeasurementDsd extends FactMeasurement<DownloadStatsDynamic> {

		private long timeRecorded;
		private long downloadId;
		private int numPeers;
		private float availability;
		private int completed;
		private int downloadRate;
		private int uploadRate;
		private int numKnownNonSeeds;
		private int numKnownSeeds;
		private int totalNumKnownNonSeeds=-1;
		private int totalNumKnownSeeds=-1;
		private int state;
		private int priority;
		private long downloaded;
		private long uploaded;
		private String downloadName;
		private int connectedSeeds;
		private int connectedLeechers;
		private int numPendingPeers;
		private long longDeltaTime=-1;
		private long shortDeltaTime=-1;
		private long swarmPeerSpeed=-1;

		private int numRst;  /*in the past 5 minutes (set by FactMeasurementDsd)*/ 
		private boolean justCompleted; /*true if dld completed in the past 3 minutes*/
		private float averageUploadNC=-1;
		private float averageNumPeersNC=-1;
		private float averageConnectedSeedsNC=-1;
		private float averageConnectedLeechersNC=-1;
		private float averageSwarmPeerSpeed=-1;

        private Float averageDownload=0f;
        private Float averageUpload=0f;
        private Float averageNumPeers=0f;
        private Float averageConnectedSeeds=0f;
        private Float averageConnectedLeechers=0f;

        private Float stdevDownload=0f;
        private Float stdevUpload=0f;
        private Float stdevNumPeers=0f;
        private Float stdevConnectedSeeds=0f;
        private Float stdevConnectedLeechers=0f;
        private Float stdevSwarmPeerSpeed=0f;
        
        private DownloadStatsDynamic lastMeasurement = new DownloadStatsDynamic();
        
		public FactMeasurementDsd() {
			
		}
		
		public FactMeasurementDsd(DownloadStatsDynamic dsd) {
			
			this.timeRecorded = dsd.timeRecorded;
			this.downloadId = dsd.downloadId;
			this.numPeers = dsd.numPeers;
			this.availability = dsd.availability;
			this.completed = dsd.completed;
			this.downloadRate = dsd.downloadRate;
			this.uploadRate = dsd.uploadRate;
			this.numKnownNonSeeds = dsd.numKnownNonSeeds;
			this.numKnownSeeds = dsd.numKnownSeeds;
			this.totalNumKnownNonSeeds = dsd.totalNumKnownNonSeeds;
			this.totalNumKnownSeeds = dsd.totalNumKnownSeeds; 
			this.state = dsd.state;
			this.priority = dsd.priority;
			this.downloaded = dsd.downloaded;
			this.uploaded = dsd.uploaded;
			this.justCompleted = false;
			this.numRst = 0;
			this.downloadName = dsd.downloadName;
			this.connectedSeeds = dsd.connectedSeeds;
			this.connectedLeechers = dsd.connectedLeechers;
			this.numPendingPeers = dsd.numPendingPeers;
			this.swarmPeerSpeed = dsd.swarmPeerSpeed;
			
			addEntry(dsd);
			
		}
		
		@Override
		public void addEntry(DownloadStatsDynamic ent) {

			DownloadStatsDynamic tmp = (DownloadStatsDynamic) ent.clone();
			
			int size = entryList.size();
			if (size == 0) 
				lastMeasurement = tmp;
			
			if (entryList.size() == maxNumEntries) {
				entryList.removeFirst();
				windowFull = true;
			}
			
			if (entryLog.size() == 3 * maxNumEntries)
				entryLog.removeFirst();

			ent.uploaded -= lastMeasurement.uploaded;
			ent.downloaded -= lastMeasurement.downloaded;
			
			entryLog.add(ent);
			entryList.add(ent);
			updateAll(ent);
			lastMeasurement = tmp;
			updateTimeStamp();
			
		}
		
		@Override
		public void updateAll(DownloadStatsDynamic dsd) {
			
			this.timeRecorded = (dsd.timeRecorded);
			this.downloadId = (dsd.downloadId);
			this.numPeers = (dsd.numPeers);
			this.availability = (dsd.availability);
			this.completed = (dsd.completed);
			this.downloadRate = (dsd.downloadRate);
			this.uploadRate = (dsd.uploadRate);
			this.numKnownNonSeeds = (dsd.numKnownNonSeeds);
			this.numKnownSeeds = (dsd.numKnownSeeds);
			this.totalNumKnownNonSeeds = (dsd.totalNumKnownNonSeeds);
			this.totalNumKnownSeeds = (dsd.totalNumKnownSeeds); 
			this.state = (dsd.state);
			this.priority = (dsd.priority);

			if (entryList.size() > 0) {
				longDeltaTime = (((DownloadStatsDynamic) entryList.get(entryList.size()-1)).timeRecorded - ((DownloadStatsDynamic) entryList.get(0)).timeRecorded)/1000;
				int lastIndex = entryList.size()-1;
				int fistIndex = entryList.size()>1 ? lastIndex-1 : lastIndex; 
				shortDeltaTime = (((DownloadStatsDynamic) entryList.get(lastIndex)).timeRecorded - ((DownloadStatsDynamic) entryList.get(fistIndex)).timeRecorded)/1000;
			}
			
			this.uploaded = (dsd.uploaded);
			this.downloaded = (dsd.downloaded);
			this.connectedSeeds = (dsd.connectedSeeds);
			this.connectedLeechers = (dsd.connectedLeechers);
			this.numPendingPeers = (dsd.numPendingPeers);
			this.swarmPeerSpeed = (dsd.swarmPeerSpeed);
			
			updateDynamic();
			
			if (DEBUG) {

				System.out.println("\nFactMeasurementDsd");
				for (int i = 0; i < entryList.size(); i++ ) {
					System.out.println(entryList.get(i).timeRecorded + " numPeers " +
							entryList.get(i).numPeers + " downloadRate " + 
							entryList.get(i).downloadRate + " uploadRate " + 
							entryList.get(i).uploadRate);
				}
			

				System.out.println("avgDownloadRate: " + averageDownload +
						" avgUploadRate: " + averageUpload +
						" avgUploadRateNC: " + averageUploadNC +
						" completed: " + completed + 
						" justCompleted " + justCompleted);
				
				System.out.println("avgNumPeers: " + averageNumPeers +
						" avgNumPeersNC: " + averageNumPeersNC +
						" avgConnectedSeeds: " + averageConnectedSeeds +
						" avgConnectedSeedsNC: " + averageConnectedSeedsNC +
						" avgConnectedLeechers: " + averageConnectedLeechers +
						" avgConnectedLeechersNC: " + averageConnectedLeechersNC);
				
				System.out.println();
			}
			
		}
		
		public long getTimeRecorded() {
			return timeRecorded;
		}
		
		public long getDownloadId() {
			return downloadId;
		}

		public int getNumPeers() {
			return numPeers;
		}
		
		public Float getAvailability() {
			return availability;
		}
		
		public int getCompleted() {
			return completed;
		}
		
		public int getDownloadRate() {
			return downloadRate;
		}
		
		public int getUploadRate() {
			return uploadRate;
		}
		
		public int getNumKnownNonSeeds() {
			return numKnownNonSeeds;
		}
		
		public int getNumKnownSeeds() {
			return numKnownSeeds;
		}
		
		public int getTotalNumKnownNonSeeds() {
			return totalNumKnownNonSeeds;
		}
		
		public int getTotalNumKnownSeeds() {
			return totalNumKnownSeeds;
		}
		
		public int getState() {
			return state;
		}
		
		public int getPriority() {
			return priority;
		}
		public long getDownloaded() {
			return downloaded;
		}
		
		public long getUploaded() {
			return uploaded;
		}
		
		public long getShortDeltaTime() {
			return shortDeltaTime;
		}

		public long getLongDeltaTime() {
			return longDeltaTime;
		}
		
		public Float getAverageDownload() {
			return averageDownload;
		}
        
		public Float getAverageUpload() {
			return averageUpload;
		}
        
		public Float getAverageNumPeers() {
			return averageNumPeers;
		}
		
        public Float getStdevDownload(){
        	return stdevDownload;
        }
        
        public Float getStdevUpload() {
        	return stdevUpload;
        }
        
        public Float getStdevNumPeers() {
        	return stdevNumPeers;
        }
        
        public Float getStdevConnectedSeeds() {
        	return stdevConnectedSeeds;
        }
        
        public Float getStdevConnectedLeechers() {
        	return stdevConnectedLeechers;
        }
        
        public Float getStdevSwarmPeerSpeed() {
        	return stdevSwarmPeerSpeed;
        }
        
		public boolean isJustCompleted() {
			return justCompleted;
		}

		public Float getAverageUploadNC() {
			return averageUploadNC;
		}
		
		public Float getAverageNumPeersNC() {
			return averageNumPeersNC;
		}

		public Float getAverageConnectedSeedsNC() {
			return averageConnectedSeedsNC;
		}
		
		public Float getAverageConnectedLeechersNC() {
			return averageConnectedLeechersNC;
		}
		
		public Float getAverageConnectedSeeds() {
			return averageConnectedSeeds;
		}
		
		public Float getAverageConnectedLeechers() {
			return averageConnectedLeechers;
		}
		
		public Float getAverageSwarmPeerSpeed() {
			return averageSwarmPeerSpeed;
		}
		
		public int getNumRst() {
			return numRst;
		}
		
		public int getConnectedSeeds() {
			return connectedSeeds;
		}
		
		public int getConnectedLeechers() {
			return connectedLeechers;
		}
		
		public int getNumPendingPeers() {
			return numPendingPeers;
		}
		
		public long getSwarmPeerSpeed() {
			return swarmPeerSpeed;
		}
		
		public String getDownloadName() {
			return downloadName;
		}
				

		public void setTimeRecorded(final Long timeRecorded) {
			Long temp = this.timeRecorded;
			this.timeRecorded = timeRecorded;
	        pcs.firePropertyChange("timeRecorded", new Long(temp), new Long(timeRecorded));
		}
		
		public void setDownloadId(final Long downloadId) {
			Long temp = this.downloadId;
			this.downloadId = downloadId;
	        pcs.firePropertyChange("downloadId", new Long(temp), new Long(downloadId));
		}

		public void setNumPeers(final Integer numPeers) {
			Integer temp = this.numPeers;
			this.numPeers = numPeers;
	        pcs.firePropertyChange("numPeers", new Integer(temp), new Integer(numPeers));
		}
		
		public void setAvailability(final Float availability) {
			Float temp = this.availability;
			this.availability = availability;
	        pcs.firePropertyChange("availability", new Float(temp), new Float(availability));
		}

		public void setCompleted(final Integer completed) {
			Integer temp = this.completed;
			this.completed = completed;
	        pcs.firePropertyChange("completed", new Integer(temp), new Integer(completed));
		}
				
		public void setDownloadRate(final Integer downloadRate) {
			Integer temp = this.downloadRate;
			this.downloadRate = downloadRate;
	        pcs.firePropertyChange("downloadRate", new Integer(temp), new Integer(downloadRate));			
		}
		
		public void setUploadRate(final Integer uploadRate) {
			Integer temp = this.uploadRate;
			this.uploadRate = uploadRate;
	        pcs.firePropertyChange("uploadRate", new Integer(temp), new Integer(uploadRate));			
		}
		
		public void setNumKnownNonSeeds(final Integer numKnownNonSeeds) {
			Integer temp = this.numKnownNonSeeds;
			this.numKnownNonSeeds = numKnownNonSeeds;
	        pcs.firePropertyChange("numKnownNonSeeds", new Integer(temp), new Integer(numKnownNonSeeds));			
		}
		
		public void setNumKnownSeeds(final Integer numKnownSeeds) {
			Integer temp = this.numKnownSeeds;
			this.numKnownSeeds = numKnownSeeds;
	        pcs.firePropertyChange("numKnownSeeds", new Integer(temp), new Integer(numKnownSeeds));			
		}
		
		public void setTotalNumKnownNonSeeds(final Integer totalNumKnownNonSeeds) {
			Integer temp = this.totalNumKnownNonSeeds;
			this.totalNumKnownNonSeeds = totalNumKnownNonSeeds;
	        pcs.firePropertyChange("totalNumKnownNonSeeds", new Integer(temp), new Integer(totalNumKnownNonSeeds));			
		}
		
		public void setTotalNumKnownSeeds(final Integer totalNumKnownSeeds) {
			Integer temp = this.totalNumKnownSeeds;
			this.totalNumKnownSeeds = totalNumKnownSeeds;
	        pcs.firePropertyChange("totalNumKnownSeeds", new Integer(temp), new Integer(totalNumKnownSeeds));			
		}
		
		public void setState(final Integer state) {
			Integer temp = this.state;
			this.state = state;
	        pcs.firePropertyChange("state", new Integer(temp), new Integer(state));			
		}
		
		public void setPriority(final Integer priority) {
			Integer temp = this.priority;
			this.priority = priority;
	        pcs.firePropertyChange("priority", new Integer(temp), new Integer(priority));			
		}
		
		public void setDownloaded(final Long downloaded) {
			Long temp = this.downloaded;
			this.downloaded = downloaded;
	        pcs.firePropertyChange("downloaded", new Long(temp), new Long(downloaded));			
		}
		
		public void setUploaded(final Long uploaded) {
			Long temp = this.uploaded;
			this.uploaded = uploaded;
	        pcs.firePropertyChange("uploaded", new Long(temp), new Long(uploaded));			
		}
		
		public void setAverageDownload(final float averageDownload) {
			Float temp = this.averageDownload;
			this.averageDownload = averageDownload;
	        pcs.firePropertyChange("averageDownload", new Float(temp), new Float(averageDownload));			
		}
        
		public void setAverageUpload(final float averageUpload) {
			Float temp = this.averageUpload;
			this.averageUpload = averageUpload;
	        pcs.firePropertyChange("averageUpload", new Float(temp), new Float(averageUpload));			
		}
        
		public void setAverageNumPeers(final float averageNumPeers) {
			Float temp = this.averageNumPeers;
			this.averageNumPeers = averageNumPeers;
	        pcs.firePropertyChange("averageNumPeers", new Float(temp), new Float(averageNumPeers));			
		}
		
		public void setAverageConnectedSeeds(final float averageConnectedSeeds) {
			Float temp = this.averageConnectedSeeds;
			this.averageConnectedSeeds = averageConnectedSeeds;
	        pcs.firePropertyChange("averageConnectedSeeds", new Float(temp), new Float(averageConnectedSeeds));			
		}

		public void setAverageConnectedLeechers(final float averageConnectedLeechers) {
			Float temp = this.averageConnectedLeechers;
			this.averageConnectedLeechers = averageConnectedLeechers;
	        pcs.firePropertyChange("averageConnectedLeechers", new Float(temp), new Float(averageConnectedLeechers));			
		}
		
        public void setStdevDownload(final float stdevDownload){
			Float temp = this.stdevDownload;
			this.stdevDownload = stdevDownload;
	        pcs.firePropertyChange("stdevDownload", new Float(temp), new Float(stdevDownload));			
        }
        
        public void setStdevUpload(final float stdevUpload) {
			Float temp = this.stdevUpload;
			this.stdevUpload = stdevUpload;
	        pcs.firePropertyChange("stdevUpload", new Float(temp), new Float(stdevUpload));			
        }
        
        public void setStdevNumPeers(final float stdevNumPeers) {
			Float temp = this.stdevNumPeers;
			this.stdevNumPeers = stdevNumPeers;
	        pcs.firePropertyChange("stdevNumPeers", new Float(temp), new Float(stdevNumPeers));			
        }
        
        public void setStdevConnectedSeeds(final float stdevConnectedSeeds) {
			Float temp = this.stdevConnectedSeeds;
			this.stdevConnectedSeeds = stdevConnectedSeeds;
	        pcs.firePropertyChange("stdevConnectedSeeds", new Float(temp), new Float(stdevConnectedSeeds));			
        }
        
        public void setStdevConnectedLeechers(final float stdevConnectedLeechers) {
			Float temp = this.stdevConnectedLeechers;
			this.stdevConnectedLeechers = stdevConnectedLeechers;
	        pcs.firePropertyChange("stdevConnectedLeechers", new Float(temp), new Float(stdevConnectedLeechers));			
        }

        public void setStdevSwarmPeerSpeed(final float stdevSwarmPeerSpeed) {
			Float temp = this.stdevSwarmPeerSpeed;
			this.stdevSwarmPeerSpeed = stdevSwarmPeerSpeed;
	        pcs.firePropertyChange("stdevSwarmPeerSpeed", new Float(temp), new Float(stdevSwarmPeerSpeed));			
        }
        
		public void setJustCompleted(boolean justCompleted) {
			boolean temp = this.justCompleted;
			this.justCompleted = justCompleted;
	        pcs.firePropertyChange("justCompleted", new Boolean(temp), new Boolean(justCompleted));	
		}

		public void setAverageUploadNC(final float averageUploadNC) {
			Float temp = this.averageUploadNC;
			this.averageUploadNC = averageUploadNC;
	        pcs.firePropertyChange("averageUploadNC", new Float(temp), new Float(averageUploadNC));			
		}
		
		public void setAverageNumPeersNC(final float averageNumPeersNC) {
			Float temp = this.averageNumPeersNC;
			this.averageNumPeersNC = averageNumPeersNC;
	        pcs.firePropertyChange("averageNumPeersNC", new Float(temp), new Float(averageNumPeersNC));			
		}	

		public void setAverageConnectedSeedsNC(final float averageConnectedSeedsNC) {
			Float temp = this.averageConnectedSeedsNC;
			this.averageConnectedSeedsNC = averageConnectedSeedsNC;
	        pcs.firePropertyChange("averageConnectedSeedsNC", new Float(temp), new Float(averageConnectedSeedsNC));			
		}	

		public void setAverageConnectedLeechersNC(final float averageConnectedLeechersNC) {
			Float temp = this.averageConnectedLeechersNC;
			this.averageConnectedLeechersNC = averageConnectedLeechersNC;
	        pcs.firePropertyChange("averageConnectedLeechersNC", new Float(temp), new Float(averageConnectedLeechersNC));			
		}	


		public void setNumRst(final Integer numRst) {
			Integer temp = this.numRst;
			this.numRst = numRst;
	        pcs.firePropertyChange("numRst", new Integer(temp), new Integer(numRst));			
		}
	
		public void setConnectedSeeds(final Integer connectedSeeds) {
			Integer temp = this.connectedSeeds;
			this.connectedSeeds = connectedSeeds;
	        pcs.firePropertyChange("connectedSeeds", new Integer(temp), new Integer(connectedSeeds));			
		}

		public void setNumPendingPeers(final Integer numPendingPeers) {
			Integer temp = this.numPendingPeers;
			this.numPendingPeers = numPendingPeers;
	        pcs.firePropertyChange("numPendingPeers", new Integer(temp), new Integer(numPendingPeers));			
		}

		public void setSwarmPeerSpeed(final Long swarmPeerSpeed) {
			Long temp = this.swarmPeerSpeed;
			this.swarmPeerSpeed = swarmPeerSpeed;
	        pcs.firePropertyChange("swarmPeerSpeed", new Long(temp), new Long(swarmPeerSpeed));			
		}
		
		public void setConnectedLeechers(final Integer connectedLeechers) {
			Integer temp = this.connectedLeechers;
			this.connectedLeechers = connectedLeechers;
	        pcs.firePropertyChange("connectedLeechers", new Integer(temp), new Integer(connectedLeechers));			
		}
		
        public void updateDynamic() {
        	
        	int size = entryList.size();
        	int logSize = entryLog.size();
        	
        	if (this.justCompleted == true) {
        		for (int i=logSize-1; i > 0; i--) {
        			if ( entryLog.get(i).completed < 1000
        					&& (logSize-i+1) * interval / 60 <= 5 ) {
        				break;
        			}
        			
        			if ((logSize-i+1) * interval / 60 > 5 )
        				this.justCompleted = false;
        				//setJustCompleted(false);
        		}
        	}
        		
        	if (this.completed == 1000 && logSize > 1 &&   
        			entryLog.get(logSize-2).completed < 1000) {
        		this.justCompleted=true;
        		this.averageUploadNC = getAverageUpload();
        		this.averageNumPeersNC = getAverageNumPeers();
        		this.averageConnectedSeedsNC = getAverageConnectedSeeds();
        		this.averageConnectedLeechersNC = getAverageConnectedLeechers();
        		this.averageSwarmPeerSpeed = getAverageSwarmPeerSpeed();
        		
        		//setJustCompleted(true);
        		//setAverageUploadNC(getAverageUpload());

        	}
        		
        	int sumDownload = 0;
        	int sumUpload = 0;
        	int sumNumPeers = 0;
        	int sumConnectedSeeds = 0;
        	int sumConnectedLeechers = 0;
        	int sumSwarmPeerSpeed = 0;
        	
        	for (int i=0; i < size; i++) {
        		
        		DownloadStatsDynamic dsd = (DownloadStatsDynamic) entryList.get(i);
        		sumDownload += dsd.downloadRate;
        		sumUpload += dsd.uploadRate;
        		sumNumPeers += dsd.numPeers;
        		sumConnectedSeeds += dsd.connectedSeeds;
        		sumConnectedLeechers += dsd.connectedLeechers;
        		sumSwarmPeerSpeed += dsd.swarmPeerSpeed;
        	}
        
        	float averageDownload = ((float) sumDownload / (float) size);
        	float averageUpload = ((float) sumUpload / (float) size);
        	float averageNumPeers = ((float) sumNumPeers / (float) size);
        	float averageConnectedSeeds = ((float) sumConnectedSeeds / (float) size);
        	float averageConnectedLeechers = ((float) sumConnectedLeechers / (float) size);
        	float averageSwarmPeerSpeed = ((float) sumSwarmPeerSpeed / (float) size);
        	
            float sdqDevSumUpload = 0;
            float sdqDevSumDownload = 0;
            float sdqDevSumNumPeers = 0;
            float sdqDevSumConnectedSeeds = 0;
            float sdqDevSumConnectedLeechers = 0;
            float sdqDevSumSwarmPeerSpeed = 0;
        	
        	for (int i=0; i < size; i++) {
        		
        		DownloadStatsDynamic dsd = (DownloadStatsDynamic) entryList.get(i);
        		sdqDevSumUpload += Math.pow( (averageUpload - dsd.uploadRate), 2);
        		sdqDevSumDownload += Math.pow( (averageDownload - dsd.downloadRate), 2);
        		sdqDevSumNumPeers += Math.pow( (averageNumPeers - dsd.numPeers), 2);
        		sdqDevSumConnectedSeeds += Math.pow( (averageConnectedSeeds - dsd.connectedSeeds), 2);
        		sdqDevSumConnectedLeechers += Math.pow( (averageConnectedLeechers - dsd.connectedLeechers), 2);
        		sdqDevSumSwarmPeerSpeed += Math.pow( (averageSwarmPeerSpeed - dsd.swarmPeerSpeed), 2);

        	}        	
        	
//          setStdevUpload( (float) Math.sqrt(sdqDevSumUpload / size ));
//          setStdevDownload( (float) Math.sqrt(sdqDevSumDownload / size) );
//          setStdevNumPeers( (float) Math.sqrt(sdqDevSumNumPeers / size) );
//        	setAverageDownload(averageDownload);
//        	setAverageUpload(averageUpload);
//        	setAverageNumPeers(averageNumPeers);
        	
            this.stdevUpload = ( (float) Math.sqrt(sdqDevSumUpload / size ));
            this.stdevDownload = ( (float) Math.sqrt(sdqDevSumDownload / size) );
            this.stdevNumPeers = ( (float) Math.sqrt(sdqDevSumNumPeers / size) );
            this.stdevConnectedSeeds = ( (float) Math.sqrt(sdqDevSumConnectedSeeds / size) );
            this.stdevConnectedLeechers = ( (float) Math.sqrt(sdqDevSumConnectedLeechers / size) );
            this.stdevSwarmPeerSpeed = ( (float) Math.sqrt(sdqDevSumSwarmPeerSpeed / size) );
            
        	this.averageDownload = averageDownload;
        	this.averageUpload = averageUpload;
        	this.averageNumPeers = averageNumPeers;
        	this.averageConnectedSeeds = averageConnectedSeeds;
        	this.averageConnectedLeechers = averageConnectedLeechers;
        	this.averageSwarmPeerSpeed = averageSwarmPeerSpeed;

//        	setNumRst(entryLog.get(entryLog.size()-1).numRst);
        	this.numRst = (entryLog.get(entryLog.size()-1).numRst);
        }
        
		private void updateDynamic2() throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException {
			
        	int logSize = entryLog.size();
        	
        	if (this.justCompleted == true) {
        		for (int i=logSize-1; i > 0; i--) {
        			if ( entryLog.get(i).completed < 1000
        					&& (logSize-i+1) * interval / 60 <= 5 ) {
        				break;
        			}
        			
        			if ((logSize-i+1) * interval / 60 > 5 )
        				this.justCompleted = false;
        				//setJustCompleted(false);
        		}
        	}
        		
        	if (this.completed == 1000 && logSize > 1 &&   
        			entryLog.get(logSize-2).completed < 1000) {
        		this.justCompleted=true;
        		this.averageUploadNC = getAverageUpload();
        		this.averageNumPeersNC = getAverageNumPeers();
        		this.averageConnectedSeedsNC = getAverageConnectedSeeds();
        		this.averageConnectedLeechersNC = getAverageConnectedLeechers();
        		this.averageSwarmPeerSpeed = getAverageSwarmPeerSpeed();
        		
        	}
			
			HashSet<String> slidingWindowVars = new HashSet<String>();
			HashMap<String, Long> sumVars = new HashMap<String, Long>();
			HashMap<String, Float> averageVars = new HashMap<String, Float>();
			HashMap<String, Float> sdqDevVars = new HashMap<String, Float>();
			
			slidingWindowVars.add("downloadRate");
			slidingWindowVars.add("uploadRate");
			slidingWindowVars.add("numPeers");
			slidingWindowVars.add("connectedSeeds");
			slidingWindowVars.add("connectedLeechers");
			slidingWindowVars.add("swarmPeerSpeed");
			
        	int sz = entryList.size();
        	int startIndex = 0;
        	
        	for (String varName : slidingWindowVars) {
        		sumVars.put(varName, 0l);
        		averageVars.put(varName, 0f);
        		sdqDevVars.put(varName, 0f);
        	}
        	
        	for (int i=startIndex; i < sz; i++) {
        		
        		DownloadStatsDynamic dsd = (DownloadStatsDynamic) entryList.get(i);
        		
            	for (String varName : slidingWindowVars) {
            		Field field = dsd.getClass().getField(varName);
            		sumVars.put(varName, sumVars.get(varName) + field.getLong(dsd));
            	}
        	}
        	
        	for (String varName : slidingWindowVars)
        		averageVars.put(varName, (float) sumVars.get(varName) / (float) sz);


        	for (int i=startIndex; i < sz; i++) {
        		
        		DownloadStatsDynamic dsd = (DownloadStatsDynamic) entryList.get(i);
        		
            	for (String varName : slidingWindowVars) {
            		Field field = dsd.getClass().getField(varName);
            		sumVars.put(varName, sumVars.get(varName) + field.getLong(dsd));
            		sdqDevVars.put(varName, (float) (sdqDevVars.get(varName) 
            				+ Math.pow( (averageVars.get(varName) - field.getLong(dsd)), 2)));
            	}
        	}
        	
        	this.stdevUpload = ( (float) Math.sqrt(sdqDevVars.get("uploadRate") / sz ));
            this.stdevDownload = ( (float) Math.sqrt(sdqDevVars.get("downloadRate") / sz) );
            this.stdevNumPeers = ( (float) Math.sqrt(sdqDevVars.get("numPeers") / sz) );
            this.stdevConnectedSeeds = ( (float) Math.sqrt(sdqDevVars.get("connectedSeeds") / sz) );
            this.stdevConnectedLeechers = ( (float) Math.sqrt(sdqDevVars.get("connectedLeechers") / sz) );
            this.stdevSwarmPeerSpeed = ( (float) Math.sqrt(sdqDevVars.get("swarmPeerSpeed") / sz) );
            
            this.averageUpload = averageVars.get("uploadRate");
            this.averageDownload = averageVars.get("downloadRate");
            this.averageNumPeers = averageVars.get("numPeers");
            this.averageConnectedSeeds = averageVars.get("connectedSeeds");
            this.averageConnectedLeechers = averageVars.get("connectedLeechers");
            this.averageSwarmPeerSpeed = averageVars.get("swarmPeerSpeed");
            
        	this.numRst = (entryLog.get(entryLog.size()-1).numRst);
            
		}
        
}
