/**
 * Dasu Project
 *
 * File:         FactMeasurementLpsd.java
 * RCS:          $Id$
 * Description:  FactMeasurementLpsd class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jul 21, 2010 at 10:08:18 AM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.drools;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.HashSet;

import org.gudy.azureus2.core3.config.COConfigurationManager;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.stats.DynamicLocalPeerStat;
import edu.northwestern.dasu.stats.PeerStatistics;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The FactMeasurementLpsd class ...
 * For a member to be visible to Drools (as part of the fact)  
 * it must have it's own public setter and getter functions
 */
public class FactMeasurementLpsd extends FactMeasurement<DynamicLocalPeerStat> {

		private long timeRecorded=-1;
		private int uploadRate=-1; 		/*instantaneous uploadRate*/
		private int downloadRate=-1;	/*instantaneous downloadRate*/
		private int maxUploadRate=-1;
		private int maxDownloadRate=-1;
		private boolean autoSpeed=false;
		private int numTorrents=0;
		private int numLeechingTorrents=0;
		private int numSeedingTorrents=0;
		public int numConnectedSeeds=0;
		public int numConnectedLeechers=0;
		private long bytesSent=-1;		/*since last read*/
		private long bytesReceived=-1;	/* since last read*/
		private int longDeltaTime=-1;
		private int shortDeltaTime=-1;
		private long swarmPeerSpeed=-1; 
		
		public int maxPeerConnectionsGlobal;
		public int maxPeerConnectionsPerTorrent;
		public int maxPeerConnectionsPerTorrentSeeding = -1; /* -1 means not enabled, so value should be ignored */
		public int maxUploadRateSeeding = -1; /* -1 means not enabled, so value should be ignored */
		public int maxSeedsPerTorrent;
		public boolean lazyBitfield;
		public boolean requireEncryption;
		public String encryptionMinLevel;
		public boolean allowNonEncryptedOut;
		public boolean allowNonEncryptedIn;
		public boolean useCryptoPortExtension;
		private int tcpPortIn;
		private int udpPortIn;
		
		private int maxUploadRateLimitAllActiveTorrents;
		private int maxDownloadRateLimitAllActiveTorrents;

		private Float averageUpload=0f;
        private Float averageDownload=0f;
		private Float averageNumConnectedSeeds=0f;
		private Float averageNumConnectedLeechers=0f;
		private Float averageBytesSent=0f;
		private Float averageBytesReceived=0f;
		private Float averageSwarmPeerSpeed=0f;
        
		private Float stdevUpload=0f;
        private Float stdevDownload=0f;
        private Float stdevNumConnectedSeeds=0f;
        private Float stdevNumConnectedLeechers=0f;
        private Float stdevBytesSent=0f;
        private Float stdevBytesReceived=0f;
        private Float stdevSwarmPeerSpeed=0f;
		
        private DynamicLocalPeerStat lastMeasurement = new DynamicLocalPeerStat();
        
		public FactMeasurementLpsd() {
			
		}
		
		public FactMeasurementLpsd(DynamicLocalPeerStat lpsd) {

			this.timeRecorded = lpsd.currentTime;
			this.uploadRate = lpsd.uploadRate;
			this.downloadRate = lpsd.downloadRate;
			this.maxUploadRate = lpsd.maxUploadRate; /*bytes per second*/
			this.maxDownloadRate = lpsd.maxDownloadRate; /*bytes per second*/
			this.autoSpeed = lpsd.autoSpeed;
			this.numTorrents = lpsd.numTorrents;
			this.numLeechingTorrents = lpsd.numLeechingTorrents;
			this.numSeedingTorrents = lpsd.numSeedingTorrents;
			this.numConnectedSeeds = lpsd.numConnectedSeeds;
			this.numConnectedLeechers = lpsd.numConnectedLeechers;
			this.bytesSent = lpsd.bytesSentLpsd;
			this.bytesReceived = lpsd.bytesReceivedLpsd;
			this.swarmPeerSpeed = lpsd.swarmPeerSpeed;
			this.maxUploadRateLimitAllActiveTorrents = lpsd.maxUploadRateLimitAllActiveTorrents;
			this.maxDownloadRateLimitAllActiveTorrents = lpsd.maxDownloadRateLimitAllActiveTorrents;
			
			PeerStatistics.getInstance();
			this.maxPeerConnectionsGlobal = PeerStatistics.getLocalPeerStatStatic().maxPeerConnections;
			this.maxPeerConnectionsPerTorrent = PeerStatistics.getLocalPeerStatStatic().maxPeerConnectionsPerTorrent;
			this.maxPeerConnectionsPerTorrentSeeding = PeerStatistics.getLocalPeerStatStatic().maxPeerConnectionsPerTorrentSeeding;
			this.maxUploadRateSeeding = PeerStatistics.getLocalPeerStatStatic().maxUploadRateSeeding;
			this.maxSeedsPerTorrent = PeerStatistics.getLocalPeerStatStatic().maxSeedsPerTorrent;
			this.lazyBitfield = PeerStatistics.getLocalPeerStatStatic().lazyBitfield;
			this.requireEncryption = PeerStatistics.getLocalPeerStatStatic().requireEncryption; 
			this.encryptionMinLevel = PeerStatistics.getLocalPeerStatStatic().encryptionMinLevel;
			this.allowNonEncryptedOut = PeerStatistics.getLocalPeerStatStatic().allowNonEncryptedOut;
			this.allowNonEncryptedIn = PeerStatistics.getLocalPeerStatStatic().allowNonEncryptedIn;
			this.useCryptoPortExtension = PeerStatistics.getLocalPeerStatStatic().useCryptoPortExtension;
			this.tcpPortIn = COConfigurationManager.getIntParameter("TCP.Listen.Port");
			this.udpPortIn = COConfigurationManager.getIntParameter("UDP.Listen.Port");

			addEntry(lpsd);

		}
		
		
		@Override
		public void addEntry(DynamicLocalPeerStat ent) {

			DynamicLocalPeerStat tmp = (DynamicLocalPeerStat) ent.clone();
			
			int size = entryList.size();
			if (size == 0) 
				lastMeasurement = tmp;
			
			if (entryList.size() == maxNumEntries) {
				entryList.removeFirst();
				windowFull = true;
			}
			
			if (entryLog.size() == 3 * maxNumEntries)
				entryLog.removeFirst();

			ent.bytesSentLpsd -= lastMeasurement.bytesSentLpsd;
			ent.bytesReceivedLpsd -= lastMeasurement.bytesReceivedLpsd;
			
			entryLog.add(ent);
			entryList.add(ent);
			updateAll(ent);
			lastMeasurement = tmp;
			updateTimeStamp();
			
		}

				
		@Override
		public void updateAll(DynamicLocalPeerStat lpsd) {
			
			if (!Main.isShuttingDown()) {
				timeRecorded = (lpsd.currentTime);
				uploadRate = (lpsd.uploadRate);
				downloadRate = (lpsd.downloadRate);
				maxUploadRate = (lpsd.maxUploadRate);
				maxDownloadRate = (lpsd.maxDownloadRate);
				autoSpeed = (lpsd.autoSpeed);
				numTorrents = (lpsd.numTorrents);
				numLeechingTorrents = (lpsd.numLeechingTorrents);
				numSeedingTorrents = (lpsd.numSeedingTorrents);
				numConnectedSeeds = (lpsd.numConnectedSeeds);
				numConnectedLeechers = (lpsd.numConnectedLeechers);
				maxUploadRateLimitAllActiveTorrents = (lpsd.maxUploadRateLimitAllActiveTorrents);
				maxDownloadRateLimitAllActiveTorrents = (lpsd.maxDownloadRateLimitAllActiveTorrents);
				
				if (entryList.size() > 0) {
					Long ld = (((DynamicLocalPeerStat) entryList.get(entryList.size()-1)).currentTime - ((DynamicLocalPeerStat) entryList.get(0)).currentTime)/1000;
					longDeltaTime = ld.intValue();
					int lastIndex = entryList.size()-1;
					int fistIndex = entryList.size()>1 ? lastIndex-1 : lastIndex; 
					Long sd = (((DynamicLocalPeerStat) entryList.get(lastIndex)).currentTime - ((DynamicLocalPeerStat) entryList.get(fistIndex)).currentTime)/1000;
					shortDeltaTime = sd.intValue();
				}
				bytesSent = (lpsd.bytesSentLpsd);
				bytesReceived = (lpsd.bytesReceivedLpsd);
				swarmPeerSpeed  = (lpsd.swarmPeerSpeed);
				
				if (PeerStatistics.getInstance() != null) {
				
					maxPeerConnectionsGlobal = PeerStatistics.getLocalPeerStatStatic().maxPeerConnections;
					maxPeerConnectionsPerTorrent = PeerStatistics.getLocalPeerStatStatic().maxPeerConnectionsPerTorrent;
					maxPeerConnectionsPerTorrentSeeding = PeerStatistics.getLocalPeerStatStatic().maxPeerConnectionsPerTorrentSeeding;
					maxUploadRateSeeding = PeerStatistics.getLocalPeerStatStatic().maxUploadRateSeeding;
					maxSeedsPerTorrent = PeerStatistics.getLocalPeerStatStatic().maxSeedsPerTorrent;
					lazyBitfield = PeerStatistics.getLocalPeerStatStatic().lazyBitfield;
					requireEncryption = PeerStatistics.getLocalPeerStatStatic().requireEncryption; 
					encryptionMinLevel = PeerStatistics.getLocalPeerStatStatic().encryptionMinLevel;
					allowNonEncryptedOut = PeerStatistics.getLocalPeerStatStatic().allowNonEncryptedOut;
					allowNonEncryptedIn = PeerStatistics.getLocalPeerStatStatic().allowNonEncryptedIn;
					useCryptoPortExtension = PeerStatistics.getLocalPeerStatStatic().useCryptoPortExtension;
				}
			}

			try {
				updateDynamic2();
			} catch (Exception e) {
				e.printStackTrace();
			} 
			
			if (DEBUG) {

				System.out.println("\nFactMeasurementLpsd");
				for (int i = 0; i < entryList.size(); i++ ) {
					System.out.println(entryList.get(i).currentTime + " downloadRate " +
							entryList.get(i).downloadRate + " uploadRate: " +
							entryList.get(i).uploadRate + " maxUploadRate: " +
							entryList.get(i).maxUploadRate + " maxDownloadRate: " +
							entryList.get(i).maxDownloadRate + " numTorrents: " +
							entryList.get(i).numTorrents + " numLeechingTorrents: " +
							entryList.get(i).numLeechingTorrents + " numSeedingTorrents: " +
							entryList.get(i).numSeedingTorrents + " numConnectedSeeds: " +
							entryList.get(i).numConnectedSeeds + " numConnectedLeechers: " +
							entryList.get(i).numConnectedLeechers + " bytesSent: " +
							entryList.get(i).bytesSentLpsd + " bytesReceived: " + 
							entryList.get(i).bytesReceivedLpsd); 
				}
						
				System.out.println("avgDownloadRate: " + averageDownload +
						" avgUploadRate: " + averageUpload);
			
				System.out.println();
			}
			
		}

		public long getTimeRecorded() {
			return timeRecorded;
		}

		public int getUploadRate() {
			return uploadRate;
		}

		public int getDownloadRate() {
			return downloadRate;
		}

		public int getMaxUploadRate() {
			return maxUploadRate;
		}

		public int getMaxDownloadRate() {
			return maxDownloadRate;
		}

		public boolean isAutoSpeed() {
			return autoSpeed;
		}

		public int getNumTorrents() {
			return numTorrents;
		}
		
		public int getNumLeechingTorrents() {
			return numLeechingTorrents;
		}
		
		public int getNumSeedingTorrents() {
			return numSeedingTorrents;
		}
		
		public int getNumConnectedSeeds() {
			return numConnectedSeeds;
		}
		
		public int getNumConnectedLeechers() {
			return numConnectedLeechers;
		}
		
		public int getShortDeltaTime() {
			return shortDeltaTime;
		}

		public int getLongDeltaTime() {
			return longDeltaTime;
		}

		public long getBytesSent() {
			return bytesSent;
		}

		public long getBytesReceived() {
			return bytesReceived;
		}
		
		public long getSwarmPeerSpeed() {
			return swarmPeerSpeed;
		}
		
		public int getMaxPeerConnectionsGlobal() {
			return maxPeerConnectionsGlobal;
		}

		public int getMaxUploadRateLimitAllActiveTorrents() {
			return maxUploadRateLimitAllActiveTorrents;
		}
		
		public int getMaxDownloadRateLimitAllActiveTorrents() {
			return maxDownloadRateLimitAllActiveTorrents;
		}

		public int getMaxPeerConnectionsPerTorrent() {
			return maxPeerConnectionsPerTorrent;
		}

		public int getMaxPeerConnectionsPerTorrentSeeding() {
			return maxPeerConnectionsPerTorrentSeeding;
		}

		public int getMaxUploadRateSeeding() {
			return maxUploadRateSeeding;
		}

		public int getMaxSeedsPerTorrent() {
			return maxSeedsPerTorrent;
		}

		public boolean isLazyBitfield() {
			return lazyBitfield;
		}

		public boolean isRequireEncryption() {
			return requireEncryption;
		}

		public String getEncryptionMinLevel() {
			return encryptionMinLevel;
		}

		public boolean isAllowNonEncryptedOut() {
			return allowNonEncryptedOut;
		}

		public boolean isAllowNonEncryptedIn() {
			return allowNonEncryptedIn;
		}

		public boolean isUseCryptoPortExtension() {
			return useCryptoPortExtension;
		}
		
		public int getTcpPortIn() {
			return tcpPortIn;
		}
		
		public int getUdpPortIn() {
			return udpPortIn;
		}

		public Float getAverageUpload() {
			return averageUpload;
		}

		public Float getAverageDownload() {
			return averageDownload;
		}

		public Float getAverageNumConnectedSeeds() {
			return averageNumConnectedSeeds;
		}
		
		public Float getAverageNumConnectedLeechers() {
			return averageNumConnectedLeechers;
		}
		
		public Float getAverageBytesSent() {
			return averageBytesSent;
		}

		public Float getAverageBytesReceived() {
			return averageBytesReceived;
		}

		public Float getAverageSwarmPeerSpeed() {
			return averageSwarmPeerSpeed;
		}
		
		public Float getStdevUpload() {
			return stdevUpload;
		}

		public Float getStdevDownload() {
			return stdevDownload;
		}

		public Float getStdevNumConnectedSeeds() {
			return stdevNumConnectedSeeds;
		}

		public Float getStdevNumConnectedLeechers() {
			return stdevNumConnectedLeechers;
		}
		
		public Float getStdevBytesSent() {
			return stdevBytesSent;
		}
		
		public Float getStdevBytesReceived() {
			return stdevBytesReceived;
		}

		public Float getStdevSwarmPeerSpeed() {
			return stdevSwarmPeerSpeed;
		}
		
		public void setTimeRecorded(long timeRecorded) {
			Long temp = this.timeRecorded;
			this.timeRecorded = timeRecorded;
	        pcs.firePropertyChange("timeRecorded", new Long(temp), new Long(timeRecorded));
		}

		public void setUploadRate(int uploadRate) {
			Integer temp = this.uploadRate;
			this.uploadRate = uploadRate;
	        pcs.firePropertyChange("uploadRate", new Integer(temp), new Integer(uploadRate));

		}

		public void setDownloadRate(int downloadRate) {
			Integer temp = this.downloadRate;
			this.downloadRate = downloadRate;
	        pcs.firePropertyChange("downloadRate", new Integer(temp), new Integer(downloadRate));

		}

		public void setMaxUploadRate(int maxUploadRate) {
			Integer temp = this.maxUploadRate;
			this.maxUploadRate = maxUploadRate;
	        pcs.firePropertyChange("maxUploadRate", new Integer(temp), new Integer(maxUploadRate));

		}

		public void setMaxDownloadRate(int maxDownloadRate) {
			Integer temp = this.maxDownloadRate;
			this.maxDownloadRate = maxDownloadRate;
	        pcs.firePropertyChange("maxDownloadRate", new Integer(temp), new Integer(maxDownloadRate));
		}

		public void setAutoSpeed(boolean autoSpeed) {
			boolean temp = this.autoSpeed;
			this.autoSpeed = autoSpeed;
	        pcs.firePropertyChange("autoSpeed", new Boolean(temp), new Boolean(autoSpeed));
		}

		public void setNumTorrents(int numTorrents) {
			Integer temp = this.numTorrents;
			this.numTorrents = numTorrents;
	        pcs.firePropertyChange("numTorrents", new Integer(temp), new Integer(numTorrents));
		}
		

		public void setNumLeechingTorrents(int numLeechingTorrents) {
			Integer temp = this.numLeechingTorrents;
			this.numLeechingTorrents = numLeechingTorrents;
	        pcs.firePropertyChange("numLeechingTorrents", new Integer(temp), new Integer(numLeechingTorrents));
		}


		public void setNumSeedingTorrents(int numSeedingTorrents) {
			Integer temp = this.numSeedingTorrents;
			this.numSeedingTorrents = numSeedingTorrents;
	        pcs.firePropertyChange("numSeedingTorrents", new Integer(temp), new Integer(numSeedingTorrents));
		}
		
		public void setNumConnectedSeeds(int numConnectedSeeds) {
			Integer temp = this.numConnectedSeeds;
			this.numConnectedSeeds = numConnectedSeeds;
	        pcs.firePropertyChange("numConnectedSeeds", new Integer(temp), new Integer(numConnectedSeeds));
		}
		
		public void setNumConnectedLeechers(int numConnectedLeechers) {
			Integer temp = this.numConnectedLeechers;
			this.numConnectedLeechers = numConnectedLeechers;
	        pcs.firePropertyChange("numConnectedLeechers", new Integer(temp), new Integer(numConnectedLeechers));
		}
		
		public void setBytesSent(long bytesSent) {
			Long temp = this.bytesSent;
			this.bytesSent = bytesSent;
	        pcs.firePropertyChange("bytesSent", new Long(temp), new Long(bytesSent));
		}

		public void setBytesReceived(long bytesReceived) {
			Long temp = this.bytesReceived;
			this.bytesReceived = bytesReceived;
	        pcs.firePropertyChange("bytesReceived", new Long(temp), new Long(bytesReceived));
		}
		
		public void setSwarmPeerSpeed(long swarmPeerSpeed) {
			Long temp = this.swarmPeerSpeed;
			this.swarmPeerSpeed = swarmPeerSpeed;
	        pcs.firePropertyChange("swarmPeerSpeed", new Long(temp), new Long(swarmPeerSpeed));
		}
		
		public void setAverageUpload(float averageUpload) {
			Float temp = this.averageUpload;
			this.averageUpload = averageUpload;
	        pcs.firePropertyChange("averageUpload", new Float(temp), new Float(averageUpload));
		}

		public void setAverageDownload(float averageDownload) {
			Float temp = this.averageDownload;
			this.averageDownload = averageDownload;
	        pcs.firePropertyChange("averageDownload", new Float(temp), new Float(averageDownload));
		}

		public void setAverageNumConnectedSeeds(float averageNumConnectedSeeds) {
			Float temp = this.averageNumConnectedSeeds;
			this.averageNumConnectedSeeds = averageNumConnectedSeeds;
	        pcs.firePropertyChange("averageNumConnectedSeeds", new Float(temp), new Float(averageNumConnectedSeeds));
		}

		public void setAverageNumConnectedLeechers(float averageNumConnectedLeechers) {
			Float temp = this.averageNumConnectedLeechers;
			this.averageNumConnectedLeechers = averageNumConnectedLeechers;
	        pcs.firePropertyChange("averageNumConnectedLeechers", new Float(temp), new Float(averageNumConnectedLeechers));
		}

		public void setAverageBytesSent(float averageBytesSent) {
			Float temp = this.averageBytesSent;
			this.averageBytesSent = averageBytesSent;
	        pcs.firePropertyChange("averageBytesSent", new Float(temp), new Float(averageBytesSent));
		}

		public void setAverageBytesReceived(float averageBytesReceived) {
			Float temp = this.averageBytesReceived;
			this.averageBytesReceived = averageBytesReceived;
	        pcs.firePropertyChange("averageBytesReceived", new Float(temp), new Float(averageBytesReceived));
		}
		
		public void setAverageSwarmPeerSpeed(float averageSwarmPeerSpeed) {
			Float temp = this.averageSwarmPeerSpeed;
			this.averageSwarmPeerSpeed = averageSwarmPeerSpeed;
	        pcs.firePropertyChange("averageSwarmPeerSpeed", new Float(temp), new Float(averageSwarmPeerSpeed));
		}
		
		public void setStdevUpload(float stdevUpload) {
			Float temp = this.stdevUpload;
			this.stdevUpload = stdevUpload;
	        pcs.firePropertyChange("stdevUpload", new Float(temp), new Float(stdevUpload));
		}

		public void setStdevDownload(float stdevDownload) {
			Float temp = this.stdevDownload;
			this.stdevDownload = stdevDownload;
	        pcs.firePropertyChange("stdevDownload", new Float(temp), new Float(stdevDownload));
		}

		public void setStdevNumConnectedSeeds(float stdevNumConnectedSeeds) {
			Float temp = this.stdevNumConnectedSeeds;
			this.stdevNumConnectedSeeds = stdevNumConnectedSeeds;
	        pcs.firePropertyChange("stdevNumConnectedSeeds", new Float(temp), new Float(stdevNumConnectedSeeds));
		}

		public void setStdevNumConnectedLeechers(float stdevNumConnectedLeechers) {
			Float temp = this.stdevNumConnectedLeechers;
			this.stdevNumConnectedLeechers = stdevNumConnectedLeechers;
	        pcs.firePropertyChange("stdevNumConnectedLeechers", new Float(temp), new Float(stdevNumConnectedLeechers));
		}

		public void setStdevBytesSent(float stdevBytesSent) {
			Float temp = this.stdevBytesSent;
			this.stdevBytesSent = stdevBytesSent;
	        pcs.firePropertyChange("stdevBytesSent", new Float(temp), new Float(stdevBytesSent));
		}

		public void setStdevBytesReceived(float stdevBytesReceived) {
			Float temp = this.stdevBytesReceived;
			this.stdevBytesReceived = stdevBytesReceived;
	        pcs.firePropertyChange("stdevBytesReceived", new Float(temp), new Float(stdevBytesReceived));
		}
		
		public void setStdevSwarmPeerSpeed(float stdevSwarmPeerSpeed) {
			Float temp = this.stdevSwarmPeerSpeed;
			this.stdevSwarmPeerSpeed = stdevSwarmPeerSpeed;
	        pcs.firePropertyChange("stdevSwarmPeerSpeed", new Float(temp), new Float(stdevSwarmPeerSpeed));
		}

		public void setMaxPeerConnectionsGlobal(int maxPeerConnectionsGlobal) {
			this.maxPeerConnectionsGlobal = maxPeerConnectionsGlobal;
		}
		
		public void setMaxUploadRateLimitAllActiveTorrents(int maxUploadRateLimitAllActiveTorrents) {
			this.maxUploadRateLimitAllActiveTorrents = maxUploadRateLimitAllActiveTorrents;
		}

		public void setMaxDownloadRateLimitAllActiveTorrents(int maxDownloadRateLimitAllActiveTorrents) {
			this.maxDownloadRateLimitAllActiveTorrents = maxDownloadRateLimitAllActiveTorrents;
		}

		public void setMaxPeerConnectionsPerTorrent(int maxPeerConnectionsPerTorrent) {
			this.maxPeerConnectionsPerTorrent = maxPeerConnectionsPerTorrent;
		}

		public void setMaxPeerConnectionsPerTorrentSeeding(
				int maxPeerConnectionsPerTorrentSeeding) {
			this.maxPeerConnectionsPerTorrentSeeding = maxPeerConnectionsPerTorrentSeeding;
		}

		public void setMaxUploadRateSeeding(int maxUploadRateSeeding) {
			this.maxUploadRateSeeding = maxUploadRateSeeding;
		}

		public void setMaxSeedsPerTorrent(int maxSeedsPerTorrent) {
			this.maxSeedsPerTorrent = maxSeedsPerTorrent;
		}

		public void setLazyBitfield(boolean lazyBitfield) {
			this.lazyBitfield = lazyBitfield;
		}

		public void setRequireEncryption(boolean requireEncryption) {
			this.requireEncryption = requireEncryption;
		}

		public void setEncryptionMinLevel(String encryptionMinLevel) {
			this.encryptionMinLevel = encryptionMinLevel;
		}

		public void setAllowNonEncryptedOut(boolean allowNonEncryptedOut) {
			this.allowNonEncryptedOut = allowNonEncryptedOut;
		}

		public void setAllowNonEncryptedIn(boolean allowNonEncryptedIn) {
			this.allowNonEncryptedIn = allowNonEncryptedIn;
		}

		public void setUseCryptoPortExtension(boolean useCryptoPortExtension) {
			this.useCryptoPortExtension = useCryptoPortExtension;
		}

		public void setTcpPortIn(int tcpPortIn) {
			this.tcpPortIn = tcpPortIn;
		}
		
		public void setUdpPortIn(int udpPortIn) {
			this.udpPortIn = udpPortIn;
		}
		
		public float getShortDeltaAverageUploadRate() {
			return getBytesSent()/(getShortDeltaTime()!=0?getShortDeltaTime():1);
		}
		
		public float getLongDeltaAverageUploadRate() {
			return getAverageUpload();
		}
		
		public float getShortDeltaAverageDownloadRate() {
			return getBytesReceived()/(getShortDeltaTime()!=0?getShortDeltaTime():1);
		}
		
		public float getLongDeltaAverageDownloadRate() {
			return getAverageDownload();
		}

		
		private void updateDynamic2() throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException {
			
			HashSet<String> slidingWindowVars = new HashSet<String>();
			HashMap<String, Long> sumVars = new HashMap<String, Long>();
			HashMap<String, Float> averageVars = new HashMap<String, Float>();
			HashMap<String, Float> sdqDevVars = new HashMap<String, Float>();
			
			slidingWindowVars.add("uploadRate");
			slidingWindowVars.add("downloadRate");
			slidingWindowVars.add("maxUploadRate");
			slidingWindowVars.add("maxDownloadRate");
			slidingWindowVars.add("numConnectedSeeds");
			slidingWindowVars.add("numConnectedLeechers");
			slidingWindowVars.add("bytesSentLpsd");
			slidingWindowVars.add("bytesReceivedLpsd");
			slidingWindowVars.add("swarmPeerSpeed");
			
        	int size = entryList.size();
        	int startIndex = 0;
        	
        	for (String varName : slidingWindowVars) {
        		sumVars.put(varName, 0l);
        		averageVars.put(varName, 0f);
        		sdqDevVars.put(varName, 0f);
        	}
        	
        	for (int i=startIndex; i < size; i++) {
        		
        		DynamicLocalPeerStat lpsd = (DynamicLocalPeerStat) entryList.get(i);
        		
            	for (String varName : slidingWindowVars) {
            		Field field = lpsd.getClass().getField(varName);
            		sumVars.put(varName, sumVars.get(varName) + field.getLong(lpsd));
            	}
        	}
        	
        	for (String varName : slidingWindowVars)
        		averageVars.put(varName, (float) sumVars.get(varName) / (float) size);


        	for (int i=startIndex; i < size; i++) {
        		
        		DynamicLocalPeerStat lpsd = (DynamicLocalPeerStat) entryList.get(i);
        		
            	for (String varName : slidingWindowVars) {
            		Field field = lpsd.getClass().getField(varName);
            		sumVars.put(varName, sumVars.get(varName) + field.getLong(lpsd));
            		sdqDevVars.put(varName, (float) (sdqDevVars.get(varName) 
            				+ Math.pow( (averageVars.get(varName) - field.getLong(lpsd)), 2)));
            	}
        	}
        	
        	this.stdevUpload = ( (float) Math.sqrt(sdqDevVars.get("uploadRate") / size ));
            this.stdevDownload = ( (float) Math.sqrt(sdqDevVars.get("downloadRate") / size) );
            this.stdevNumConnectedSeeds = ( (float) Math.sqrt(sdqDevVars.get("numConnectedSeeds") / size) );
            this.stdevNumConnectedLeechers = ( (float) Math.sqrt(sdqDevVars.get("numConnectedLeechers") / size) );
            this.stdevBytesSent = ( (float) Math.sqrt(sdqDevVars.get("bytesSentLpsd") / size) );
            this.stdevBytesReceived = ( (float) Math.sqrt(sdqDevVars.get("bytesReceivedLpsd") / size) );
            this.stdevSwarmPeerSpeed = ( (float) Math.sqrt(sdqDevVars.get("swarmPeerSpeed") / size) );
            
            this.averageUpload = averageVars.get("uploadRate");
            this.averageDownload = averageVars.get("downloadRate");
            this.averageNumConnectedSeeds = averageVars.get("numConnectedSeeds");
            this.averageNumConnectedLeechers = averageVars.get("numConnectedLeechers");
            this.averageBytesSent = averageVars.get("bytesSent");
            this.averageBytesSent = averageVars.get("bytesReceived");
            this.averageSwarmPeerSpeed = averageVars.get("swarmPeerSpeed");
            
		}
		
        public void updateDynamic() {

        	int size = entryList.size();
        	int startIndex = 0;
        	
    		int sumUpload=0;
            int sumDownload=0;
            int sumMaxUpload=0;
            int sumMaxDownload=0;
            int sumNumConnectedSeeds=0;
            int sumNumConnectedLeechers=0;
            long sumBytesSent=0;
            long sumBytesReceived=0;

        	for (int i=startIndex; i < size; i++) {
        		
        		DynamicLocalPeerStat lpsd = (DynamicLocalPeerStat) entryList.get(i);
        		sumUpload += lpsd.uploadRate;
        		sumDownload += lpsd.downloadRate;
        		sumMaxUpload += lpsd.maxUploadRate;
        		sumMaxDownload += lpsd.maxDownloadRate;
        		sumNumConnectedSeeds += lpsd.numConnectedSeeds;
        		sumNumConnectedLeechers += lpsd.numConnectedLeechers;
        		sumBytesSent += lpsd.bytesSentLpsd;
        		sumBytesReceived += lpsd.bytesReceivedLpsd;
        	}

        	float averageUpload = ((float) sumUpload / (float) size);
            float averageDownload = ((float) sumDownload / (float) size);
            float averageMaxUpload = ((float) sumMaxUpload / (float) size);
            float averageMaxDownload = ((float) sumMaxDownload / (float) size);
            float averageNumConnectedSeeds = ((float) sumNumConnectedSeeds / (float) size);
            float averageNumConnectedLeechers = ((float) sumNumConnectedLeechers / (float) size);
            float averageBytesSent = ((float) sumBytesSent / (float) size);
            float averageBytesReceived = ((float) sumBytesReceived / (float) size);
          
            float sdqDevUpload = 0;
            float sdqDevDownload = 0;
            float sdqDevMaxUpload = 0;
            float sdqDevMaxDownload = 0;
            float sdqDevNumConnectedSeeds = 0;
            float sdqDevNumConnectedLeechers = 0;
            float sdqDevBytesSent = 0;
            float sdqDevBytesReceived = 0;
        	
        	for (int i=startIndex; i < size; i++) {
        		
        		DynamicLocalPeerStat lpsd = (DynamicLocalPeerStat) entryList.get(i);
        		sdqDevUpload += Math.pow( (averageUpload - lpsd.uploadRate), 2);
        		sdqDevDownload += Math.pow( (averageDownload - lpsd.downloadRate), 2);
        		sdqDevMaxUpload += Math.pow( (averageMaxUpload - lpsd.maxUploadRate), 2);
        		sdqDevMaxDownload += Math.pow( (averageMaxDownload - lpsd.maxDownloadRate), 2);
        		sdqDevNumConnectedSeeds += Math.pow( (averageNumConnectedSeeds - lpsd.numConnectedSeeds), 2);
        		sdqDevNumConnectedLeechers += Math.pow( (averageNumConnectedLeechers - lpsd.numConnectedLeechers), 2);
        		sdqDevBytesSent += Math.pow( (averageBytesSent - lpsd.bytesSentLpsd), 2);
        		sdqDevBytesReceived += Math.pow( (averageBytesReceived - lpsd.bytesReceivedLpsd), 2);
        	}        	
        	        	
        	this.stdevUpload = ( (float) Math.sqrt(sdqDevUpload / size ));
            this.stdevDownload = ( (float) Math.sqrt(sdqDevDownload / size) );
            this.stdevNumConnectedSeeds = ( (float) Math.sqrt(sdqDevNumConnectedSeeds / size) );
            this.stdevNumConnectedLeechers = ( (float) Math.sqrt(sdqDevNumConnectedLeechers / size) );
            this.stdevBytesSent = ( (float) Math.sqrt(sdqDevBytesSent / size ));
            this.stdevBytesReceived = ( (float) Math.sqrt(sdqDevBytesReceived / size ));
            
            this.averageUpload = (averageUpload);
            this.averageDownload = (averageDownload);
            this.averageNumConnectedSeeds = (averageNumConnectedSeeds);
            this.averageNumConnectedLeechers = (averageNumConnectedLeechers);
            this.averageBytesSent = (averageBytesSent);
            this.averageBytesReceived = (averageBytesReceived);

        }
        
}
