/**
 * Dasu Project
 *
 * File:         FactMeasurementRst.java
 * RCS:          $Id$
 * Description:  FactMeasurementRst class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:20:20 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.drools
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.drools;

import java.util.HashMap;
import java.util.Iterator;
import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.measurement.InterferenceSimulator;
import edu.northwestern.dasu.stats.RSTStat;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The FactMeasurementRst class ...
 */
public class FactMeasurementRst extends FactMeasurement<RSTStat> {

		public long timeRecorded = -1;
		public long downloadId = -1;
		public int numRst = -1;		/* in the last minute*/
		public Float averageRst=0f; /* per minute for the past five minutes */
		public Float stdevRst=0f;
		
		int timeWindowSize = DasuConfiguration.getInstance().getRSTimeWindowSize();
				
		public FactMeasurementRst() {
			
		}
		
		public FactMeasurementRst(RSTStat rststat) {

			this.timeRecorded = rststat.timeRecorded;
			this.downloadId = rststat.downloadId;
			this.numRst = 1;
			
			if (DEBUG)
				System.out.println( rststat.timeRecorded + " " + rststat.timeRecorded + " " +
					rststat.downloadId + " "  + rststat.peerIp);
			
			addEntry(rststat);
			
		}

		/* 
		 * Returns the size of the entryLog, if the list is zero, caller
		 * should delete the fact
		 */
		public int updateEntryList() {

			int numRst = 0; 
			long firstTime = 0;
			int initialSize = entryList.size();
			Iterator<RSTStat> iter = entryList.iterator();
			long time = Util.currentGMTTime();
			
			if (DasuManager.USE_SIMULATION)
				time = Util.currentGMTTime() + InterferenceSimulator.getTimeTick()*30000;
			
			while (iter.hasNext()) {
				RSTStat rstat = iter.next();
				
				if (time - rstat.timeRecorded <= timeWindowSize*60*1000) {
					if (firstTime == 0)
						firstTime = rstat.timeRecorded;
					numRst+=rstat.numRst;
				}

				if (time - rstat.timeRecorded >= timeSpan*60*1000) 
					iter.remove();
				
			}
	
			if ((initialSize - entryList.size() != 0))
				updateDynamic();
			
			this.timeRecorded = (firstTime);
			this.numRst = (numRst);

			return getNumRst();
		}
		
		
		@Override
		public void addEntry(RSTStat ent) {
			
			int nRst = 0; 
			long firstTime = 0;
			Iterator<RSTStat> iter = entryList.iterator();

			long time = Util.currentGMTTime();
			if (DasuManager.USE_SIMULATION)
				time = Util.currentGMTTime() + InterferenceSimulator.getTimeTick()*30000;

			while (iter.hasNext()) {
				RSTStat rstat = iter.next();

				if (time - rstat.timeRecorded <= timeWindowSize*60*1000) {
					if (firstTime == 0)
						firstTime = rstat.timeRecorded;
					nRst++;
				}

				if (time - rstat.timeRecorded >= timeSpan*60*1000) 
					iter.remove();		

			}

			if (entryLog.size() == 3 * maxNumEntries)
				entryLog.removeFirst();

			entryLog.add(ent);

			entryList.add(ent);
			updateDynamic();
			updateAll(new RSTStat(ent.downloadId, ent.peerIp, firstTime, nRst));
			
		}

		@Override
		public void updateAll(RSTStat rststat) {
						
			this.timeRecorded = (rststat.timeRecorded);
			this.downloadId = (rststat.downloadId);
			this.numRst = (rststat.numRst);	
			
			if (DEBUG) {
				
				System.out.println("\nFactMeasurementRst");			
				for (int i = 0; i < entryList.size(); i++ ) {
					System.out.println(entryList.get(i).timeRecorded + " did: " + 
							entryList.get(i).downloadId + " peerIp: " + 
							entryList.get(i).peerIp);
				}
			
				System.out.println();
			}
			
		}
		
        public void updateDynamic() {
        	
        	int rstSum = 0;
        	for ( RSTStat r : entryList)
        		rstSum += r.numRst;
        	
            float averageRst = ((float) rstSum / (float) timeSpan);
            HashMap<Integer, Integer> perMinute = new HashMap<Integer, Integer>();
            
            for (int i=1; i<= timeSpan; i++) {
            	perMinute.put(i, 0);
            }
            
			long time = Util.currentGMTTime();			
			if (DasuManager.USE_SIMULATION)
				time = Util.currentGMTTime() + InterferenceSimulator.getTimeTick()*30000;

            int index = 0;
        	Iterator<RSTStat> iter = entryList.iterator();
        	while (iter.hasNext()) {
        		RSTStat rstat = iter.next();

            	index = (int) Math.ceil((float)(time - rstat.timeRecorded)/1000/60);
        		if (index == 0) index++;
        		
        		perMinute.put(index, perMinute.get(index)+rstat.numRst);
            }
            
        	if (index == timeSpan)
        		setWindowFull(true);
        	
        	float sdqDevSumRst = 0;
    		for (int i=1; i <= timeSpan; i++) {

    			if (DEBUG)
    				System.out.println(averageRst + " " + perMinute.get(i));
    			
    			sdqDevSumRst += Math.pow( (averageRst - perMinute.get(i)), 2);
    		}
            
        	this.stdevRst = ( (float) Math.sqrt(sdqDevSumRst / timeSpan) );        	
            this.averageRst = (averageRst);

            
        }
		
		
		public long getTimeRecorded() {
			return timeRecorded;
		}

		public int getNumRst() {
			return numRst;
		}
		
		public long getDownloadId() {
			return downloadId;
		}
			
		public Float getAverageRst() {
			return averageRst;
		}
		
		public Float getStdevRst() {
			return stdevRst;
		}
		
		public void setTimeRecorded(Long timeRecorded) {
			Long temp = this.timeRecorded;
			this.timeRecorded = timeRecorded;
	        pcs.firePropertyChange("timeRecorded", new Long(temp), new Long(timeRecorded));
		}

		public void setNumRst(Integer numRst) {
			Integer temp = this.numRst;
			this.numRst = numRst;
	        pcs.firePropertyChange("numRst", new Integer(temp), new Integer(numRst));
		}
		
		public void setDownloadId(Long downloadId) {
			Long temp = this.downloadId;
			this.downloadId = downloadId;
	        pcs.firePropertyChange("downloadId", new Long(temp), new Long(downloadId));
		}

		public void setAverageRst(float averageRst) {
			Float temp = this.averageRst;
			this.averageRst = averageRst;
	        pcs.firePropertyChange("averageRst", new Float(temp), new Float(averageRst));			
		}

		public void setStdevRst(float stdevRst) {
			Float temp = this.stdevRst;
			this.stdevRst = stdevRst;
	        pcs.firePropertyChange("stdevRst", new Float(temp), new Float(stdevRst));			
		}

        
}
