package edu.northwestern.dasu.drools;

import edu.northwestern.dasu.stats.NetstatSystemStats;

/* 
 * For an member to be visible to Drools (as part of the fact)  
 *  it must have it's own public setter and getter functions
 */
public class FactMeasurementSystem extends FactMeasurement<NetstatSystemStats> {
		
		public long timeRecorded = -1;
		private int rst = -1;
		private int numConnections=-1;
		private int passiveOpens=-1;
		private int activeOpens=-1;
		private int failedConns=-1;
		private long bytesSent=-1;
		private long bytesReceived=-1;
		private long shortDeltaTime=-1;
		private long longDeltaTime=-1;
		
		private Float averageRst=0f;
        private Float averageNumConnections=0f;
        private Float averagePassiveOpen=0f;
        private Float averageActiveOpen=0f;
        private Float averageFailedCons=0f;
        private Float averageBytesSent=0f;
        private Float averageBytesReceived=0f;
		private Float stdevRst=0f;
        private Float stdevNumConnections=0f;
        private Float stdevPassiveOpen=0f;
        private Float stdevActiveOpen=0f;
        private Float stdevFailedCons=0f;
        private Float stdevBytesSent=0f;
        private Float stdevBytesReceived=0f;
        
        private NetstatSystemStats lastMeasurement = new NetstatSystemStats();
		
		public FactMeasurementSystem(NetstatSystemStats netstatStat) {
			
			this.timeRecorded = netstatStat.timeRecorded;
			this.rst = netstatStat.rst;
			this.numConnections = netstatStat.numConnections;
			this.passiveOpens = netstatStat.passiveOpens;
			this.activeOpens = netstatStat.activeOpens;
			this.failedConns = netstatStat.failedConns;
			this.bytesSent = netstatStat.bytesSent;
			this.bytesReceived = netstatStat.bytesReceived;
			
			if (DEBUG)
				System.out.println( netstatStat );
			
			addEntry(netstatStat);
		}
		
		@Override
		public void addEntry(NetstatSystemStats netstatStat) {

			NetstatSystemStats tmp = (NetstatSystemStats) netstatStat.clone();

			int size = entryList.size();
			if (size == 0) 
				lastMeasurement = tmp;
			
			if (size == maxNumEntries) {
				entryList.removeFirst();
				windowFull = true;
//				setWindowFull(true);
			}
			
			netstatStat.rst -= lastMeasurement.rst;
			netstatStat.numConnections -= lastMeasurement.numConnections;
			netstatStat.passiveOpens -= lastMeasurement.passiveOpens;
			netstatStat.activeOpens -= lastMeasurement.activeOpens;
			netstatStat.failedConns -= lastMeasurement.failedConns;	
			netstatStat.bytesSent -= lastMeasurement.bytesSent;	
			netstatStat.bytesReceived -= lastMeasurement.bytesReceived;
			
			updateAll(netstatStat);
			lastMeasurement = tmp;
			
			updateTimeStamp();
			
		}

				
		@Override
		public void updateAll(NetstatSystemStats netstatStat) {
						
			/* Holds the 3 times the number of entries in a movingWindow */
			if (entryList.size() == maxNumEntries * 3)
				entryList.removeFirst();

			entryList.add(netstatStat);
			
			timeRecorded = (netstatStat.timeRecorded);
			rst = (netstatStat.rst);
			numConnections = (netstatStat.numConnections);
			passiveOpens = (netstatStat.passiveOpens);
			activeOpens = (netstatStat.activeOpens);
			failedConns = (netstatStat.failedConns);
			bytesSent = (netstatStat.bytesSent);
			bytesReceived = (netstatStat.bytesReceived);

			if (entryList.size() > 0) {
				longDeltaTime = (((NetstatSystemStats) entryList.get(entryList.size()-1)).timeRecorded - ((NetstatSystemStats) entryList.get(0)).timeRecorded)/1000;
				int lastIndex = entryList.size()-1;
				int fistIndex = entryList.size()>1 ? lastIndex-1 : lastIndex; 
				shortDeltaTime = (((NetstatSystemStats) entryList.get(lastIndex)).timeRecorded - ((NetstatSystemStats) entryList.get(fistIndex)).timeRecorded)/1000;
			}


			updateDynamic();
			
			
			if (DEBUG) {
				
				System.out.println("\nFactMeasurementSystem");			
				for (int i = 0; i < entryList.size(); i++ ) {
					System.out.println(entryList.get(i).timeRecorded + " RST " +
							entryList.get(i).rst + " numConnections: " + 
							entryList.get(i).numConnections + " passiveOpens: " +
							entryList.get(i).passiveOpens + " activeOpens: " + 
							entryList.get(i).activeOpens + " bytesSent: " + 
							entryList.get(i).bytesSent + " bytesReceived: " + 
							entryList.get(i).bytesReceived);
				}
			
				System.out.println("avgRST: " + averageRst + 
					" avgNunConn: " + averageNumConnections + 
					" avgPassiveOpens: " + averagePassiveOpen +
					" avgActiveOpens: " + averageActiveOpen +
					" averageBytesSent: " + averageBytesSent +
					" averageBytesRecieved " + averageBytesReceived);
				
				System.out.println();
			}

		}
		
				
		public long getTimeRecorded() {
			return timeRecorded;
		}

		public int getRst() {
			return rst;
		}
		
		public int getNumConnections() {
			return numConnections;
		}
		
		public int getPassiveOpens() {
			return passiveOpens;
		}
		
		public int getActiveOpens() {
			return activeOpens;
		}
		
		public int getFailedConns() {
			return failedConns;
		}

		public long getBytesSent() {
			return bytesSent;
		}

		public long getBytesReceived() {
			return bytesReceived;
		}
		
		public long getShortDeltaTime() {
			return shortDeltaTime;
		}
		
		public long getLongDeltaTime() {
			return longDeltaTime;
		}

		public Float getAverageRst() {
			return averageRst;
		}

		public Float getAverageNumConnections() {
			return averageNumConnections;
		}

		public Float getAveragePassiveOpen() {
			return averagePassiveOpen;
		}
		
		public Float getAverageActiveOpen() {
			return averageActiveOpen;
		}
		
		public Float getAverageFailedCons() {
			return averageFailedCons;
		}
		
		public Float getAverageBytesSent() {
			return averageBytesSent;
		}
		
		public Float getAverageBytesReceived() {
			return averageBytesReceived;
		}

		public Float getStdevRst() {
			return stdevRst;
		}
		
		public Float getStdevNumConnections() {
			return stdevNumConnections;
		}

		public Float getStdevPassiveOpen() {
			return stdevPassiveOpen;
		}

		public Float getStdevActiveOpen() {
			return stdevActiveOpen;
		}

		public Float getStdevFailedCons() {
			return stdevFailedCons;
		}
		
		public Float getStdevBytesSent() {
			return stdevBytesSent;
		}
		
		public Float getStdevBytesReceived() {
			return stdevBytesReceived;
		}
		
		public void setTimeRecorded(Long timeRecorded) {
			Long temp = this.timeRecorded;
			this.timeRecorded = timeRecorded;
	        pcs.firePropertyChange("timeRecorded", new Long(temp), new Long(timeRecorded));
		}

		public void setRst(Integer rst) {
			Integer temp = this.rst;
			this.rst = rst;
	        pcs.firePropertyChange("rst", new Integer(temp), new Integer(rst));
		}
		
		public void setNumConnections(Integer numConnections) {
			Integer temp = this.numConnections;
			this.numConnections = numConnections;
	        pcs.firePropertyChange("numConnections", new Integer(temp), new Integer(numConnections));
		}
				
		public void setPassiveOpens(Integer passiveOpens) {
			Integer temp = this.passiveOpens;
			this.passiveOpens = passiveOpens;
	        pcs.firePropertyChange("passiveOpens", new Integer(temp), new Integer(passiveOpens));			
		}
		
		public void setActiveOpens(Integer activeOpens) {
			Integer temp = this.activeOpens;
			this.activeOpens = activeOpens;
	        pcs.firePropertyChange("activeOpens", new Integer(temp), new Integer(activeOpens));			
		}
		
		public void setFailedConns(Integer failedConns) {
			Integer temp = this.failedConns;
			this.failedConns = failedConns;
	        pcs.firePropertyChange("failedConns", new Integer(temp), new Integer(failedConns));			
		}
		
		public void setBytesSent(Long bytesSent) {
			Long temp = this.bytesSent;
			this.bytesSent = bytesSent;
	        pcs.firePropertyChange("bytesSent", new Long(temp), new Long(bytesSent));			
		}
		
		public void setBytesReceived(Long bytesReceived) {
			Long temp = this.bytesReceived;
			this.bytesReceived = bytesReceived;
	        pcs.firePropertyChange("bytesReceived", new Long(temp), new Long(bytesReceived));			
		}
        
		public void setAverageRst(float averageRst) {
			Float temp = this.averageRst;
			this.averageRst = averageRst;
	        pcs.firePropertyChange("averageRst", new Float(temp), new Float(averageRst));			
		}

		public void setAverageNumConnections(float averageNumConnections) {
			Float temp = this.averageNumConnections;
			this.averageNumConnections = averageNumConnections;
	        pcs.firePropertyChange("averageNumConnections", new Float(temp), new Float(averageNumConnections));			
		}

		public void setAveragePassiveOpen(float averagePassiveOpen) {
			Float temp = this.averagePassiveOpen;
			this.averagePassiveOpen = averagePassiveOpen;
	        pcs.firePropertyChange("averagePassiveOpen", new Float(temp), new Float(averagePassiveOpen));			
		}

		public void setAverageActiveOpen(float averageActiveOpen) {
			Float temp = this.averageActiveOpen;
			this.averageActiveOpen = averageActiveOpen;
	        pcs.firePropertyChange("averageActiveOpen", new Float(temp), new Float(averageRst));			
		}

		public void setAverageFailedCons(float averageFailedCons) {
			Float temp = this.averageFailedCons;
			this.averageFailedCons = averageFailedCons;
	        pcs.firePropertyChange("averageFailedCons", new Float(temp), new Float(averageFailedCons));			
		}
		
		public void setAverageBytesSent(float averageBytesSent) {
			Float temp = this.averageBytesSent;
			this.averageBytesSent = averageBytesSent;
	        pcs.firePropertyChange("averageBytesSent", new Float(temp), new Float(averageBytesSent));			
		}

		public void setAverageBytesReceived(float averageBytesReceived) {
			Float temp = this.averageBytesReceived;
			this.averageBytesReceived = averageBytesReceived;
	        pcs.firePropertyChange("averageBytesReceived", new Float(temp), new Float(averageBytesReceived));			
		}
		
		public void setStdevRst(float stdevRst) {
			Float temp = this.stdevRst;
			this.stdevRst = stdevRst;
	        pcs.firePropertyChange("stdevRst", new Float(temp), new Float(stdevRst));			
		}

		public void setStdevNumConnections(float stdevNumConnections) {
			Float temp = this.stdevNumConnections;
			this.stdevNumConnections = stdevNumConnections;
	        pcs.firePropertyChange("stdevNumConnections", new Float(temp), new Float(stdevNumConnections));			
		}

		public void setStdevPassiveOpen(float stdevPassiveOpen) {
			Float temp = this.stdevPassiveOpen;
			this.stdevPassiveOpen = stdevPassiveOpen;
	        pcs.firePropertyChange("stdevPassiveOpen", new Float(temp), new Float(stdevPassiveOpen));			
		}

		public void setStdevActiveOpen(float stdevActiveOpen) {
			Float temp = this.stdevActiveOpen;
			this.stdevActiveOpen = stdevActiveOpen;
	        pcs.firePropertyChange("stdevActiveOpen", new Float(temp), new Float(stdevActiveOpen));			
		}

		public void setStdevFailedCons(float stdevFailedCons) {
			Float temp = this.stdevFailedCons;
			this.stdevFailedCons = stdevFailedCons;
	        pcs.firePropertyChange("stdevFailedCons", new Float(temp), new Float(stdevFailedCons));			
		}
		
		public void setStdevBytesSent(float stdevBytesSent) {
			Float temp = this.stdevBytesSent;
			this.stdevBytesSent = stdevBytesSent;
	        pcs.firePropertyChange("stdevBytesSent", new Float(temp), new Float(stdevBytesSent));			
		}

		public void setStdevBytesReceived(float stdevBytesReceived) {
			Float temp = this.stdevBytesReceived;
			this.stdevBytesReceived = stdevBytesReceived;
	        pcs.firePropertyChange("stdevBytesReceived", new Float(temp), new Float(stdevBytesReceived));			
		}
	
		public float getShortDeltaAverageUploadRate() {
			return getBytesSent()/(getShortDeltaTime()!=0?getShortDeltaTime():1);
		}
		
		public float getLongDeltaAverageUploadRate() {
			return getAverageBytesSent()/(getLongDeltaTime()!=0?getLongDeltaTime():1);
		}
		
		public float getShortDeltaAverageDownloadRate() {
			return getBytesReceived()/(getShortDeltaTime()!=0?getShortDeltaTime():1);
		}
		
		public float getLongDeltaAverageDownloadRate() {
			return getAverageBytesReceived()/(getLongDeltaTime()!=0?getLongDeltaTime():1);
		}

		
        private void updateDynamic() {

        	int size = entryList.size();
        	int startIndex = 0;
        	
    		int sumRst=0;
            int sumNumConnections=0;
            int sumPassiveOpen=0;
            int sumActiveOpen=0;
            int sumFailedCons=0;
            long sumBytesSent=0;
            long sumBytesReceived=0;

        	for (int i=startIndex; i < size; i++) {
        		
        		NetstatSystemStats netstatStat = (NetstatSystemStats) entryList.get(i);
        		sumRst += netstatStat.rst;
        		sumNumConnections += netstatStat.numConnections;
        		sumPassiveOpen += netstatStat.passiveOpens;
        		sumActiveOpen += netstatStat.activeOpens;
        		sumFailedCons += netstatStat.failedConns;
        		sumBytesSent += netstatStat.bytesSent;
        		sumBytesReceived += netstatStat.bytesReceived;
        	}
        
            float averageRst = ((float) sumRst / (float) size);
            float averageNumConnections = ((float) sumNumConnections / (float) size);
            float averagePassiveOpen = ((float) sumPassiveOpen / (float) size);
            float averageActiveOpen = ((float) sumActiveOpen / (float) size);
            float averageFailedCons = ((float) sumFailedCons / (float) size);
            float averageBytesSent = ((float) sumBytesSent / (float) size);
            float averageBytesReceived = ((float) sumBytesReceived / (float) size);
            
            float sdqDevSumRst = 0;
            float sdqDevSumNumConnections = 0;
            float sdqDevSumPassiveOpen = 0;
            float sdqDevSumActiveOpen = 0;
            float sdqDevSumFailedCons = 0;
            float sdqDevSumBytesSent = 0;
            float sdqDevSumBytesReceived = 0;
        	
        	for (int i=startIndex; i < size; i++) {
        		
        		NetstatSystemStats netstatStat = (NetstatSystemStats) entryList.get(i);
        		sdqDevSumRst += Math.pow( (averageRst - netstatStat.rst), 2);
        		sdqDevSumNumConnections += Math.pow( (averageNumConnections - netstatStat.numConnections), 2);
        		sdqDevSumPassiveOpen += Math.pow( (averagePassiveOpen - netstatStat.passiveOpens), 2);
        		sdqDevSumActiveOpen += Math.pow( (averageActiveOpen - netstatStat.activeOpens), 2);
        		sdqDevSumFailedCons += Math.pow( (averageFailedCons - netstatStat.failedConns), 2);
        		sdqDevSumBytesSent += Math.pow( (averageBytesSent - netstatStat.bytesSent), 2);
        		sdqDevSumBytesReceived += Math.pow( (averageBytesReceived - netstatStat.bytesReceived), 2);

        	}        	
        	
            this.stdevRst = ( (float) Math.sqrt(sdqDevSumRst / size ));
            this.stdevNumConnections = ( (float) Math.sqrt(sdqDevSumNumConnections / size) );
            this.stdevPassiveOpen = ( (float) Math.sqrt(sdqDevSumPassiveOpen / size) );
            this.stdevActiveOpen = ( (float) Math.sqrt(sdqDevSumActiveOpen / size) );
            this.stdevFailedCons = ( (float) Math.sqrt(sdqDevSumFailedCons / size) );
            this.stdevBytesSent = ( (float) Math.sqrt(sdqDevSumBytesSent / size) );
            this.stdevBytesReceived = ( (float) Math.sqrt(sdqDevSumBytesReceived / size) );
            
            this.averageRst = ((float) sumRst / (float) size);
            this.averageNumConnections = ((float) sumNumConnections / (float) size);
            this.averagePassiveOpen = ((float) sumPassiveOpen / (float) size);
            this.averageActiveOpen = ((float) sumActiveOpen / (float) size);
            this.averageFailedCons = ((float) sumFailedCons / (float) size);
            this.averageBytesSent = ((float) sumBytesSent / (float) size);
            this.averageBytesReceived = ((float) sumBytesReceived / (float) size);
            
        }

        
}
