package edu.northwestern.dasu.drools;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.HashSet;

import edu.northwestern.dasu.stats.UPnPSystemStats;

/*
 * For an member to be visible to Drools (as part of the fact)  
 *  it must have it's own public setter and getter functions
 */
public class FactMeasurementUPnP extends FactMeasurement<UPnPSystemStats> {
		
		public long timeRecorded=-1;
		public long numberOfActiveConnections=-1;
		public long totalBytesSent=-1;
		public long totalBytesReceived=-1;
		public long totalPacketsSent=-1;
		public long totalPacketsReceived=-1;
		public long uptime=-1;
		private long shortDeltaTime=-1;
		private long longDeltaTime=-1;
		
		private Float averageNumberOfActiveConnections=0f;
        private Float averageTotalBytesSent=0f;
        private Float averageTotalBytesReceived=0f;
        private Float averageTotalPacketsSent=0f;
        private Float averageTotalPacketsReceived=0f;
		private Float stdevNumberOfActiveConnections=0f;
        private Float stdevTotalBytesSent=0f;
        private Float stdevTotalBytesReceived=0f;
        private Float stdevTotalPacketsSent=0f;
        private Float stdevTotalPacketsReceived=0f;
        
        private UPnPSystemStats lastMeasurement = new UPnPSystemStats();
		
		public FactMeasurementUPnP(UPnPSystemStats upnpStat) {
			
			this.timeRecorded = upnpStat.timeRecorded;
			this.numberOfActiveConnections = upnpStat.numberOfActiveConnections;
			this.totalBytesSent = upnpStat.totalBytesSent;
			this.totalBytesReceived = upnpStat.totalBytesReceived;
			this.totalPacketsSent = upnpStat.totalPacketsSent;
			this.totalPacketsReceived = upnpStat.totalPacketsReceived;
			this.uptime = upnpStat.uptime;
			
			if (DEBUG)
				System.out.println( upnpStat );
			
			addEntry(upnpStat);
		}
		
		@Override
		public void addEntry(UPnPSystemStats upnpStat) {

			UPnPSystemStats tmp = (UPnPSystemStats) upnpStat.clone();

			int size = entryList.size();
			if (size == 0) 
				lastMeasurement = tmp;
			
			if (size == maxNumEntries) {
				entryList.removeFirst();
				windowFull = true;
//				setWindowFull(true);
			}
			
			upnpStat.numberOfActiveConnections -= lastMeasurement.numberOfActiveConnections;
			upnpStat.totalBytesSent -= lastMeasurement.totalBytesSent;
			upnpStat.totalBytesReceived -= lastMeasurement.totalBytesReceived;
			upnpStat.totalPacketsSent -= lastMeasurement.totalPacketsSent;
			upnpStat.totalPacketsReceived -= lastMeasurement.totalPacketsReceived;
			
			updateAll(upnpStat);
			lastMeasurement = tmp;
			
			updateTimeStamp();
			
		}

				
		@Override
		public void updateAll(UPnPSystemStats upnpStat) {
						
			/* Holds the 3 times the number of entries in a movingWindow */
			if (entryList.size() == maxNumEntries * 3)
				entryList.removeFirst();

			entryList.add(upnpStat);
			
			timeRecorded = upnpStat.timeRecorded;
			numberOfActiveConnections = upnpStat.numberOfActiveConnections;
			totalBytesSent = upnpStat.totalBytesSent;
			totalBytesReceived = upnpStat.totalBytesReceived;
			totalPacketsSent = upnpStat.totalPacketsSent;
			totalPacketsReceived = upnpStat.totalPacketsReceived;
			
			if (entryList.size() > 0) {
				longDeltaTime = (((UPnPSystemStats) entryList.get(entryList.size()-1)).timeRecorded - ((UPnPSystemStats) entryList.get(0)).timeRecorded)/1000;
				int lastIndex = entryList.size()-1;
				int fistIndex = entryList.size()>1 ? lastIndex-1 : lastIndex; 
				shortDeltaTime = (((UPnPSystemStats) entryList.get(lastIndex)).timeRecorded - ((UPnPSystemStats) entryList.get(fistIndex)).timeRecorded)/1000;
			}


			try {
				updateDynamic2();
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			
			if (DEBUG) {
				
				System.out.println("\nFactMeasurementSystem");			
				for (int i = 0; i < entryList.size(); i++ ) {
					System.out.println(entryList.get(i).timeRecorded + " numberOfActiveConnections " +
							entryList.get(i).numberOfActiveConnections + " totalBytesSent: " + 
							entryList.get(i).totalBytesSent + " totalBytesReceived: " +
							entryList.get(i).totalBytesReceived + " totalPacketsSent: " + 
							entryList.get(i).totalPacketsSent + " totalPacketsReceived: " + 
							entryList.get(i).totalPacketsReceived);
				}
			
				System.out.println("averageNumberOfActiveConnections: " + averageNumberOfActiveConnections + 
					" averageTotalBytesSent: " + averageTotalBytesSent + 
					" averageTotalBytesReceived: " + averageTotalBytesReceived +
					" averageTotalPacketsSent: " + averageTotalPacketsSent +
					" averageTotalPacketsReceived: " + averageTotalPacketsReceived);
				
				System.out.println();
			}

		}
		
				
		public long getTimeRecorded() {
			return timeRecorded;
		}
		
		public long getShortDeltaTime() {
			return shortDeltaTime;
		}
		
		public long getLongDeltaTime() {
			return longDeltaTime;
		}
		
		public void setTimeRecorded(Long timeRecorded) {
			Long temp = this.timeRecorded;
			this.timeRecorded = timeRecorded;
	        pcs.firePropertyChange("timeRecorded", new Long(temp), new Long(timeRecorded));
		}
		
        private void updateDynamic() {

        	int size = entryList.size();
        	int startIndex = 0;
        	
    		int sumNumberOfActiveConnections=0;
            int sumTotalBytesSent=0;
            int sumTotalBytesReceived=0;
            int sumTotalPacketsSent=0;
            int sumtTotalPacketsReceived=0;
            
        	for (int i=startIndex; i < size; i++) {

        		UPnPSystemStats upnpStat = (UPnPSystemStats) entryList.get(i);
        		sumNumberOfActiveConnections += upnpStat.numberOfActiveConnections;
        		sumTotalBytesSent += upnpStat.totalBytesSent;
        		sumTotalBytesReceived += upnpStat.totalBytesReceived;
        		sumTotalPacketsSent += upnpStat.totalPacketsSent;
        		sumtTotalPacketsReceived += upnpStat.totalPacketsReceived;
        	}
        
        	float averageNumberOfActiveConnections = ((float) sumNumberOfActiveConnections / (float) size);
        	float averageTotalBytesSent = ((float) sumTotalBytesSent / (float) size);
        	float averageTotalBytesReceived = ((float) sumTotalBytesReceived / (float) size);
        	float averageTotalPacketsSent = ((float) sumTotalPacketsSent / (float) size);
        	float averageTotalPacketsReceived = ((float) sumtTotalPacketsReceived / (float) size);
        	            
            float sdqNumberOfActiveConnections = 0;
            float sdqTotalBytesSent = 0;
            float sdqTotalBytesReceived = 0;            
            float sdqTotalPacketsSent = 0;            
            float sdqTotalPacketsReceived = 0;
            
        	for (int i=startIndex; i < size; i++) {
        		
        		UPnPSystemStats upnpStat = (UPnPSystemStats) entryList.get(i);
        		sdqNumberOfActiveConnections += Math.pow( (averageNumberOfActiveConnections - upnpStat.numberOfActiveConnections), 2);
        		sdqTotalBytesSent += Math.pow( (averageTotalBytesSent - upnpStat.totalBytesSent), 2);        		
        		sdqTotalBytesReceived += Math.pow( (averageTotalBytesReceived - upnpStat.totalBytesReceived), 2);        		
        		sdqTotalPacketsSent += Math.pow( (averageTotalPacketsSent - upnpStat.totalPacketsSent), 2);        		
        		sdqTotalPacketsReceived += Math.pow( (averageTotalPacketsReceived - upnpStat.totalPacketsReceived), 2);

        	}        	
        	
            this.stdevNumberOfActiveConnections = ( (float) Math.sqrt( sdqNumberOfActiveConnections / size ));
            this.stdevTotalBytesSent = ( (float) Math.sqrt( sdqTotalBytesSent / size) );
            this.stdevTotalBytesReceived = ( (float) Math.sqrt( sdqTotalBytesReceived / size) );
            this.stdevTotalPacketsSent = ( (float) Math.sqrt( sdqTotalPacketsSent / size) );
            this.stdevTotalPacketsReceived = ( (float) Math.sqrt (sdqTotalPacketsReceived / size) );
            
            this.averageNumberOfActiveConnections = ((float) sumNumberOfActiveConnections / (float) size);
            this.averageTotalBytesSent = ((float) sumTotalBytesSent / (float) size);
            this.averageTotalBytesReceived = ((float) sumTotalBytesReceived / (float) size);
            this.averageTotalPacketsSent = ((float) sumTotalPacketsSent / (float) size);
            this.averageTotalPacketsReceived = ((float) sumTotalPacketsSent / (float) size);
            
        }
        
		private void updateDynamic2() throws SecurityException, NoSuchFieldException, IllegalArgumentException, IllegalAccessException {
			
			HashSet<String> slidingWindowVars = new HashSet<String>();
			HashMap<String, Long> sumVars = new HashMap<String, Long>();
			HashMap<String, Float> averageVars = new HashMap<String, Float>();
			HashMap<String, Float> sdqDevVars = new HashMap<String, Float>();
			
			slidingWindowVars.add("numberOfActiveConnections");
			slidingWindowVars.add("totalBytesSent");
			slidingWindowVars.add("totalBytesReceived");
			slidingWindowVars.add("totalPacketsSent");
			slidingWindowVars.add("totalPacketsReceived");
			
        	int size = entryList.size();
        	int startIndex = 0;
        	
        	for (String varName : slidingWindowVars) {
        		sumVars.put(varName, 0l);
        		averageVars.put(varName, 0f);
        		sdqDevVars.put(varName, 0f);
        	}
        	
        	for (int i=startIndex; i < size; i++) {
        		
        		UPnPSystemStats upnp = (UPnPSystemStats) entryList.get(i);
        		
            	for (String varName : slidingWindowVars) {
            		Field field = upnp.getClass().getField(varName);
            		sumVars.put(varName, sumVars.get(varName) + field.getLong(upnp));
            	}
        	}
        	
        	for (String varName : slidingWindowVars)
        		averageVars.put(varName, (float) sumVars.get(varName) / (float) size);


        	for (int i=startIndex; i < size; i++) {
        		
        		UPnPSystemStats upnp = (UPnPSystemStats) entryList.get(i);
        		
            	for (String varName : slidingWindowVars) {
            		Field field = upnp.getClass().getField(varName);
            		sumVars.put(varName, sumVars.get(varName) + field.getLong(upnp));
            		sdqDevVars.put(varName, (float) (sdqDevVars.get(varName) 
            				+ Math.pow( (averageVars.get(varName) - field.getLong(upnp)), 2)));
            	}
        	}
        	
        	this.stdevNumberOfActiveConnections = ( (float) Math.sqrt(sdqDevVars.get("numberOfActiveConnections") / size ));
            this.stdevTotalBytesSent = ( (float) Math.sqrt(sdqDevVars.get("totalBytesSent") / size) );
            this.stdevTotalBytesReceived = ( (float) Math.sqrt(sdqDevVars.get("totalBytesReceived") / size) );
            this.stdevTotalPacketsSent = ( (float) Math.sqrt(sdqDevVars.get("totalPacketsSent") / size) );
            this.stdevTotalPacketsReceived = ( (float) Math.sqrt(sdqDevVars.get("totalPacketsReceived") / size) );
            
            this.averageNumberOfActiveConnections = averageVars.get("numberOfActiveConnections");
            this.averageTotalBytesSent = averageVars.get("totalBytesSent");
            this.averageTotalBytesReceived = averageVars.get("totalBytesReceived");
            this.averageTotalPacketsSent = averageVars.get("totalPacketsSent");
            this.averageTotalPacketsReceived = averageVars.get("totalPacketsReceived");
            
		}

		/**
		 * @return Returns the numberOfActiveConnections.
		 */
		public long getNumberOfActiveConnections() {
			return numberOfActiveConnections;
		}

		/**
		 * @param numberOfActiveConnections The numberOfActiveConnections to set.
		 */
		public void setNumberOfActiveConnections(long numberOfActiveConnections) {
			this.numberOfActiveConnections = numberOfActiveConnections;
		}

		/**
		 * @return Returns the totalBytesSent.
		 */
		public long getTotalBytesSent() {
			return totalBytesSent;
		}

		/**
		 * @param totalBytesSent The totalBytesSent to set.
		 */
		public void setTotalBytesSent(long totalBytesSent) {
			this.totalBytesSent = totalBytesSent;
		}

		/**
		 * @return Returns the totalBytesReceived.
		 */
		public long getTotalBytesReceived() {
			return totalBytesReceived;
		}

		/**
		 * @param totalBytesReceived The totalBytesReceived to set.
		 */
		public void setTotalBytesReceived(long totalBytesReceived) {
			this.totalBytesReceived = totalBytesReceived;
		}

		/**
		 * @return Returns the totalPacketsSent.
		 */
		public long getTotalPacketsSent() {
			return totalPacketsSent;
		}

		/**
		 * @param totalPacketsSent The totalPacketsSent to set.
		 */
		public void setTotalPacketsSent(long totalPacketsSent) {
			this.totalPacketsSent = totalPacketsSent;
		}

		/**
		 * @return Returns the totalPacketsReceived.
		 */
		public long getTotalPacketsReceived() {
			return totalPacketsReceived;
		}

		/**
		 * @param totalPacketsReceived The totalPacketsReceived to set.
		 */
		public void setTotalPacketsReceived(long totalPacketsReceived) {
			this.totalPacketsReceived = totalPacketsReceived;
		}

		/**
		 * @return Returns the uptime.
		 */
		public long getUptime() {
			return uptime;
		}

		/**
		 * @param uptime The uptime to set.
		 */
		public void setUptime(long uptime) {
			this.uptime = uptime;
		}

		/**
		 * @return Returns the averageNumberOfActiveConnections.
		 */
		public Float getAverageNumberOfActiveConnections() {
			return averageNumberOfActiveConnections;
		}

		/**
		 * @param averageNumberOfActiveConnections The averageNumberOfActiveConnections to set.
		 */
		public void setAverageNumberOfActiveConnections(
				Float averageNumberOfActiveConnections) {
			this.averageNumberOfActiveConnections = averageNumberOfActiveConnections;
		}

		/**
		 * @return Returns the averageTotalBytesSent.
		 */
		public Float getAverageTotalBytesSent() {
			return averageTotalBytesSent;
		}

		/**
		 * @param averageTotalBytesSent The averageTotalBytesSent to set.
		 */
		public void setAverageTotalBytesSent(Float averageTotalBytesSent) {
			this.averageTotalBytesSent = averageTotalBytesSent;
		}

		/**
		 * @return Returns the averageTotalBytesReceived.
		 */
		public Float getAverageTotalBytesReceived() {
			return averageTotalBytesReceived;
		}

		/**
		 * @param averageTotalBytesReceived The averageTotalBytesReceived to set.
		 */
		public void setAverageTotalBytesReceived(Float averageTotalBytesReceived) {
			this.averageTotalBytesReceived = averageTotalBytesReceived;
		}

		/**
		 * @return Returns the averageTotalPacketsSent.
		 */
		public Float getAverageTotalPacketsSent() {
			return averageTotalPacketsSent;
		}

		/**
		 * @param averageTotalPacketsSent The averageTotalPacketsSent to set.
		 */
		public void setAverageTotalPacketsSent(Float averageTotalPacketsSent) {
			this.averageTotalPacketsSent = averageTotalPacketsSent;
		}

		/**
		 * @return Returns the averageTotalPacketsReceived.
		 */
		public Float getAverageTotalPacketsReceived() {
			return averageTotalPacketsReceived;
		}

		/**
		 * @param averageTotalPacketsReceived The averageTotalPacketsReceived to set.
		 */
		public void setAverageTotalPacketsReceived(Float averageTotalPacketsReceived) {
			this.averageTotalPacketsReceived = averageTotalPacketsReceived;
		}

		/**
		 * @return Returns the stdevNumberOfActiveConnections.
		 */
		public Float getStdevNumberOfActiveConnections() {
			return stdevNumberOfActiveConnections;
		}

		/**
		 * @param stdevNumberOfActiveConnections The stdevNumberOfActiveConnections to set.
		 */
		public void setStdevNumberOfActiveConnections(
				Float stdevNumberOfActiveConnections) {
			this.stdevNumberOfActiveConnections = stdevNumberOfActiveConnections;
		}

		/**
		 * @return Returns the stdevTotalBytesSent.
		 */
		public Float getStdevTotalBytesSent() {
			return stdevTotalBytesSent;
		}

		/**
		 * @param stdevTotalBytesSent The stdevTotalBytesSent to set.
		 */
		public void setStdevTotalBytesSent(Float stdevTotalBytesSent) {
			this.stdevTotalBytesSent = stdevTotalBytesSent;
		}

		/**
		 * @return Returns the stdevTotalBytesReceived.
		 */
		public Float getStdevTotalBytesReceived() {
			return stdevTotalBytesReceived;
		}

		/**
		 * @param stdevTotalBytesReceived The stdevTotalBytesReceived to set.
		 */
		public void setStdevTotalBytesReceived(Float stdevTotalBytesReceived) {
			this.stdevTotalBytesReceived = stdevTotalBytesReceived;
		}

		/**
		 * @return Returns the stdevTotalPacketsSent.
		 */
		public Float getStdevTotalPacketsSent() {
			return stdevTotalPacketsSent;
		}

		/**
		 * @param stdevTotalPacketsSent The stdevTotalPacketsSent to set.
		 */
		public void setStdevTotalPacketsSent(Float stdevTotalPacketsSent) {
			this.stdevTotalPacketsSent = stdevTotalPacketsSent;
		}

		/**
		 * @return Returns the stdevTotalPacketsReceived.
		 */
		public Float getStdevTotalPacketsReceived() {
			return stdevTotalPacketsReceived;
		}

		/**
		 * @param stdevTotalPacketsReceived The stdevTotalPacketsReceived to set.
		 */
		public void setStdevTotalPacketsReceived(Float stdevTotalPacketsReceived) {
			this.stdevTotalPacketsReceived = stdevTotalPacketsReceived;
		}

		/**
		 * @return Returns the lastMeasurement.
		 */
		public UPnPSystemStats getLastMeasurement() {
			return lastMeasurement;
		}

		/**
		 * @param lastMeasurement The lastMeasurement to set.
		 */
		public void setLastMeasurement(UPnPSystemStats lastMeasurement) {
			this.lastMeasurement = lastMeasurement;
		}

		/**
		 * @param timeRecorded The timeRecorded to set.
		 */
		public void setTimeRecorded(long timeRecorded) {
			this.timeRecorded = timeRecorded;
		}

		/**
		 * @param shortDeltaTime The shortDeltaTime to set.
		 */
		public void setShortDeltaTime(long shortDeltaTime) {
			this.shortDeltaTime = shortDeltaTime;
		}

		/**
		 * @param longDeltaTime The longDeltaTime to set.
		 */
		public void setLongDeltaTime(long longDeltaTime) {
			this.longDeltaTime = longDeltaTime;
		}

        
}
