/**
 * Dasu Project
 *
 * File:         AzureusUpnpTest.java
 * RCS:          $Id$
 * Description:  AzureusUpnpTest class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:37:25 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.core3.ipchecker.natchecker.NatChecker;
import org.gudy.azureus2.core3.util.AEThread;
import org.gudy.azureus2.ui.swt.shells.CoreWaiterSWT;

import com.aelitis.azureus.core.AzureusCore;
import com.aelitis.azureus.core.AzureusCoreFactory;
import com.aelitis.azureus.core.AzureusCoreRunningListener;
import com.aelitis.azureus.core.networkmanager.admin.NetworkAdmin;

import edu.northwestern.dasu.drools.FactUpnpTestResult;
import edu.northwestern.dasu.util.Pair;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The AzureusUpnpTest class ...
 *  The test is capable of receiving the port and protocol to test, 
 * this version takes default port and default tcp protocol 
 */
public class AzureusUpnpTest {
	
  int serverTCPListenPort;
  Checker checker;
  boolean DEBUG = false;
  
  public class Checker extends AEThread {

	private int TCPListenPort;
	private boolean bContinue;
	private final AzureusCore core;

    public Checker(AzureusCore core, int tcp_listen_port) {
        super("NAT Checker");
  			this.core = core;

  		this.TCPListenPort = tcp_listen_port;
        this.bContinue = true;
      }

    public void runSupport() {
    	
    	if (DEBUG)
    		System.out.println(MessageText.getString("configureWizard.nat.testing") + " " + TCPListenPort + " ... ");
    	
          NatChecker checker = new NatChecker(AzureusCoreFactory.getSingleton(), NetworkAdmin.getSingleton().getMultiHomedServiceBindAddresses(true)[0], TCPListenPort, false);          
          switch (checker.getResult()) {
          case NatChecker.NAT_OK :
        	  if (DEBUG) 
        		  System.out.println(MessageText.getString("configureWizard.nat.ok") + "\n" + checker.getAdditionalInfo());
        	  
        	  updateUpnpTestResultQueue(TCPListenPort, true);
        	  break;
        	  
          case NatChecker.NAT_KO :
        	  if (DEBUG)
        		  System.out.println( "\n" + MessageText.getString("configureWizard.nat.ko") + " - " + checker.getAdditionalInfo()+".\n");
        	  
        	  updateUpnpTestResultQueue(TCPListenPort, false);
        	  break;
        	  
          default :
        	  if (DEBUG)
        		  System.out.println( "\n" + MessageText.getString("configureWizard.nat.unable") + ". \n(" + checker.getAdditionalInfo()+").\n");
          
          	  updateUpnpTestResultQueue(TCPListenPort, false, true);
          	  break;
          }
    }
  }
  
  public void runTest(final int TCPListenPort) {

	serverTCPListenPort = COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
   	int	old_tcp 	= COConfigurationManager.getIntParameter( "TCP.Listen.Port" );
   	int	old_udp 	= COConfigurationManager.getIntParameter( "UDP.Listen.Port" );
   	int	old_udp2 	= COConfigurationManager.getIntParameter( "UDP.NonData.Listen.Port" );
    	
    COConfigurationManager.setParameter("TCP.Listen.Port",serverTCPListenPort);
        
    if ( old_tcp == old_udp ){
    	COConfigurationManager.setParameter("UDP.Listen.Port",serverTCPListenPort);
    }
    if ( old_tcp == old_udp2 ){
    	COConfigurationManager.setParameter("UDP.NonData.Listen.Port",serverTCPListenPort);
    }

    COConfigurationManager.save();
    
    CoreWaiterSWT.waitForCoreRunning(new AzureusCoreRunningListener() {
		
		public void azureusCoreRunning(AzureusCore core) {		
			checker = new Checker(core, TCPListenPort);
			checker.start();
		}
	});
    
  }

  protected void updateUpnpTestResultQueue(int port, boolean success) {
	  updateUpnpTestResultQueue(port, success, "tcp", false);
  }
  
  protected void updateUpnpTestResultQueue(int port, boolean success, boolean error) {
	  updateUpnpTestResultQueue(port, success, "tcp", error);
  }
  
  protected void updateUpnpTestResultQueue(int port, boolean success, String protocol, boolean error) {
	
		Long id = System.nanoTime();
		FactUpnpTestResult result = new FactUpnpTestResult(port, success, protocol, error);
		
		synchronized(AMCoordinatorV2.testResultTable) {
			AMCoordinatorV2.testResultTable.put(id, result);
		}
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.UPNP_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
   }

}
