/**
 * Dasu Project
 *
 * File:         DasuNdtModule.java
 * RCS:          $Id$
 * Description:  OptimalEdge class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 15, 2008 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.Random;
import java.util.logging.Logger;

import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.Utilities;

import edu.northwestern.dasu.DasuConfiguration;
import edu.northwestern.dasu.DasuManager;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.asn.NetworkAdminASNLookupImpl;
import edu.northwestern.dasu.drools.FactNdtProbeResult;
import edu.northwestern.dasu.measurement.NetstatBWStats.NetstatBWData;
import edu.northwestern.dasu.stats.Statistics;
import edu.northwestern.dasu.util.Pair;
import edu.uc.ndt.Tcpbw100;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 *         The DasuNdtModule class manages the set of 
 *         Pings performed by the peer.
 */

public class DasuNdtModule extends DasuProbeModule {	
		
	private class TaskEntry {
		
		public Integer taskId;
		public String server;
		public String target;
		
		public TaskEntry(String server, Integer taskId, String target) {
			this.taskId = taskId;
			this.server = server; 
			this.target = target;
		}
		
		// TODO: this constructor is never called and functionality is not implemented. should be deleted?
		public TaskEntry(String server, Integer taskId, boolean stopBT, String target) {
			this.taskId = taskId;
			this.server = server;
			this.target = target;
		}
	}
	
	private final static Logger LOGGER = Logger.getLogger(DasuNdtModule.class.getName());
	
	public Hashtable<Long, FactNdtProbeResult> ndtResultTable;
	private ArrayList<TaskEntry> toDoServerList = new ArrayList<TaskEntry>();
	private static DasuNdtModule self;
	protected static Tcpbw100 ndtTest = new Tcpbw100();
	private static LinkedList<String> servers = new LinkedList<String>();
	
	public DasuNdtModule() {
		super("DasuNdtModule");
		this.pi = Main.getPluginInterface();
		ndtResultTable = new Hashtable<Long, FactNdtProbeResult>();
		
		active = true;
	}
	
	public void loadServerList() {
		String myIp = Main.getPublicIpAddress();
		String append = "";
		
		if (( NetworkAdminASNLookupImpl.getInstance() != null) && (NetworkAdminASNLookupImpl.getInstance().getASNInfo() != null)) 
			append = "&asn=" + NetworkAdminASNLookupImpl.getInstance().getASNInfo().getAS() + "&cc=" +
			NetworkAdminASNLookupImpl.getInstance().getASNInfo().getCountry();
 
		String url = DasuManager.getInstance().getNewRandomDasuUrl() 
			+ DasuConfiguration.NDT_SCRIPT + "?ip=" + myIp + append;

		HashMap<String, String> s = Main.getPropertiesFromServer(url);
		synchronized(servers) {
			servers =  new LinkedList<String>(s.values());
		}

		LOGGER.fine(url);
		LOGGER.fine(servers.toString());
	}
	
	public synchronized static DasuNdtModule getInstance() {
		if (self == null) {
			self = new DasuNdtModule();
			self.setDaemon(true);
			return self;
		} else {
			return self;
		}
	}

	@Override
	public void run() {
		
		loadServerList();
		
		while (!Main.isShuttingDown() && isActive()) {
		 
			TaskEntry taskEntry = null;
			synchronized (toDoServerList) {
				while ((toDoServerList.size()==0) && !Main.isShuttingDown()) {
					try {
						toDoServerList.wait(sleepInterval*1000);
					} catch (InterruptedException e) {
						self = null;
						return;
					}
				}
				
				taskEntry = toDoServerList.remove(0);
			}
			
			if (taskEntry.server.equals("HALT"))
				return;

			while (isPause())
				try {
					Thread.sleep(pauseInterval*1000);
				} catch (InterruptedException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			
			FactNdtProbeResult result = (FactNdtProbeResult) probeIp(taskEntry.server, taskEntry.taskId, taskEntry.target);
			updateNdtResultQueue(result);


	  } // end while active
	
	  self = null;

	}
		
	
	@Override
	public void probeWithPriority(String server, Integer taskId, String target) {
		
		if (server == null)
			//server = servers.getFirst();
			server = getServerFromList();
			
		synchronized (toDoServerList) {
			toDoServerList.add(new TaskEntry(server, taskId, target));
			toDoServerList.notifyAll();
		}		
	}

	@Override
	public void probe(String server, Integer taskId, String target) {

		if (server == null)
			//server = servers.getFirst();
			server = getServerFromList();

		synchronized (toDoServerList) {
			toDoServerList.add(new TaskEntry(server, taskId, target));
			toDoServerList.notifyAll();
		}
	}
	
	protected String getServerFromList() {
	
		Random rand = new Random();

		if (servers.size() == 0)
			loadServerList();
		
		if (servers.size() == 0) {
			return "ndt.iupui.donar.measurement-lab.org";
		}
		
		return servers.get(rand.nextInt( servers.size()));
		

	}
	
	@Override
	protected Object probeIp(String server, Integer taskId, String target) {
	
		if (server == null) {
			//server = servers.getFirst();
			server = getServerFromList();
		}
		
		FactNdtProbeResult ret = new FactNdtProbeResult();
		ret.setTarget(target);

		NetstatBanwidthPerformer nbp = new NetstatBanwidthPerformer(Main.getInterfaceName(), "ndt netstat");
		nbp.loop(1000);
		
    	Long timeBefore = NetstatBWStats.getInstance().getMostRecent().timeRecorded;
    	LOGGER.fine("NETSTAT BEFORE: " + NetstatBWStats.getInstance());
    	
	    try {
	    	
	    	ret = ndtTest.dottcp(server);
	    	ret.interfaceName = Main.getInterfaceName();
	    	
	    	/*
	    	 * sleep for five seconds after ndt finishes
	    	 * to ensure timer already fired at least once
	    	 * before trying to destroy it 
	    	 * in case ndt measurement fails
	    	 */
	    	
	    	Thread.sleep(5000);

		} catch (IOException e) {
			e.printStackTrace();
			
	    	try {
				Thread.sleep(5000);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
	    	
			
	    	ret.failed = true;
			    	
		} catch (InterruptedException e) {
			e.printStackTrace();
			ret.failed = true;
		}
		
		nbp.cancelTimer();
		
		Long timeAfter = NetstatBWStats.getInstance().getMostRecent().timeRecorded;
    	
		ret.c2sRateNetstat = (double) NetstatBWStats.getInstance().getPercentileUploadBetween(timeBefore, timeAfter, 95)/1000;
		ret.s2cRateNetstat = (double) NetstatBWStats.getInstance().getPercentileDownloadBetween(timeBefore, timeAfter, 95)/1000;

		Statistics.getInstance().addNdtResultDasu(ret);

		LOGGER.fine("RESULT: timeBefore: " + timeBefore + " timeAfter: " + timeAfter);
		LOGGER.fine("ndt received:" + ret.getS2cRate());
		LOGGER.fine("Netstat 95th received:" + ret.getS2cRateNetstat());
		LOGGER.fine("Netstat max received: " + NetstatBWStats.getInstance().getFastestDownloadBetween(timeBefore, timeAfter)/1000);
		LOGGER.fine("ndt sent:" + ret.getC2sRate());
		LOGGER.fine("Netstat 95th sent:" + ret.getC2sRateNetstat());
		LOGGER.fine("Netstat max sent: " + NetstatBWStats.getInstance().getFastestUploadBetween(timeBefore, timeAfter)/1000);
		LOGGER.fine("message: " + ret.errorMsg);
		LOGGER.fine("NETSTAT AFTER: " + NetstatBWStats.getInstance());
		
		if (ret.getErrorMsg() != null)
			ret.failed = true;
		
		return ret;
		
	}
	
	private void updateNdtResultQueue(FactNdtProbeResult result) {

		LOGGER.fine("updateNdtResultQueue!!!!");
		
		Long id = System.nanoTime();
		ndtResultTable.put(id, result);
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.HNDT_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
		
		synchronized(servers) {

			if ( ((result.failed) && servers.size() != 0) 
					&& (servers.contains(result.server)) ) {
					servers.remove(result.server);
			}
		}
	}

	public static LinkedList<String> getServers() {
		return servers;
	}
		
	
	@Override
	public void stopModule() {
		
		setActive(false);

		synchronized(toDoServerList) {
			toDoServerList.add(new TaskEntry("HALT", -1, null));
			toDoServerList.notify();
		}
		
		self = null;
	}
	
	
}
