/**
 * Dasu Project
 *
 * File:         DasuPortThrottlingTest.java
 * RCS:          $Id$
 * Description:  OptimalEdge class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 15, 2010 at 07:38:30 PM
 * Language:     Java
 * Package:      edu.northwestern.ono.experiment
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.measurement;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.config.ParameterListener;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadException;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.drools.FactForcedSeedingThroughPortTestResult;
import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.PluginInterface;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 */

public class DasuPortThrottlingTest extends Thread {	
		
	private boolean isActive;
	private PluginInterface pi = Main.getPluginInterface();
	private boolean DEBUG = true;
	private int sleepInterval = 1; //minutes 
	private long serverTCPListenPort;
	private long tempServerTCPListenPort;
	private ParameterListener listener;
	private long testStartTime;
	private boolean success = false;
	
	private Boolean awake = false;
	
	public DasuPortThrottlingTest(long toPort) {
		super("DasuPortThrottlingTest");
		tempServerTCPListenPort = toPort;
	}
	
	public DasuPortThrottlingTest(long toPort, int sleepInterval) {
		super("DasuPortThrottlingTest");
		tempServerTCPListenPort = toPort;
		this.sleepInterval = sleepInterval;
	}
	
	public void setActive(boolean isActive) {
		this.isActive = isActive;
	}
	
	public boolean isActive() {
		return isActive;
	}

	
	private void addListener() {

		listener = new ParameterListener() {
			public void parameterChanged(String name ) {
				
				synchronized(awake) {
					awake = true;
					awake.notify();
				}

				if (DEBUG)
					System.out.println("Port Number changed to: " 
							+ COConfigurationManager.getParameter( name ));
			}
		};

		COConfigurationManager.addParameterListener(
				"TCP.Listen.Port",
				listener
				);

		
	}
	
	@Override
	public synchronized void start() {
		super.start();
	}	
	
	@Override
	public void run() {
		
		try {
			setActive(true);
			testStartTime = Util.currentGMTTime();
				
			
			boolean doit = false;
			for (Download d: pi.getDownloadManager().getDownloads()) {
				if (d.getState() == Download.ST_SEEDING)
					doit = true;
			}
			
			if (doit) {

				
				serverTCPListenPort = (Long) COConfigurationManager
						.getParameter("TCP.Listen.Port");
				
				if (DEBUG) {
					System.out.println("DasuPortThrottlingTest: temporaryServerTCPListenPort: " + 
							"serverTCPListenPort: " + serverTCPListenPort);
					System.out.println("DasuPortThrottlingTest: Changing port number to: " + 
							tempServerTCPListenPort);
				}
				
				COConfigurationManager.setParameter("TCP.Listen.Port",
						tempServerTCPListenPort);
				COConfigurationManager.save();
						
				if (DEBUG)
					System.out.println("DasuPortThrottlingTest: Restarting all downloads...");
				
				for ( Download d : pi.getDownloadManager().getDownloads() ) {
					//if (d.getState() == Download.ST_DOWNLOADING || 
					if(d.getState() == Download.ST_SEEDING) {
						d.stop();
						d.restart();
					}
				}
		
				addListener();
		
				if (DEBUG)
					System.out.println("DasuPortThrottlingTest: going to sleep... ");
	
				synchronized(awake) {
					if (awake == false)
						awake.wait(sleepInterval*60*1000);
				}
				
				if (DEBUG)
					System.out.println("DasuPortThrottlingTest: waking up...");
		
				
				synchronized(awake) {
					if (awake)					
						handlePortModification();
					
					else {
		
						COConfigurationManager.removeParameterListener("TCP.Listen.Port", listener);
						COConfigurationManager.setParameter("TCP.Listen.Port", 
								serverTCPListenPort);
						COConfigurationManager.save();
	
						for ( Download d : pi.getDownloadManager().getDownloads() ) {
							//if (d.getState() == Download.ST_DOWNLOADING || 
							if (d.getState() == Download.ST_SEEDING) {
								d.stop();
								d.restart();
							}
						}
						
						success = true;
						
					}
					
				}
			}
			
			setActive(false);
			
			if (DEBUG)
				System.out.println("DasuPortThrottlingTest: Done");
		
			
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (DownloadException e) {

			COConfigurationManager.removeParameterListener("TCP.Listen.Port", listener);
			COConfigurationManager.setParameter("TCP.Listen.Port", 
					serverTCPListenPort);
			COConfigurationManager.save();
		} finally {
			
			updateTestResultQueue();
			
		}
		
	}
	
	private void handlePortModification() {
		if (DEBUG)
			System.out.println("DasuPortThrottlingTest: Port modified in the middle of test!");

		COConfigurationManager.removeParameterListener("TCP.Listen.Port", listener);
	}
	
	public void updateTestResultQueue() {
		
		Long id = System.nanoTime();
		FactForcedSeedingThroughPortTestResult result = null;
			
		if (!success)
			result = new FactForcedSeedingThroughPortTestResult();
		else 
			result = new FactForcedSeedingThroughPortTestResult(testStartTime,
				serverTCPListenPort, tempServerTCPListenPort,
				success);
		
		synchronized(AMCoordinatorV2.testResultTable) {
			AMCoordinatorV2.testResultTable.put(id, result);
		}
		
		synchronized(AMCoordinatorV2.events) {
			AMCoordinatorV2.events.add(
				new Pair<Long, EventType>(id, EventType.FORCED_SEEDING_THROUGH_PORT_RESULT));
			AMCoordinatorV2.events.notifyAll();
		}
	}

	
	
}
