package edu.northwestern.dasu.measurement;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Field;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.stats.*;

public class InterferenceSimulator {

	static TreeMap<Integer, TreeMap<String, TreeMap<String, Integer>>> events = new TreeMap();
	static int timeTick = 0; 
	static boolean active = false;
	static Set<Integer> didSet = new TreeSet<Integer>();
	static Set<Integer> didSetRst = new TreeSet<Integer>();
	static int maxTick = 0;

	public static int getTimeTick() {
		return timeTick;
	}
	
	public InterferenceSimulator() {
		
	}
	
	public static void increaseTick() {

		System.out.println("\n\n TICK: " + timeTick + "\n\n");

		if (timeTick < maxTick)
			timeTick++;
		else {
			timeTick = 0;
			System.out.println("Simulation Ended, RESTARTING TIMER");
		}
	}
	
	public static boolean isActive() {
		return active;
	}

	public static HashMap<Long, RSTStat> getModifiedRSTStatMap() { 

		HashMap<Long, RSTStat> didTempMap = 
			new HashMap<Long, RSTStat>();
	
		for ( Integer did: didSetRst) {
			RSTStat rststat = getModifiedRSTStat( (long) did );
			didTempMap.put( (long) did, rststat);
		}
		
		return didTempMap;
	}
	
	private static RSTStat getModifiedRSTStat(Long did) {
		
		RSTStat rststat = new RSTStat();
		rststat.timeRecorded += timeTick * 30000; //istats.UPDATE_INTERVAL;
		
		Map<String, Integer> rstSim =
			events.get(timeTick).get( new String("RSTStat").concat(String.valueOf(did)) );

		try {

			for ( String k : rstSim.keySet() ) {
			    Field field;
				field = rststat.getClass().getField(k);
				if (field.getType().toString().indexOf("String") != -1)
					field.set(rststat, Integer.toString(rstSim.get(k)));
				else
					field.set(rststat, rstSim.get(k));
			}
			
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		return rststat;
		
	}
	
	
	public static HashMap<Long, DownloadStatsDynamic> getModifiedDownloadStatsDynamicMap() { 

		HashMap<Long, DownloadStatsDynamic> didTempMap = 
			new HashMap<Long, DownloadStatsDynamic>();
	
		for ( Integer did: didSet) {
			DownloadStatsDynamic dsd = getModifiedDownloadStatsDynamic( (long) did );
			didTempMap.put( (long) did, dsd);
		}
		
		return didTempMap;
	}


	private static DownloadStatsDynamic getModifiedDownloadStatsDynamic(Long did) {
		
		DownloadStatsDynamic dsd = new DownloadStatsDynamic();
		dsd.timeRecorded += timeTick * 30000; //istats.UPDATE_INTERVAL;
		
		Map<String, Integer> dsdSim =
			events.get(timeTick).get( new String("DownloadStatsDynamic").concat(String.valueOf(did)) );

		try {

			for ( String k : dsdSim.keySet() ) {
			    Field field;
				field = dsd.getClass().getField(k);
				if (field.getType().toString().indexOf("String") != -1)
					field.set(dsd, Integer.toString(dsdSim.get(k)));
				else
					field.set(dsd, dsdSim.get(k));
			}
			
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		
		return dsd;
		
	}
	
	public static NetstatSystemStats getModifiedNetstatSystemStats() {
		
		NetstatSystemStats nsss = new NetstatSystemStats();
		nsss.timeRecorded += timeTick * 30000; //istats.UPDATE_INTERVAL;

		Map<String, Integer> nsssSim =
			events.get(timeTick).get("NetstatSystemStats");
		
		System.out.println(nsssSim);

		try {

			for ( String k : nsssSim.keySet() ) {
			    Field field;
				field = nsss.getClass().getField(k);
			    field.set(nsss, nsssSim.get(k));
			}
			
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		return nsss;
	}
	
	public static DynamicLocalPeerStat getModifiedDynamicLocalPeerStat() {
		
		DynamicLocalPeerStat lpsd = new DynamicLocalPeerStat();
		lpsd.currentTime += timeTick * 30000; //istats.UPDATE_INTERVAL;

		Map<String, Integer> lpsdSim =
			events.get(timeTick).get( "DynamicLocalPeerStat");

		try {

			for ( String k : lpsdSim.keySet() ) {
			    Field field;
				field = lpsd.getClass().getField(k);
			    field.set(lpsd, lpsdSim.get(k));
			}
			
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}

		return lpsd;
		
	}
	
	public static boolean loadTimeline() {
		
		ClassLoader classLoader = Main.getPluginInterface().getPluginClassLoader();
		
		if (classLoader instanceof URLClassLoader) {
			URLClassLoader current = (URLClassLoader) classLoader;

			URL url = current.findResource("timeline.txt");
	        
			try {
				
				DataInputStream in = new DataInputStream(url.openStream());
		        BufferedReader br = new BufferedReader(new InputStreamReader(in));
		        
		        boolean start = false;
		        String line;
		        while ((line = br.readLine()) != null) {
		      
		    	  if (line.contains("startfile")) {
		    		  start = true;
		    		  continue;
		    	  }
		    	  
		    	  if (start) {
		    		  if (line.isEmpty())
		    			  continue;
		    		  
		    		  if (line.contains("endfile")) {
		    			  completeTimeline();
		    			  return true;
		    		  }
		    		  
		    		  String[] tokens = line.split(",");
		    			  
	    			  int time = Integer.parseInt(tokens[0].trim());
	    			  String type = tokens[1].trim();
	    			  
	    			  if ( !events.containsKey(time) ) 
	    				  events.put(time, new TreeMap());
	    			  
	    			  if (!events.get(time).containsKey(type)) 
	    				  events.get(time).put(type, new TreeMap());
	    			  
		    		  for (int i=2; i < tokens.length; i++) {

		    			  String[] pair = tokens[i].split("=");
		    			  String field = pair[0].trim();
		    			  Integer value = Integer.parseInt(pair[1].trim());
		    			  
		    			  if ( type.contains("DownloadStatsDynamic") && i==2) {
		    				  didSet.add( Integer.parseInt(pair[1]) );
		    				  events.get(time).remove(type);
		    				  type = type.concat(pair[1]);
		    				  events.get(time).put(type, new TreeMap());
		    			  }
		    			  
		    			  if ( type.contains("RSTStat") && i==2) {
		    				  didSetRst.add( Integer.parseInt(pair[1]) );
		    				  events.get(time).remove(type);
		    				  type = type.concat(pair[1]);
		    				  events.get(time).put(type, new TreeMap());
		    			  }
		    			  
		    			  events.get(time).get(type).put(field,value);
		    			  
		    		  }
		    		  		    		  
		    	  }
		    	  
		      }
		      
		    } catch (FileNotFoundException e) {
			      e.printStackTrace();
			    } catch (IOException e) {
			      e.printStackTrace();
			    }
		}
		else {
		
			try {
				throw (new Exception("failed to load timeline.txt (simulation file) from jars"));
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
				
		return false;
	}
	
	@SuppressWarnings("unchecked")
	private static void completeTimeline() {

		HashMap<String, Integer> lastIndex = new HashMap();
		for ( String type : events.get(0).keySet() )
			lastIndex.put(type, 0);

		String mt = events.keySet().toArray()[events.size()-1].toString();
		maxTick = Integer.parseInt(mt);

		for (int i=1; i <= maxTick; i++) {

			if ( !events.containsKey(i))
				events.put(i, new TreeMap());
			
			Iterator iter = events.get(0).keySet().iterator();
			while ( iter.hasNext() ) {
				
				String type = (String) iter.next();
				 
				if ( !events.get(i).containsKey(type) ) {
					
					if ( type.contains("NetstatSystemStats"))
						events.get(i).put(type, 
								(TreeMap<String, Integer>) randomizeRst( events.get(i-1).get(type)).clone() );
								//(TreeMap<String, Integer>) randomizeRst( events.get( lastIndex.get(type) ).get(type)).clone() );
					
					if ( type.contains("DynamicLocalPeerStat"))
						events.get(i).put(type, 
								(TreeMap<String, Integer>) randomizeLpsd( events.get(i-1).get(type)).clone() );
								//(TreeMap<String, Integer>) randomizeLpsd( events.get( lastIndex.get(type) ).get(type)).clone() );

					if ( type.contains("DownloadStatsDynamic"))
						events.get(i).put(type, 
								//(TreeMap<String, Integer>) randomizeDsd( events.get(0).get(type)).clone() );
								(TreeMap<String, Integer>) randomizeDsd( events.get( lastIndex.get(type) ).get(type)).clone() );

					if ( type.contains("RSTStat"))
						events.get(i).put(type, 
								//(TreeMap<String, Integer>) randomizeDsd( events.get(0).get(type)).clone() );
								(TreeMap<String, Integer>) randomizeRSTStat( events.get( lastIndex.get(type) ).get(type)).clone() );

				
				}
				else {
					lastIndex.put(type, i);

				}
				
			}
			
		}
		
		active = true;
		
	}

	
	private static TreeMap<String, Integer> randomize(Map<String, Integer> map, String[] fields) {

		TreeMap<String, Integer> retMap = new TreeMap<String, Integer>();

		for ( String f : fields ) {
			int prev = map.get(f);
			prev *= ( ( 95 + Math.random() * (105-95))/100 );
			retMap.put(f, prev); 
		}

		return retMap;

	}
		
	private static TreeMap<String, Integer> randomizeLpsd(Map<String, Integer> map) {

		String[] fields = new String[]{"uploadRate", "downloadRate", "maxUploadRate", 
				"maxDownloadRate", "numLeechingTorrents", "numSeedingTorrents" };
		
		String[] staticFields = new String[]{"bytesSentNetstat", "bytesReceivedNetstat", 
				"bytesSentLpsd", "bytesReceivedLpsd"};

		TreeMap<String, Integer> lpsd = randomize(map, fields);		
		
		for (int i=0; i < staticFields.length; i++)
			lpsd.put(staticFields[i], map.get(staticFields[i])*110/100);

		return lpsd;

	}
	
	private static TreeMap<String, Integer> randomizeRst(Map<String, Integer> map) {

		String[] fields = new String[]{"rst", "numConnections", "passiveOpens", 
				"activeOpens", "failedConns"}; 

		String[] staticFields = new String[]{"bytesSent", "bytesReceived"};
		
		TreeMap<String, Integer> rstat = new TreeMap<String, Integer>();
		
		for ( String f : fields )
			rstat.put(f, map.get(f) + timeTick + 1);
		
		for (int i=0; i < staticFields.length; i++)
			rstat.put(staticFields[i], map.get(staticFields[i])*110/100);
		
		return rstat;

	}
	
	private static TreeMap<String, Integer> randomizeDsd(Map<String, Integer> map) {

		String[] fields = new String[]{"numPeers", "downloadRate", "uploadRate", 
				"numKnownNonSeeds", "numKnownSeeds"};
			
		String[] staticFields = new String[]{"downloadId", "availability", 
				"completed", "shareRatio", "state", "downloadName"};

		TreeMap<String, Integer> dsd = randomize(map, fields);
		
		for (int i=0; i < staticFields.length; i++)
			dsd.put(staticFields[i], map.get(staticFields[i]));
		
		return dsd;

	}
	
	private static TreeMap<String, Integer> randomizeRSTStat(Map<String, Integer> map) {

		String[] fields = new String[]{"numRst"};
			
		String[] staticFields = new String[]{"downloadId", "peerIp", 
				"type"};

		TreeMap<String, Integer> dsd = randomize(map, fields);
		
		for (int i=0; i < staticFields.length; i++)
			dsd.put(staticFields[i], map.get(staticFields[i]));
		
		return dsd;

	}
	
	public static void printEvents() {
		
		Iterator iterTime = events.keySet().iterator();
		
		while (iterTime.hasNext()) {

			Integer time = (Integer) iterTime.next();
			Iterator iterType = events.get(time).keySet().iterator();
			
			while (iterType.hasNext()) {
				
				String type = (String) iterType.next();
				Iterator iterEntry = events.get(time).get(type).keySet().iterator();
				
				while (iterEntry.hasNext()) {
					
					String field = (String) iterEntry.next();
					System.out.println(time + " " + type + " " + field + " " + 
							events.get(time).get(type).get(field));
				}
			}
		}
	}
	
	
	
}
