/**
 * Dasu Project
 *
 * File:         ProbeTask.java
 * RCS:          $Id$
 * Description:  ProbeTask class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 31, 2010 at 5:23:15 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.measurement;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The ProbeTask class ...
 */
public class ProbeTask {

	public Integer taskId;
	public Integer priority = 0;	/*highest priority*/
	public Long startTime = -1L; 	/*right away*/
	public Long endTime = -1L;		/*right away*/
	public List<String> ipsToProbe = null;
	public ProbeType probeType = null;
	public Map<String, Object> params = null;
	
	public TaskType taskType = TaskType.BULK;
	public SourceType sourceType = 
		SourceType.AME;			/* to redirect output for task by the AMC*/	
	
	public String target = null;
		 
	public ProbeTask(Integer tid, List<String> ipsToProbe, ProbeType probeType) {
		this.taskId = tid;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);	
		this.probeType = probeType;
	}
	
	public ProbeTask(Integer tid, Integer prior, Long st, Long et, List<String> ipsToProbe, ProbeType probeType) {
		this.taskId = tid;
		this.priority = prior;
		this.startTime = st;
		this.endTime = et;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);
		this.probeType = probeType;
	}
		
	public ProbeTask(Integer tid, Integer prior, Long st, Long et, List<String> ipsToProbe, ProbeType probeType, TaskType taskType, SourceType sourceType) {
		this.taskId = tid;
		this.priority = prior;
		this.startTime = st;
		this.endTime = et;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);
		this.probeType = probeType;
		this.taskType = taskType;
		this.sourceType = sourceType;
	}
	
	public ProbeTask(Integer tid, Integer prior, Long st, Long et, List<String> ipsToProbe, ProbeType probeType, TaskType taskType, SourceType sourceType, Map<String, Object> params) {
		this.taskId = tid;
		this.priority = prior;
		this.startTime = st;
		this.endTime = et;
		this.ipsToProbe = new ArrayList<String>(ipsToProbe);
		this.probeType = probeType;
		this.taskType = taskType;
		this.sourceType = sourceType;
		this.params = params;
	}


	@Override
	public String toString() {
		
		return new String("taskId: " + taskId + "\npriority: " + priority + 
				"\nstartTime: " + new Date(startTime) + "\nendTime: " + new Date(endTime) +
				"\nipsToProbe: " + ipsToProbe + "\nProbeType: " + probeType + 
				"\nTaskType: " + taskType + "\nSourceType: " + sourceType + "\nTarget: " + target);
		
	}
	
	public Integer size() {
		if (ipsToProbe != null)
			return ipsToProbe.size();
		else
			return 0;
	}

	public boolean equals(Object obj) {

		if (obj == null) {
	        return false;
	    }
	    //if (!obj.getClass().equals(ProbeTask.class)) {
		if (!(obj instanceof ProbeTask)) {
	        return false;
	    }
	    
	    return taskId.equals(((ProbeTask) obj).taskId);
	}

	public int hashCode()
	{
		return taskId;
	}
	
	public void setTarget(String target) {
		this.target = target;
	}
	
	public String getTarget() {
		return target;
	}
}
