/**
 * News Project
 *
 * File:         DownloadStats.java
 * RCS:          $Id$
 * Description:  DownloadStats class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 10, 2007 at 1:39:15 PM
 * Language:     Java
 * Package:      edu.northwestern.news.stats
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.stats;


import java.util.ArrayList;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadListener;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.stats.DownloadStats.DownloadStatsDynamic;
import edu.northwestern.dasu.util.Util;

/**
 * @author Mario Sanchez &lt;msanchez@u.northwestern.edu&gt;
 * 
 * downloadStatSummary, (every 30 minutes), localId, downloadId (for this session), 
 * currentTime (in ms), downloadingTime (time spent downloading), 
 * seedingTime (time spent seeding), maxDownloadRateLimit, maxUploadLimit, 
 * totalUploaded (of bytes for this download), totalDownloaded (of bytes for thisdownload),
 * size (of the download), pieceSize (piece size for download)
 */

public class DownloadStatsSummary{

	public long timeRecorded = Util.currentGMTTime();
	public Download download;
	public long downloadingTime;
	public long seedingTime;
	public int maxDownloadRateLimit;
	public int maxUploadLimit;
	public long totalUploaded;
	public long totalDownloaded;
	public long size;
	public int downloadId;
	public int priority;
	public long pieceSize;
	public String downloadName;	

	public DownloadStatsSummary(Download download) {

		this.download = download;
		this.pieceSize = (download.getTorrent()!=null) ? (download.getTorrent().getPieceSize()) : -1;
		this.size = (download.getTorrent()!=null) ? roundSize(download.getTorrent().getSize()) : -1;
		this.downloadName = download.getName();
		this.downloadId = Main.getId(download);
		
		this.downloadingTime = download.getStats().getSecondsDownloading();
		this.seedingTime = download.getStats().getSecondsOnlySeeding();
		this.maxDownloadRateLimit = download.getMaximumDownloadKBPerSecond()/1024;
		this.maxUploadLimit = download.getUploadRateLimitBytesPerSecond();
		this.totalUploaded = download.getStats().getUploaded();
		this.totalDownloaded = download.getStats().getDownloaded();
		this.priority = download.getIndex();
		
	}
	
	/**
	 * Rounds the size of the download.
	 * 
	 * @param dlSize
	 * @return
	 */
	public static long roundSize(long dlSize) {
		if (dlSize < 10000) return dlSize;
		else if (dlSize < 100000) return 1000* (dlSize/1000);
		else if (dlSize < 1000000) return 10000* (dlSize/10000); 
		else if (dlSize < 10000000) return 100000* (dlSize/100000); 
		else if (dlSize < 100000000) return 1000000* (dlSize/1000000);
		else return 10000000* (dlSize/10000000); 
	}

	
}
