package edu.northwestern.dasu.timer;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import edu.northwestern.dasu.measurement.CronjobType;

public class CronjobEntry implements Comparable, Serializable{

	private static final long serialVersionUID = 1221642255540144145L;
	
	private Set<Integer> days = new HashSet<Integer>();
	private Set<Integer> hours = new HashSet<Integer>();
	private String commandToExecute = new String();
	private Long lastExecuted = -1L;
	private Boolean active = true;
	private String id = new String();
	private CronjobType cronjobType = CronjobType.ONCE_PER_DAY;
	
	public CronjobEntry() {
		
	}
	
	public CronjobEntry(String id, Set<Integer> hours, Set<Integer> days, 
			String commandToExecute, CronjobType cronjobType) {
		this.id = id;
		this.hours = hours;
		this.days = days;
		this.commandToExecute = commandToExecute;
		this.cronjobType = cronjobType;
	}
	
	private int compareSets(Set<Integer> s1, Set<Integer> s2) {

		if (s1.containsAll(s2) 
	    		&& s2.containsAll(s1))
	    	return 0;
		
		else return 1;

	}
	
	@Override
	public String toString() {
		return id + ", " + days.toString() + ", " + hours.toString() +
			", " + lastExecuted + ", " + active + ", " + commandToExecute + ", " +
			cronjobType;
	}
	
	public int compareTo(Object o) {
	    if (!(o instanceof CronjobEntry))
	    	return 1;
	    
	    CronjobEntry anotherEntry = (CronjobEntry) o;
	    
	    return (anotherEntry.commandToExecute.compareTo(this.commandToExecute) +
	    		compareSets(anotherEntry.days, this.days) +
	    		compareSets(anotherEntry.hours, this.hours) + 
	    		anotherEntry.cronjobType.compareTo(cronjobType));
	}
	
	public boolean equals(Object o) {
		if (compareTo(o) == 0)
			return true;
		else 
			return false;
	}

	public Set<Integer> getDays() {
		return days;
	}

	public Set<Integer> getHours() {
		return hours;
	}

	public String getCommandToExecute() {
		return commandToExecute;
	}

	public Long getLastExecuted() {
		return lastExecuted;
	}

	public Boolean isActive() {
		return active;
	}
	
	public String getId() {
		return id;
	}

	public CronjobType getCronjobType() {
		return cronjobType;
	}
	
	public void setDays(Set<Integer> days) {
		this.days = days;
	}

	public void setHours(Set<Integer> hours) {
		this.hours = hours;
	}

	public void setCommandToExecute(String commandToExecute) {
		this.commandToExecute = commandToExecute;
	}

	public void setLastExecuted(Long lastExecuted) {
		this.lastExecuted = lastExecuted;
	}

	public void setActive(Boolean active) {
		this.active = active;
	}
	
	public void setId(String id) {
		this.id = id;
	}

	public void setCronjobType(CronjobType cronjobType) {
		this.cronjobType = cronjobType;
	}
	
}