package edu.northwestern.dasu.ui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import edu.northwestern.dasu.util.Pair;

public class BarPlot implements Serializable {
	
	private HashMap<String, ArrayList<Pair<String, Float>>> series = new HashMap<String, ArrayList<Pair<String, Float>>>();
	private HashMap<String, Object> options = new HashMap<String, Object>();
	
	private String id = "";
	private String xLabel = "";
	private String yLabel = "";	
	
	public BarPlot(){
	}
	
	public BarPlot(String id, String plotTitle, Collection<String> xValues, Collection<Float> yValues, 
			String xLabel, String yLabel) {
		
		this.id = id;
		ArrayList<String> xAxis = genXaxis(xValues, yValues);		
		ArrayList<Pair<String, Float>> dataValues = new ArrayList<Pair<String, Float>>();
		
		if ( xAxis.size() != yValues.size())
			return;
		
		Iterator<String> xIter = xAxis.iterator();
		Iterator<Float> yIter = yValues.iterator();
		while (xIter.hasNext())
			dataValues.add(new Pair<String, Float>(xIter.next(), yIter.next()));
		
		if (dataValues.size() > 0) {
						
			//this.points = dataValues;
			
			if (plotTitle == null)
				plotTitle = "default";
			
			if (xLabel != null)
				this.xLabel = xLabel;
			
			if (yLabel != null)
				this.yLabel = yLabel;			
			
			series.put(plotTitle, dataValues);			
			setXorder(xAxis);
		}
	}
	
	public BarPlot(String id, String plotTitle, String[] xValues, Float[] yValues, 
			String xLabel, String yLabel) {
		
		this(id, plotTitle, xValues != null ? Arrays.asList(xValues) : null, 
				Arrays.asList(yValues), xLabel, yLabel);
		
		System.out.println(id + " " + plotTitle + " " + xValues + " " + Arrays.asList(yValues) + " " + xLabel + " " + yLabel);
		
	}
	
	public BarPlot(ArrayList<Pair<String,Float>> dataValues,String plotTitle, String xlabel, 
			String ylabel){

		if (plotTitle != null)
			plotTitle = "default";

		if (xlabel == null)
			this.xLabel = xlabel;
		
		if (ylabel != null)
			this.yLabel = ylabel;
		
		series.put(plotTitle, dataValues);
		ArrayList<String> xAxis = new ArrayList<String>();
		for ( Pair<String, Float> p : dataValues) 
			xAxis.add(p.getKey());
		
		setXorder(xAxis);
		
	}
	
	public ArrayList<String> genXaxis(Collection<String> xValues, Collection<Float> yValues) {
	
		ArrayList<String> xAxis = new ArrayList<String>();
		
		if ( xValues == null ) {
			for (Integer i=0; i<yValues.size(); i++)
				xAxis.add(i.toString());
		}
		else
			xAxis = new ArrayList<String>(xValues);
		
		return xAxis;
	}
	
	public void setXorder( List<String> order ) {		
		options.put("xOrder", new ArrayList<String>(order));
	}
	
	public ArrayList<String> getXorder() {	
		return (ArrayList<String>) options.get("xOrder");
	}
	
	public boolean addSeries(String title, Collection<Float> yValues) {
		
		ArrayList<Pair<String, Float>> points = 
			(ArrayList<Pair<String, Float>>) series.values().toArray()[0];
		
		if ( yValues.size() != points.size()) {	
			System.out.println("series is not the same length as original");
			return false;
		}
		
		ArrayList<String> xAxis = new ArrayList<String>();
		for ( int i=0; i < points.size(); i++)
			xAxis.add(points.get(i).getKey());
		
		ArrayList<Pair<String, Float>> dataValues = new ArrayList<Pair<String, Float>>();
		
		Iterator<String> xIter = xAxis.iterator();
		Iterator<Float> yIter = yValues.iterator();
		while (xIter.hasNext())
			dataValues.add(new Pair<String, Float>(xIter.next(), yIter.next()));
		
		if (dataValues.size() > 0) {
			series.put(title, new ArrayList<Pair<String, Float>>());
			series.get(title).addAll(dataValues);						
			return true;
		}

		return false;		
		
	}
	
	public boolean addSeries(String title, Float[] yValues) {		
		return addSeries(title, Arrays.asList(yValues));
	}
	
	private ArrayList<Object> genSeries(ArrayList<Pair<String, Float>> points) {

		ArrayList<Object> plot = new ArrayList<Object>();
		for (int i=0; i < points.size(); i++) {
			Pair<String, Float> p = points.get(i);
			ArrayList<Object> point = new ArrayList<Object>();

			point.add(i);			
			point.add(p.getValue());
			plot.add(point);
		}

		return plot;
	}
	
	private ArrayList<Object> genSeriesMap(String name, 
			ArrayList<Pair<String, Float>> points) {
		
		ArrayList<Object> ret = new ArrayList<Object>();
		
		HashMap<String, Float> plotMap = new HashMap<String, Float>();
		for ( Pair<String, Float> p : series.get(name))
			plotMap.put(p.getKey(), p.getValue());
		
		ret.add(name);
		ret.add(plotMap);

		return ret;
	}
		
	public ArrayList<Object> generatePlot() {

		ArrayList<Object> w1 = new ArrayList<Object>();
		ArrayList<Object> w2 = new ArrayList<Object>();
		
		for ( String name : series.keySet() )
			w1.add(genSeriesMap(name, series.get(name)));
				
		w2.add(w1);
		w2.add(options);		
				
		return w2;
	}
	
	public ArrayList<Pair<String, Float>> getSeries(String seriesName) {

		if (seriesName != null) {
			if ( series.containsKey(seriesName))
				return series.get(seriesName);
		}
		return null;
	}
	
	public Collection<String> getSeriesNames() {
		return series.keySet();
	}
	
	public void addOption( String name, Object val) {
		options.put(name, val);
	}
	
	public Collection<String> getOptions() {
			
		ArrayList<String> ret = new ArrayList<String>();
		for ( String key : options.keySet()) {
			if (!key.equals("xOrder"))
				ret.add(key);
		}
		
		return ret;
	}
	
	public void setXLabel(String xlabel){
		this.xLabel = xlabel;
	}
	
	public void setYLabel(String ylabel){
		this.yLabel = ylabel;
	}
	
	public String getXLabel(){
		return xLabel;
	}
	
	public String getyLabel(){
		return yLabel;
	}	

	public String getxLabel() {
		return xLabel;
	}

	public void setxLabel(String xLabel) {
		this.xLabel = xLabel;
	}

	public void setyLabel(String yLabel) {
		this.yLabel = yLabel;
	}

	public HashMap<String, ArrayList<Pair<String, Float>>> getSeries() {
		return series;
	}

	public void setSeries(HashMap<String, ArrayList<Pair<String, Float>>> series) {
		this.series = series;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getYLabel() {
		return yLabel;
	}
	
}
