package edu.northwestern.dasu.util;

import java.net.URL;
import java.net.URLClassLoader;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.X509EncodedKeySpec;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import edu.northwestern.dasu.Main;

public class DigitalSignatureManager {
	
	private static PublicKey pubKey = loadKey();
	static String sigAlg = "SHA1withDSA";
	private static boolean DEBUG = false;

	public static PublicKey loadKey() {
			
		 ClassLoader classLoader = Main.getPluginInterface().getPluginClassLoader();
			
		 if (classLoader instanceof URLClassLoader) {
			 URLClassLoader current = (URLClassLoader) classLoader;
			
			 URL url = current.findResource("public-DSA.key");
			 if (url != null) {
			    
				 try {
			
				     InputStream is = new DataInputStream(url.openStream());

				     long length = 1024;
				     byte[] bytes = new byte[(int)length];
				  
				     int offset = 0; 
				     int numRead = 0;
				     while (offset < bytes.length
				             && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
				          offset += numRead;
				     }

				     return getPublicKeyFromBytes(bytes);
		
				} catch (IOException e) {
		
					try {
						throw (new Exception("No Interferece Detection Rules file from jars"));
					} catch (Exception e1) {
						e1.printStackTrace();
					}
											
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		 }
		 
		 return null;

	 }
		  
	 private static boolean verify(String datafile, byte[] sigbytes) throws Exception {

		 Signature sig = Signature.getInstance(sigAlg);
		 sig.initVerify(pubKey);
		 FileInputStream fis = new FileInputStream(datafile);
		 byte[] dataBytes = new byte[1024];
		 int nread = fis.read(dataBytes);
		 while (nread > 0) {
			 sig.update(dataBytes, 0, nread);
			 nread = fis.read(dataBytes);
		 };
		 return sig.verify(sigbytes);
	 }

	 private static char[] hexChars = {
		  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
		  'A', 'B', 'C', 'D', 'E', 'F'};

	 public static String byteArray2Hex(byte[] ba) {
		  StringBuffer sb = new StringBuffer();
		  for (int i = 0; i < ba.length; i++) {
		    int hbits = (ba[i] & 0x000000f0) >> 4;
		    int lbits = ba[i] & 0x0000000f;
		    sb.append("" + hexChars[hbits] + hexChars[lbits] + "");
		  }
		  return sb.toString();
	 }
	 
	 public static byte[] readSignatureFile(String filename) throws IOException {
		 
		 FileInputStream sigfis = new FileInputStream(filename);
		 byte[] sigToVerify = new byte[sigfis.available()]; 
		 sigfis.read(sigToVerify);
		 sigfis.close();
		 return sigToVerify;
		 
	 }
		 
	 public static byte[] getBytesFromFile(String filename) throws IOException {
		  
		  File file = new File(filename);
		  if (!file.isFile()) 
		      throw new IOException("Not a valid file to read "+file.getName());
			  
	      InputStream is = new FileInputStream(file);
	      long length = file.length();
	      byte[] bytes = new byte[(int)length];
	  
	      int offset = 0;
	      int numRead = 0;
	      while (offset < bytes.length
	             && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
	          offset += numRead;
	      }
	  
	      if (offset < bytes.length)
	          throw new IOException("Could not completely read file "+file.getName());
	  
	      is.close();
	      return bytes;
	 }
	 
	 public static PublicKey getPublicKeyFromFile(String pubKeyFilename) throws Exception {
		 
		 X509EncodedKeySpec pubKeySpec = 
			 new X509EncodedKeySpec(getBytesFromFile(pubKeyFilename));
		 KeyFactory keyFactory = KeyFactory.getInstance("DSA");
		 PublicKey pubk = keyFactory.generatePublic(pubKeySpec);

		 return pubk;

	 }
	 
	 public static PublicKey getPublicKeyFromBytes(byte[] keyBytes) throws Exception {
		 
		 X509EncodedKeySpec pubKeySpec = 
			 new X509EncodedKeySpec(keyBytes);
		 KeyFactory keyFactory = KeyFactory.getInstance("DSA");
		 PublicKey pubk = keyFactory.generatePublic(pubKeySpec);

		 return pubk;

	 }
	 
	 public static boolean verify(String dataFile, String signatureFile) {

		 boolean result = false;
		 try {
			 
			 byte[] sigbytes;
			 sigbytes = readSignatureFile(signatureFile);
			 result = verify(dataFile, sigbytes);
			 
			 if (DEBUG) {
				 System.out.println("\nverify: " + dataFile + "\n" + signatureFile);
				 System.out.println("Signature(in hex):: " +
						 byteArray2Hex(sigbytes));
				 System.out.println("Signature Verification Result = " + result + "\n");

			 }

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		 
		 return result;

	 }
	 
		
}

