/**
 * Dasu Project
 *
 * File:         Util.java
 * RCS:          $Id$
 * Description:  Util class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Aug 28, 2006 at 11:21:35 AM
 * Language:     Java
 * Package:      edu.northwestern.news.util
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2006, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.dasu.util;



import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.nio.LongBuffer;
import java.nio.ShortBuffer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.TimeZone;
import java.util.Map.Entry;
import java.util.logging.Logger;

import org.xbill.DNS.DClass;
import org.xbill.DNS.ExtendedFlags;
import org.xbill.DNS.Message;
import org.xbill.DNS.Name;
import org.xbill.DNS.Record;
import org.xbill.DNS.ReverseMap;
import org.xbill.DNS.Section;
import org.xbill.DNS.SimpleResolver;
import org.xbill.DNS.TSIG;
import org.xbill.DNS.TextParseException;
import org.xbill.DNS.Type;



/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The Util class provides utility functions for the package.
 */
public class Util {
	
	private final static Logger LOGGER = Logger.getLogger(Util.class.getName());
	    
	static BufferedReader in;

	private static Util self;
    
	public static long dasuTimeOffset = 0;
	public static long sntpOffset = 0;
	
	private static boolean failed = false;
    
    public static interface PingResponse {
    	public void response(double rtt);
    }

    @SuppressWarnings("deprecation")
	public static String currentGMTDayHour() {
    	Date d = new Date(currentGMTTime());
    	String dayHour = d.getDay() + "-" + d.getHours(); 
    	return dayHour;
    }
    
    public static long currentGMTTime(){
    	return System.currentTimeMillis()-dasuTimeOffset;
    	//return currentDasuTime();
    	//return System.currentTimeMillis()-TimeZone.getDefault().getRawOffset();
    } 
    
    public static long currentDasuTime(){
    	return currentGMTTime() + TimeZone.getDefault().getRawOffset();
    }
    
    public static long currentGMTTimeV2() {
    	
    	return System.currentTimeMillis() + sntpOffset;
    }
    
    public static long currentDasuTimeV2() {
    	
    	return System.currentTimeMillis() + dasuTimeOffset;
    	
    }
    
    public static long fromDasuTime(long dasuTime){
    	return dasuTime+dasuTimeOffset;
    }
    
    public static void setDasuTimeOffset( long offset ){
    	
    	if (offset == -1)
    		failed = true;
    	
    	dasuTimeOffset = offset + TimeZone.getDefault().getRawOffset();
    	sntpOffset = offset;
    }
    
    public static long getDasuSntpTimeOffset() {
    	return sntpOffset;
    }
    
    public static long getDasuTimeOffset() {
    	return dasuTimeOffset;
    }
    
    public static long fromGMTTime(long gmtTime){
    	return gmtTime+TimeZone.getDefault().getRawOffset();
    }
    
    public synchronized static Util getInstance(){

	    	if (self == null){
	    		self = new Util();
	    	}
    	
    	return self;
    }
    
    public static boolean isFailed() {
    	return failed;
    }

   

    /**
     * @param newEdge
     * @return
     */
    public static String getClassCSubnet(String ipaddress) {
        // TODO Auto-generated method stub
        return ipaddress.substring(0, ipaddress.lastIndexOf("."));
    }

	public static byte[] convertLong(long l) {
		byte[] bArray = new byte[8];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        LongBuffer lBuffer = bBuffer.asLongBuffer();
        lBuffer.put(0, l);
        return bArray;
	}
	
	public static  byte[] convertShort(short s) {
		byte[] bArray = new byte[2];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        ShortBuffer sBuffer = bBuffer.asShortBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertInt(int s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        IntBuffer sBuffer = bBuffer.asIntBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static  byte[] convertFloat(float s) {
		byte[] bArray = new byte[4];
        ByteBuffer bBuffer = ByteBuffer.wrap(bArray);
        bBuffer.order(ByteOrder.LITTLE_ENDIAN);
        FloatBuffer sBuffer = bBuffer.asFloatBuffer();
        sBuffer.put(0, s);
        return bArray;
	}
	
	public static long byteToLong(byte data[]){
		ByteBuffer bBuffer = ByteBuffer.wrap(data);
		bBuffer.order(ByteOrder.LITTLE_ENDIAN);
		LongBuffer  lBuffer = bBuffer.asLongBuffer();
		return lBuffer.get();
	}

	public static byte[] convertStringToBytes(String key) {
		try {
			return key.getBytes("ISO-8859-1");
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	public static String convertByteToString(byte[] value){
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			baos.write(value);
		
			return baos.toString("ISO-8859-1");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}
	
	public static HashMap<Integer, BigDecimal> findUnionProbability(
			int numComb, HashMap<Integer, Double> perPeerProbs, 
			int maxPeersForProb) {

		HashMap<Integer,BigDecimal> indepProbs = 
			new HashMap<Integer, BigDecimal>();

		double notProbs = -1.0;
		for (Entry<Integer, Double> ent : perPeerProbs.entrySet()){
			if (notProbs<0) notProbs = 1-ent.getValue();
			else notProbs*=1-ent.getValue();
		}

		// store independent probabilities
		indepProbs.put(0, new BigDecimal(1-notProbs));

		if (perPeerProbs.size()>maxPeersForProb){
			double probs[] = new double[perPeerProbs.size()];
			int j = 0;

			for (Entry<Integer, Double> ent2 : perPeerProbs.entrySet()){
				probs[j]+= ent2.getValue();
				j++;
			}

			Arrays.sort(probs);
			double minValue = probs[perPeerProbs.size()-maxPeersForProb];
			HashMap<Integer, Double> newProbs = 
				new HashMap<Integer, Double>();
			double temp = 0;
			for (Entry<Integer,Double> ent : 
				perPeerProbs.entrySet()){


				temp= ent.getValue();

				if (temp>=minValue){
					newProbs.put(ent.getKey(), ent.getValue());
				}
			}
			// switch them over
			perPeerProbs = newProbs;
		}

		// first find all the possible terms, i.e., the unique combinations 
		// of (index+1) nodes seeing problems at the same time

		ArrayList<BigDecimal> terms = new ArrayList<BigDecimal>();
		for (Entry<Integer, Double> ent : 
			perPeerProbs.entrySet()){
			terms.add(new BigDecimal(ent.getValue(), MathContext.UNLIMITED));
		}

//		// now get the union probabilities
		BigDecimal firstParts = new BigDecimal(0);


		int n = terms.size();

		BigDecimal prob[] = new BigDecimal[n];
		for (int i = 0; i < n; i++) prob[i] = 
			terms.get(i);
		BigDecimal psum[] = new BigDecimal[n];
		BigDecimal term[] = new BigDecimal[n];
		BigDecimal prun;
		for (int j=0; j < n; j++){
			psum[j] = new BigDecimal(0, MathContext.UNLIMITED);
			term[j] = new BigDecimal(0, MathContext.UNLIMITED);
		}
		term[n-1] = new BigDecimal(1, MathContext.UNLIMITED);
		prun = new BigDecimal(0, MathContext.UNLIMITED);

		for (int j = 0; j < n; j++){
			term[0] = term[0].add(prob[j], MathContext.UNLIMITED);
			term[n-1] = term[n-1].multiply(prob[j], MathContext.UNLIMITED);
			psum[j] = prob[j];
		}

		if (n-2>0) {

			int i2 = n-1;
			int j2 = n;
			for (int i = 1; i < i2; i++) {
				j2--;
				int k2 = j2 +1;
				for (int j = 0; j < j2; j++){
					psum[j] = new BigDecimal(0, MathContext.UNLIMITED);
					int k1 = j+1;
					for (int k = k1; k < k2; k++){
						psum[j] = psum[j].add(prob[j].multiply(psum[k], MathContext.UNLIMITED)
								, MathContext.UNLIMITED);
					}
					term[i] = term[i].add(psum[j], MathContext.UNLIMITED);
				}
			}

		}


		for (int offset = 1; offset < numComb; offset++){
			int sign = -1;
			prun = new BigDecimal(0, MathContext.UNLIMITED);
			for (int j = offset; j < n; j++){								
				sign*=-1;
//				if (j==index2) continue;
prun=prun.add((new BigDecimal(sign)).multiply(term[j], MathContext.UNLIMITED)
		, MathContext.UNLIMITED);
			}
//			System.out.println(prun);

			indepProbs.put(offset, prun);
		}


		return indepProbs;
	}

	public static boolean isValidIp(final String ip) {
		return ip.matches("^[\\d]{1,3}\\.[\\d]{1,3}\\.[\\d]{1,3}\\.[\\d]{1,3}$");
	}
	
	public static String convertIntToIp(int ip) {
		int temp;
		String out = "";
		for (int i = 0; i < 4; i++){
			temp = ip;
			temp >>= (8*(3-i));
			temp &= 0xFF;
			out+=temp+".";
		}
		return out.substring(0, out.length()-1);
	}

	public static int convertIpToInt(String ipv4) {
		String parts[] = ipv4.split("\\.");
		if (parts.length!=4) return -1;
		long ip = 0;
		long temp;
		for (int i = 0; i < parts.length; i++){
			temp = Integer.parseInt(parts[i]);
			temp <<= (8*(3-i));
			ip+=temp;
		}
		return (new Long(ip)).intValue();
	}

	public static String join(String string, Object[] array) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < array.length; i++){
			sb.append(array[i]);
			if (i<array.length-1) sb.append(string);
		}
		return sb.toString();
	}
	
	public static byte[] hexStringToByteArray(String s) {
		
		byte[] b = new byte[s.length() / 2];
	    for (int i = 0; i < b.length; i++){
	    	int index = i * 2;
	    	int v = Integer.parseInt(s.substring(index, index + 2), 16);
	    	b[i] = (byte)v;
	    }
	    return b;
	}
	
	public static String getHexString(byte[] b) {
		String result = "";
		for (int i=0; i < b.length; i++) {
			result +=
		          Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
		}
		return result;
	}
	
	/**
	 * Does a remote lookup. If the -x option is specified, only authoritative
	 * name servers are returned. Otherwise, an array of resolved IP addresses
	 * is returned
	 * 
	 * @param argv
	 *            the dig command parameterse
	 * @return
	 * @throws IOException
	 */
	public static Record[] doLookup(String[] argv) throws IOException {

		org.xbill.DNS.Name name = null;
		int type = org.xbill.DNS.Type.A;
		int dclass = DClass.IN;
		String server = null;
		int arg;
		Message query;
		Message response = null;
		Record rec;
		SimpleResolver res = null;
		boolean printQuery = false;

		if (argv.length < 1) {
			throw new RuntimeException("Invalid dig format!");
		}

		try {
			arg = 0;

			if (argv[arg].startsWith("@")) {
				server = argv[arg++].substring(1);
				res = new SimpleResolver(server);
			}

			String nameString = argv[arg++];

			if (nameString.equals("-x")) {
				name = ReverseMap.fromAddress(argv[arg++]);
				type = Type.PTR;
				dclass = DClass.IN;
			} else {
				name = Name.fromString(nameString, Name.root);
				type = Type.value(argv[arg]);

				if (type < 0) {
					type = Type.A;
				} else {
					arg++;
				}

				dclass = DClass.value(argv[arg]);

				if (dclass < 0) {
					dclass = DClass.IN;
				} else {
					arg++;
				}
			}

			while (argv[arg].startsWith("-") && (argv[arg].length() > 1)) {
				switch (argv[arg].charAt(1)) {
				case 'p':

					String portStr;
					int port;

					if (argv[arg].length() > 2) {
						portStr = argv[arg].substring(2);
					} else {
						portStr = argv[++arg];
					}

					port = Integer.parseInt(portStr);

					if ((port < 0) || (port > 65536)) {
						LOGGER.warning("Invalid port");

						return null;
					}

					res.setPort(port);

					break;

				case 'b':

					String addrStr;

					if (argv[arg].length() > 2) {
						addrStr = argv[arg].substring(2);
					} else {
						addrStr = argv[++arg];
					}

					InetAddress addr;

					try {
						addr = InetAddress.getByName(addrStr);
					} catch (Exception e) {
						LOGGER.warning("Invalid address");

						return null;
					}

					res.setLocalAddress(addr);

					break;

				case 'k':

					String key;

					if (argv[arg].length() > 2) {
						key = argv[arg].substring(2);
					} else {
						key = argv[++arg];
					}

					res.setTSIGKey(TSIG.fromString(key));

					break;

				case 't':
					res.setTCP(true);

					break;

				case 'i':
					res.setIgnoreTruncation(true);

					break;

				case 'e':

					String ednsStr;
					int edns;

					if (argv[arg].length() > 2) {
						ednsStr = argv[arg].substring(2);
					} else {
						ednsStr = argv[++arg];
					}

					edns = Integer.parseInt(ednsStr);

					if ((edns < 0) || (edns > 1)) {
						LOGGER.info("Unsupported " + "EDNS level: " + edns);

						return null;
					}

					res.setEDNS(edns);

					break;

				case 'd':
					res.setEDNS(0, 0, ExtendedFlags.DO, null);

					break;

				case 'q':
					printQuery = true;

					break;

				default:
					LOGGER.info("Invalid option: " + argv[arg]);
				}

				arg++;
			}
		} catch (ArrayIndexOutOfBoundsException e) {
			if (name == null) {
				// log.log("Name is null!");
				throw new RuntimeException("Name is null!");
			}
		} catch (UnknownHostException e) {
			LOGGER.warning(e.toString());
		} catch (TextParseException e) {
			LOGGER.warning(e.toString());
		}

		if (res == null) {
			try {
				res = new SimpleResolver();
			} catch (UnknownHostException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				LOGGER.warning(e.toString());
			}
		}

		if (name == null) {
			return null;
		}

		rec = Record.newRecord(name, type, dclass);
		query = Message.newQuery(rec);

		if (printQuery) {
			System.out.println(query);
		}

		response = res.send(query);

		// return only the authority section
		if ((response != null) && argv[0].equals("-x")) {
			if (argv.length == 2)
				return response.getSectionArray(Section.ANSWER);
		}

		// return the actual ip addresses
		if (response != null) {
			return response.getSectionArray(Section.ANSWER);
		}

		return null;
	}
}
