package edu.northwestern.pathload;



import java.util.*;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.*;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;

public class NetworkPrimitives {

    protected static final int TIMEOUT = 10000;
	protected static final boolean DEBUG = true;
	
	/* Port numbers (UDP for receiver, TCP for sender) */
	protected static final int TCPSND_PORT = 55002;
	protected static final int UDPRCV_PORT = 55001;
	protected static final int UDP_BUFFER_SZ = 400000; /*bytes*/
	
	protected static boolean quiet = false;
	protected static int max_pkt_sz = 1472;
	protected static Integer cur_pkt_sz = -1;
	protected static long transmissionRate; /*bps*/
	protected static Integer streamLen;
	protected static Long minTimeInterval, timeInterval, sndTimeInterval;
	protected static int numStream;
	
	protected static final int SCALE_FACTOR = 2;
	protected static final int MIN_TIME_INTERVAL = 7; /*microseconds*/
	protected static final int MAX_TIME_INTERVAL = 200000; /*microseconds*/
	protected static final int MIN_PKT_SZ = 200; /*bytes*/
	protected static final int MAX_TRAIN = 5;
	protected static final int TRAIN_LEN = 50;
	protected static final int MAX_STREAM_LEN = 400;
	protected static final int STREAM_LEN = 100; /* # of packets */
	protected static final int NUM_STREAM = 12;
	
	/* Code numbers sent from receiver to sender using the TCP control stream */
	protected static int CTR_CODE = 0x40000000;
	protected static int SEND_FLEET = 0x00000001;
	protected static int RECV_FLEET = 0x00000002;
	protected static int CONTINUE_STREAM = 0x00000003;
	protected static int FINISHED_STREAM = 0x00000007;
	protected static int TERMINATE = 0x00000005;
	protected static int ABORT_FLEET = 0x00000006;
	protected static int SEND_TRAIN = 0x00000008;
	protected static int FINISHED_TRAIN = 0x00000009;
	protected static int BAD_TRAIN = 0x0000000a;
	protected static int GOOD_TRAIN = 0x0000000b;
	
	static Charset charset = Charset.forName("ISO-8859-1");
	static CharsetEncoder encoder = charset.newEncoder();
	static CharsetDecoder decoder = charset.newDecoder();
	protected static ArrayList<String> inPackets = new ArrayList<String>(max_pkt_sz);
	
	static long getTimeOfDayLatency() {
		List<Long> list = new ArrayList<Long>();
		for (int i=0; i<30; i++) {
			Long nanoTime1 = System.nanoTime();
			Long nanoTime2 = System.nanoTime();
			list.add(nanoTime2-nanoTime1);
		}
		
		Collections.sort(list);
		if (DEBUG)
			System.out.println("DEBUG:: getTimeOfDayLatency = " + list.get(14));
		
		return list.get(14);
		
	}
	

    /*
    Send a message through the control stream
	*/
    static void SendCtrMesg(SocketChannel tcpSocketChannel, String ctr_code) {
    	
    	//System.out.println("OUT TCP: " + ctr_code);

    	ctr_code = ctr_code + ":";
    	byte[] ctr_buff = ctr_code.getBytes();
    	
    	try {
    		
	    	if (!tcpSocketChannel.isBlocking()) {
					ByteBuffer buf = ByteBuffer.allocateDirect(max_pkt_sz);
					buf.put(ctr_buff);
					buf.flip();
					tcpSocketChannel.write(buf);
					tcpSocketChannel.socket().getOutputStream().flush();
	    	}

	    	else {
		    	BufferedWriter r;
		    	r = new BufferedWriter(new OutputStreamWriter(  tcpSocketChannel.socket().getOutputStream() ));
				r.write(ctr_code);
				r.flush();
	    	}
	    	
	    	
		} catch (IOException e) {
			System.err.println("SendCtrMesg ERROR: the TCP connection maybe have been closed, retrying....");
		}

	}
    
    
    /* 
    Receive a message from the control stream 
    */
    public static Long RcvCtrMesg(SocketChannel tcpSocketChannel, int timeout) {
    	
        ByteBuffer buf = ByteBuffer.allocateDirect(max_pkt_sz);
        CharBuffer charBuf = CharBuffer.allocate(max_pkt_sz);
    	
        if (inPackets.size() > 0) {
        	//System.out.println("inPakets: " + inPackets);
        	return Long.valueOf(inPackets.remove(0)).longValue();
        }
        
        String ret;
        try {
            buf.clear();

            tcpSocketChannel.socket().setSoTimeout(timeout);
            int numBytesRead = 0;
            String packet;

            if (!tcpSocketChannel.isBlocking()) {
            	//System.out.println("non-blocking");
            	numBytesRead = tcpSocketChannel.read(buf);
            	buf.flip();
            	packet = decoder.decode(buf).toString();
            }

            else {
            	//System.out.println("blocking");
            	BufferedReader d = new BufferedReader(new InputStreamReader(  tcpSocketChannel.socket().getInputStream() ));
            	numBytesRead = d.read( charBuf );       
                charBuf.flip(); 
                packet = charBuf.toString();
            }
            
            //int numBytesRead = tcpSocketChannel.read(buf);
            
            if (numBytesRead > 0) {
                // To read the bytes, flip the buffer
                //buf.flip();
                //packet = decoder.decode(buf).toString();
                //RetPacketPayload(decoder.decode(buf).toString());
                
                RetPacketPayload(packet);
            }


        } catch (SocketTimeoutException e) {
        	System.out.println("TIMEOUT Reading Control Channel");
        	System.exit(1);
            
        } catch (IOException e1) {
            // Connection may have been closed
        	System.err.println("RcvCtrMesg ERROR: the TCP connection maybe have been closed, retrying....");
        	System.exit(1);
        }
       
        if (inPackets.size() > 0) {
        	ret = inPackets.remove(0);
        	return Long.valueOf(ret).longValue();
        }
        	
        
        return (long) -1;
        
        
    }
    
    public static void RetPacketPayload(String param) {

    	for (String s : param.split(":"))
    		inPackets.add(s);
    	
    }
    
    
}