package edu.northwestern.pathload;

import java.util.*;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.net.*;
import java.nio.ByteBuffer;
import java.nio.channels.DatagramChannel;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.util.PluginInterface;
import static edu.northwestern.pathload.NetworkPrimitives.*;

public class PathloadReceiver extends Thread {
	
	protected static SocketChannel tcpSocketChannel;
	protected static DatagramChannel udpClientSocketChannel;
	protected static Long rcv_latency, snd_latency;
	protected static Long adr;
	//protected static Double minTimeInterval, timeInterval, sndTimeInterval;
	protected static Long minRate;
	protected static Long maxRate;
	protected static boolean interruptCoalescence;
	protected static Long tr, tr_min = (long) 0, tr_max = (long) 0;
	protected static Long greyMin = (long) 0, greyMax = (long) 0;
	protected static Long transmissionRate; /* in bps */
	protected static Long  maxRateFlag , minRateFlag;
	protected static int expFlag;
	protected static int requestedDelay = 0;
	protected static boolean converged_gmx_rmx_tm = false;
	protected static boolean converged_gmn_rmn = false;
	protected static boolean converged_rmn_rmx = false;
	protected static boolean converged_gmn_rmn_tm = false;
	protected static boolean converged_gmx_rmx = false;
	protected static boolean converged_rmn_rmx_tm = false;
	protected static int increaseStreamLen;
	protected static Long BWresol;
	protected static int trendIDX;
	//protected static ArrayList<Long> pctMetric = new ArrayList<Long>(50);
	//protected static ArrayList<Long> pdtMetric = new ArrayList<Long>(50);
	protected static Map<Integer, Double> pctMetric = new TreeMap<Integer, Double>();
	protected static Map<Integer, Double> pdtMetric = new TreeMap<Integer, Double>();
	
	protected static boolean slow;
	protected static int num=0;
	protected static long curReqRate, curActualRate;
	protected static int repeat_1 = 0, repeat_2 = 0;
	
	protected static int NOTREND = 1;
	protected static int INCREASING = 2;
	protected static int GREY = 3;
	protected static int NUM_RETRY_CS = 1;
	protected static int TREND_ARRAY_LEN = 50;
	protected static int INCR = 1;
	protected static int NOTR = 2;
	protected static int DISCARD = 3;
	protected static int UNCL = 4;
	protected static float AGGREGATE_THRESHOLD = (float) 0.6;
	protected static float PCT_THRESHOLD = (float) 0.55;
	protected static float PDT_THRESHOLD = (float) 0.4;
	
	protected static Long expStartTime;
	protected static int retryFleetCntCs, retryFleetCntRateMismatch;
	protected static boolean badFleetCs;
	protected static boolean lowerBound;
	protected static boolean icFlag;
	protected static int expFleetId;
    protected static int fleetId = 0;
    protected static int streamId = 0;
    protected static int pktId = 0;

    protected static int HIGH_LOSS_RATE = 15;
    protected static int MEDIUM_LOSS_RATE = 3;
    protected static int MAX_LOSSY_STREAM_FRACTION = 50;
    protected static int MIN_PARTITIONED_STREAM_LEN = 5;       /* number of packet */
    protected static int MIN_STREAM_LEN = 36;      /* # of packets */

    
	private static PathloadReceiver self;
	private boolean isActive;
	private PluginInterface pi;
	private static final boolean DEBUG = true;
	private static String sender = "127.0.0.1";

	public PathloadReceiver(String pSender) {
		super("PathloadManager");
		sender = pSender;
		this.pi = Main.getPluginInterface();
		self = this;
		isActive = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#start()
	 */
	@Override
	public synchronized void start() {
		// TODO Auto-generated method stub
		super.start();
	}
    
	/**
	 * The static interface to the OessExperiment.
	 * 
	 * @return
	 */
	public synchronized static PathloadReceiver getInstance(String sender) {
		if (self == null) {
			self = new PathloadReceiver(sender);
			return self;
		} else {
			return self;
		}
	}
    
	public void setActive(boolean isActive) {
		this.isActive = isActive;
		if (!isActive) {
			self = null;
		}
	}
	

    /* Control stream: TCP channel */
	static SocketChannel openControlChannel(InetAddress sndAdd) {

		SocketChannel sChannel = null;
		
		try {
			sChannel = SocketChannel.open();
			sChannel.configureBlocking(true);
			sChannel.socket().setReuseAddress(true);
			sChannel.connect(new InetSocketAddress(sndAdd, TCPSND_PORT));

						
	  	    if (DEBUG) {
			  System.out.println("DEBUG:: Address: " + sChannel.socket().getInetAddress() + 
					  ", Port: " + sChannel.socket().getLocalPort() + 
					  ", AddressReuse?: " + sChannel.socket().getReuseAddress() + 
					  ", BufferSize: " + sChannel.socket().getLocalPort() + "\n");
		    }
		}catch (IOException e) {
	    	  System.err.println("Make sure that pathload_snd runs at sender:" + sndAdd + 
	    			  " on port " + TCPSND_PORT);
	          System.exit(-1);
		}
		return sChannel;
	}

	
    /* Data stream: UDP channel */
	private static DatagramChannel openDataChannel() {
	    		
        DatagramChannel dChannel = null;

	    InetSocketAddress rcvAdd;
	    try {
	    	dChannel = DatagramChannel.open();
	    	InetAddress IPAddress = InetAddress.getLocalHost();
	    	//dChannel.configureBlocking(false);
				
	      	dChannel.socket().setSendBufferSize(UDP_BUFFER_SZ);
	      	dChannel.socket().setReceiveBufferSize(UDP_BUFFER_SZ);
	      	if (DEBUG)
	      		System.out.print("UDP buffer established!, SendBufferSz: " + dChannel.socket().getSendBufferSize() + "\n");
	    	dChannel.socket().setReuseAddress(true);
	    	
			rcvAdd = new InetSocketAddress(IPAddress, UDPRCV_PORT);
			dChannel.socket().bind(rcvAdd);
			if (DEBUG) 
				System.out.println("Bound to UDP port " + dChannel.socket().getLocalPort() +
					" at " + IPAddress); 								
	    	
		} catch (SocketException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
	    } catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(-1);
		}

		return dChannel;
	}
	
    
	public void run() {
		
		if (DEBUG) { getTimeOfDayLatency(); 	}		
		
		
		try {

			/* Data stream: UDP socket */
			udpClientSocketChannel = openDataChannel();
			
			if (!quiet) {
	    		InetAddress host_rcv = InetAddress.getLocalHost();
	    		java.util.Date localtm = new java.util.Date();
				if (DEBUG)
					System.out.println("Receiver " +  host_rcv +
						" starts measurements at sender " + InetAddress.getByName(sender) +
						" on " + localtm);
			}

			/* Control stream: TCP connection */
			tcpSocketChannel = openControlChannel(InetAddress.getByName(sender));
			tcpSocketChannel.socket().setTcpNoDelay(true);
			System.out.println(tcpSocketChannel.socket().getTcpNoDelay());
			
	    	//Assume Ethernet sized MTU for sender and receiver (1472)
			rcv_latency = RcvLatency();
			rcv_latency = rcv_latency/1000; /*convert to microseconds*/
        	Long snd_latency = RcvCtrMesg(tcpSocketChannel, 60*1000);
        	//System.out.println("MARIO snd_latency: " + snd_latency);
        	requestedDelay = 0;
        	Integer ctrCode;
        	Integer trend, prevTrend = 0;
  	    	BWresol = (long) 0;
        	interruptCoalescence=false;
        	streamLen = STREAM_LEN ;
        	expFlag = 1;
        	expStartTime = System.nanoTime();
        	increaseStreamLen=0;
        	lowerBound=false;
        	trendIDX=0;

        	slow=false;
        	//  requested_delay = 0;
        	badFleetCs=false;
        	numStream = NUM_STREAM;
        	sndTimeInterval= (long)0; 
        	curActualRate = 0 ;
        	curReqRate = 0 ;
        	num =0;
        	
        	
			if (snd_latency == null) {
	    		System.out.println("pathload_snd did not respond for 60 sec");
	    	}
	    	
	    	minTimeInterval = SCALE_FACTOR * Math.max(snd_latency, rcv_latency);
	    	minTimeInterval = Math.max(minTimeInterval, MIN_TIME_INTERVAL);
	    	
	    	maxRate = (max_pkt_sz+28) * (long) 8. / minTimeInterval;
	    	minRate = (MIN_PKT_SZ+28) * (long) 8./ MAX_TIME_INTERVAL;
	    	
	    	 if (DEBUG) {
	    		System.out.printf("  Maximum packet size          :: %d bytes\n",max_pkt_sz);
	    		System.out.printf("  send latency @sndr           :: %d usec\n",snd_latency);
	    		System.out.printf("  recv latency @rcvr           :: %d usec\n",rcv_latency);
	    		System.out.printf("  Minimum packet spacing       :: %d usec\n",minTimeInterval );
	    	    System.out.printf("  Max rate(max_pktsz/min_time) :: %.2f Mbps\n", (double) maxRate);
	    	 }
	    	
	    	 /* Estimate ADR (Asymptotic Dispersion Rate)*/
	    	 adr = getADR();
	    	
	    	 if (BWresol == 0) {
	    		if (adr != 0)
	    			BWresol = (long) 0.02*adr;
	    		else
	    			BWresol = (long) 2;
	    	 }

	    	  if (DEBUG)
	    	    System.out.printf("  Grey bandwidth resolution    :: %.2f\n", (double) greyBWresolution());
	    	
	    	  if ( interruptCoalescence ) {
	    		BWresol = (long) 0.05*adr;
	    		if (DEBUG) 
	    			System.out.printf("  Interrupt coalescion detected\n");
	    	  }

	    	  if ( adr == 0 || adr > maxRate || adr < minRate)
	    		 tr = (maxRate+minRate)/ (long) 2.;
	    	  else 
	    		 tr = (long) adr;

	    	  /* Estimate the available bandwidth.*/
	    	  transmissionRate = (long) Math.round(1000000 * tr);
	    	 
	    	  maxRateFlag = (long) 0;
	    	  minRateFlag = (long) 0 ; 

	    	  long startTime = System.currentTimeMillis();
	    	  
	    	  while (true) {
		    	  
		    	  if ( requestedDelay != 0 ) {
		    		  if (System.currentTimeMillis() - startTime >= requestedDelay*1000)
		    			  break;  /* RECEIVER ABORTS TIME */
		    	  }
		    	  
		    	  if (calcParam() == -1) {
		    		  ctrCode = TERMINATE | CTR_CODE;
		    		  SendCtrMesg(tcpSocketChannel, ctrCode.toString());
		    	      terminateGracefully(expStartTime) ;
		    	  }

		    	  //System.out.println("MARIO: transmissionRate: " + transmissionRate);
		    	  SendCtrMesg(tcpSocketChannel, transmissionRate.toString());
		    	  //System.out.println("MARIO: cur_pkt_sz: " + cur_pkt_sz);
		    	  SendCtrMesg(tcpSocketChannel,cur_pkt_sz.toString()) ;
		    	  
		    	  if ( increaseStreamLen != 0 )
		    	      streamLen=3*STREAM_LEN;
		    	   else
		    	      streamLen = STREAM_LEN;
		    	   
		    	  //System.out.println("MARIO: streamLen: " + streamLen);
		    	  SendCtrMesg(tcpSocketChannel,streamLen.toString());
		    	  //System.out.println("MARIO: timeInterval: " + timeInterval);
		    	  SendCtrMesg(tcpSocketChannel,timeInterval.toString());
		    	  ctrCode = SEND_FLEET | CTR_CODE ;
		    	  SendCtrMesg(tcpSocketChannel, ctrCode.toString());

		    	  Long retVal = (long) 0;
		    	  
		    	  while (true) {

		    		  /*
		    		  long sTime = System.currentTimeMillis();
		    		  while (retVal <= 0) {
		    			  retVal = RcvCtrMesg(tcpSocketChannel, 5*1000);
		    			  if (System.currentTimeMillis() - sTime >= 5000)
		    				  break;
		    		  }
		    		  */
		    		  
		    		  retVal = RcvCtrMesg(tcpSocketChannel, 30*1000);
		    		  
		    	      if ((((retVal & CTR_CODE) >> 30) == 1) &&     
		    	           ((retVal & 0x00ffffff) == RECV_FLEET )) {
		    	    	  //System.out.println("MARIO: RECVFLEET!");
		    	    	  break ;
		    	      }
		    	      else if ( (((retVal & CTR_CODE) >> 30) == 1) &&    
		    	                 ((retVal & 0x00ffffff) == FINISHED_STREAM ))
			          	  retVal = RcvCtrMesg(tcpSocketChannel, 60*1000);

		    	  } //while
           	       
		    	  if (RecvFleet() == -1) {
		    	  	
		    	  		if ( increaseStreamLen != 0) {
		    	  			trend = INCREASING;
		    	        
		    	  			if ( expFlag == 1 && prevTrend != 0 && prevTrend != trend)
		    	  				expFlag = 0;
		    	        
		    	  			prevTrend = trend;
		    	  			System.out.println("rate_adjustment!");
		    	  			if (rateAdjustment(INCREASING) == -1)
		    	  				terminateGracefully(expStartTime);
		    	  		}
		    	   }
		    	   else {
		    	    	  //System.out.println("GETTREND");
			    	      getSendingRate() ;
			    	      trend = aggregateTrendResult();
			    	      
			    	      if (( trend == -1) && badFleetCs && retryFleetCntCs > NUM_RETRY_CS ) 
			    	        terminateGracefully(expStartTime) ;
			    	      
			    	      else {
			    	    	  if ( (trend == -1) && badFleetCs && (retryFleetCntCs <= NUM_RETRY_CS )) /* repeat fleet with current rate. */
			    	    		  continue ;
			    	      }
	
			    	      if (trend != GREY) {
			    	    	  if (expFlag == 1 && prevTrend != 0 && prevTrend != trend)
			    	    		  expFlag = 0;
			    	    	  prevTrend = trend;
			    	      }
	
			    	      if (rateAdjustment(trend) == -1) {
			    	    	  //System.out.println("I'M OUT!");
			    	        terminateGracefully(expStartTime);
			    	      }
		    	    }

	    	  }

			
		} catch (IOException e) {
			System.err.println("TCP control channel Accept Error!");
		}
		
	}
	

	/*
	  calculates fleet param L,T .
	  calc_param returns -1, if we have 
	  reached to upper/lower limits of the 
	  stream parameters like L,T .
	  otherwise returns 0 .
	*/

	private static int calcParam() {
		Long tmpTr;
		Integer tmp;
		Long tmpTimeInterval;

		
		if (tr < 150) {
			timeInterval = 80>minTimeInterval ? 80 : minTimeInterval;
			cur_pkt_sz = (int) (Math.round(tr*timeInterval/8.) - 28);
			if (cur_pkt_sz < MIN_PKT_SZ) {
				cur_pkt_sz = MIN_PKT_SZ;
				timeInterval = (Math.round(cur_pkt_sz + 28)*8/tr);
				tr = (cur_pkt_sz + 28) * (long) 8. / timeInterval;
			}
			else if (cur_pkt_sz > max_pkt_sz) {
				  cur_pkt_sz = max_pkt_sz;
			      timeInterval = minTimeInterval ;
			      tmpTr = ( cur_pkt_sz + 28 )* (long) 8. /timeInterval ;
			      if ( equal(tr, tmpTr) )
			    	  tr = tmpTr;
			      else
			    	  return -1;
			}
		}
		else if ( tr < 600 ) {
			
			tmpTr = tr ;
			tmpTimeInterval = (long) Math.rint(( max_pkt_sz + 28 )* 8 / tr) ;

			if ( (cur_pkt_sz == max_pkt_sz) && (tmpTimeInterval == timeInterval) )
					return -1 ;

			timeInterval = tmpTimeInterval ; 
			tmp = (int) Math.round(tr*timeInterval/8.)-28;
			cur_pkt_sz = tmp<max_pkt_sz?tmp:max_pkt_sz;
			tr = ( cur_pkt_sz + 28 ) * (long) 8./timeInterval ;
			
			//System.out.println("MARIO: tmpTimeInterval " + tmpTimeInterval + " tmpTr: " + tmpTr);
			//System.out.println(cur_pkt_sz + " " + max_pkt_sz);
			//System.out.println("tr_min " + tr_min + " greyMax " + greyMax);
			
			if ( ((tr_min != 0) && (equal(tr,tr_min) || tr<tr_min)) 
				        || ((greyMax!=0) && tmpTr>greyMax && (equal(tr,greyMax) || tr<greyMax))) { 
			
				do {
					--timeInterval;
					cur_pkt_sz=(int) (Math.round(tr*timeInterval/8.)-28);
				} while (cur_pkt_sz > max_pkt_sz);
				tr = ( cur_pkt_sz + 28 ) * (long) 8./timeInterval ;
			}
		}

		else {
			cur_pkt_sz = max_pkt_sz ;
		    timeInterval = (long) Math.rint(( cur_pkt_sz + 28 )* 8 / tr); 
		    tr = ( cur_pkt_sz + 28 ) * (long) 8./timeInterval ;
		    if (((tr_min != 0) && (equal(tr,tr_min) || tr<tr_min)) )
		      return -1 ;
		    
		    if( equal(tr,tr_max) ) {
		      tr_max = tr ;
		      if ( greyMax != 0) {
		        converged_gmx_rmx_tm = true;
		        if ( !converged_gmn_rmn && !converged_gmn_rmn_tm )
		          radj_notrend();
		        else 
		        	return -1 ;
		      }
		      else {
		        converged_rmn_rmx=true;
		        return -1 ;
		      }
			}
		}
		
		return 0;
	}
	

	/*
    Calculate next fleet rate
    when fleet showed GREY trend. 
	*/
	private static void radj_greymax() {

		//System.out.println("MARIO: radj_greymin(): greyMax: " + greyMax + " tr_max: " + tr_max);
		if ( tr_max == 0 )
			tr = (tr+.5*tr)<maxRate?(tr+ (long) .5*tr):maxRate ;
		
		else if ( tr_max - greyMax <= greyBWresolution() ) {
			converged_gmx_rmx = true;
			radj_greymin() ;
		}
		else 
			tr = ( tr_max + greyMax)/ (long) 2. ;
	}
	
	
	/*
    Calculate next fleet rate
    when fleet showed GREY trend. 
	*/
	private static void radj_greymin() {
		System.out.println("radj_greymin(): greyMin: " + greyMin + " tr_min: " + tr_min);
		if ( greyMin - tr_min <= greyBWresolution() ) {
			converged_gmn_rmn = true;
			radj_greymax() ;
		}
		else 
			tr = (tr_min+greyMin)/(long)2.<minRate?minRate:(tr_min+greyMin)/(long)2. ;
	}
	
	
	/*
    Calculate next fleet rate
    when fleet showed NOTREND trend. 
	 */
	private static void radj_notrend() {
		
	  //System.out.println("radj_notrend!!");
		
	  if ( expFlag != 0 )
	     tr =  2*tr > maxRate ? maxRate : 2*tr ;
	  else
	  {
	    if ( greyMin != 0 && greyMin <= tr_max )
	    {
	      if ( greyMin - tr_min <= greyBWresolution() )
	      {
	        converged_gmn_rmn = true;
	        radj_increasing() ;
	      }
	      else 
	        tr =  (tr_min+greyMin)/2. < minRate ? minRate : (tr_min+greyMin)/(long)2. ;
	    }
	    else
	      tr =  (tr_max + tr_min)/2. < minRate ? minRate : (tr_min+tr_max)/(long)2. ;
	  }
	}
	
	/*
    Calculate next fleet rate
    when fleet showed INCREASING trend. 
	 */
	private static void radj_increasing() { 
	
		if ( greyMax != 0 && greyMax >= tr_min ) {
			
		    if ( tr_max - greyMax <= greyBWresolution() ) {
		    	
		      converged_gmx_rmx = true;
		      expFlag=0;
		      if ( (greyMin != 0) || (tr_min != 0) )
		        radj_notrend() ;
		      else {
		        if ( greyMin < greyMax )
		          tr = (long) greyMin /  (long) 2. ;
		        else
		          tr = (long) greyMax /  (long) 2. ;
		      }
		      
		    }
		    else 
		     tr = (long) ( tr_max + greyMax)/ (long) 2. ;
		
		}
		else
			tr =  (long) (tr_max + tr_min)/ (long) 2.< minRate?minRate:(tr_min+tr_max)/ (long) 2. ;
	}
	

	private static Double greyBWresolution() {
		
		  if ( adr != 0 )
			    return (.05*adr < 12 ? .05*adr : 12) ;
			  else 
			    return (double) minRate ;
		
	}
	
	//pathload_snd_func
    private static Long RcvLatency() {
    	
    	long latency = -1;
        DatagramSocket udpRcvSocket = null;
        DatagramSocket udpSndSocket = null;
        InetSocketAddress rcvAdd;
      	try {
      		
      		InetAddress IPAddress = InetAddress.getLocalHost();
			udpRcvSocket = new DatagramSocket(null);
			rcvAdd = new InetSocketAddress(IPAddress, 6789);
			udpRcvSocket.bind(rcvAdd);
			if (DEBUG)
				System.out.println("Bound to UDP port " + udpRcvSocket.getLocalPort() +
					" at " + IPAddress); 

			udpSndSocket = new DatagramSocket(null);

			/* Create random payload; does it matter? */
			byte[] sendData = new byte[max_pkt_sz];
	        for (int i=0; i<max_pkt_sz; i++) {
	        	sendData[i] = (byte) i;
	        }

	        DatagramPacket sendPacket =
	            new DatagramPacket(sendData, sendData.length, 
	            		IPAddress, 6789);

			byte[] receiveData = new byte[max_pkt_sz];
		    DatagramPacket receivePacket =
	            new DatagramPacket(receiveData,
	                               receiveData.length);

	        long first_time, current_time;
	        ArrayList<Long> min_OSdelta = new ArrayList<Long>();
	        for (int i=0; i<50; i++) {
	        	udpSndSocket.send(sendPacket);
	        	first_time = System.nanoTime();
	        	udpRcvSocket.receive(receivePacket);
	        	current_time = System.nanoTime();
	        	min_OSdelta.add(current_time - first_time);
	        }
	        
	        /* Use median  of measured latencies to avoid outliers */
	        Collections.sort(min_OSdelta);
	        latency = min_OSdelta.get(25);
				
			
		} catch (SocketException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return latency;		
    }

	
    /* 
    get ADR (Asymptotic Dispersion Rate)
    */
    private static long getADR() {
    	Integer ctrCode = SEND_TRAIN | CTR_CODE;
    	SendCtrMesg(tcpSocketChannel, ctrCode.toString());
    	int expTrainId = 0;
    	int retry = 0;
    	boolean badTrain = true;
    	int numBadTrain = 0;
    	int trainLen = 3;
    	int spaceCnt = 24;
    	long delta = 0;  /*usec*/
    	int sum = 0;
    	ArrayList<Long> arrv_tv = null;
    	int numBurst;
    	int last = 0;
    	long BWmsr=0;
    	ArrayList<Long> badBWmsr = new ArrayList<Long>();
    	Object retObject[];
    	
    	if (DEBUG) 
    		System.out.print("  ADR[");
    	
    	while ( (retry < MAX_TRAIN) & (badTrain) ) {
    		if (trainLen == 5)
    			trainLen = 3;
    		else
    			trainLen = TRAIN_LEN - expTrainId*15;
    			
			if (DEBUG)
				System.out.print(".");
			
	    	arrv_tv = new ArrayList<Long>();
			spaceCnt--;
			//badTrain = RecvTrain(expTrainId, arrv_tv, trainLen);
			retObject = RecvTrain(expTrainId, trainLen);
			badTrain = Boolean.parseBoolean(retObject[0].toString());
			arrv_tv = (ArrayList<Long>) retObject[1];
			
			/* Compute dispersion and bandwidth measurement */
			if (!badTrain) {

				//System.out.println("\nNOT A BAD TRAIN " + arrv_tv.size() + "\n");
				numBurst = 0;
				
			    //interruptCoalescence=CheckIntrCoalescence(arrv_tv,trainLen,numBurst);
				retObject = CheckIntrCoalescence(arrv_tv,trainLen,numBurst);
				interruptCoalescence = Boolean.parseBoolean(retObject[0].toString());
				numBurst = Integer.parseInt( retObject[1].toString());
				
			    last = trainLen;
			    delta = arrv_tv.get(last) - arrv_tv.get(1);  /*skip first packet*/
			    BWmsr = ((28+max_pkt_sz) * 8) * (last-1) / delta;
			    //System.out.println("BWmsr " + BWmsr + " last " + last + " delta " + delta);
			    
			    /* tell sender that it was agood train.*/
			    ctrCode = GOOD_TRAIN | CTR_CODE;
			    SendCtrMesg( tcpSocketChannel, ctrCode.toString() ) ;
			}
			else {
				
				//System.out.println("\nBAD TRAIN " + arrv_tv.size() + "\n");
				retry++;
				
			    /* wait for atleast 10msec before requesting another train */
				delta = 0;
				last = arrv_tv.size()-1;
				int first = 1;
				if (last >0 )
					delta = arrv_tv.get(last) - arrv_tv.get(first);  /*skip the first packet*/
				
			    if (delta == 0) { 
			    	delta = 1; 
			    }
			    //System.out.println("numBadTrains: " + numBadTrain + " first: " + first + " last: " + last + " delta: " + delta);
			    badBWmsr.add(numBadTrain++, ((28+max_pkt_sz) * 8) * (last-first-1) / delta);

			    try {
					Thread.sleep(10); //10 millisecond
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			    
			    ctrCode = BAD_TRAIN | CTR_CODE ;
			    SendCtrMesg( tcpSocketChannel, ctrCode.toString() ) ;
			    expTrainId++ ;
			}
    		
    	}//while
    	
    	if (DEBUG) {
    		int i = spaceCnt;
    		System.out.print("]");
    		while (--i >= 0)
    			System.out.print(" ");
    		System.out.print(":: ");
    	}
    	
    	if (!badTrain) {
    		if (DEBUG) {
    		      System.out.printf("%.2fMbps\n", (float)BWmsr) ;
    		}
    	}
    	else {
    		for (int i=0; i<numBadTrain; i++) {
    			if( badBWmsr.get(i)>=0 )
    				sum+= badBWmsr.get(i);
    		}
    		BWmsr = sum/numBadTrain;
    		if (DEBUG) {
    			System.out.printf("%.2fMbps (I)\n", (float)BWmsr);
    		}
    	}
    	
    	return BWmsr;
    }
    
    
    /* Receive a complete packet train from the sender */
    private static Object[] RecvTrain( int expTrainId, int trainLen) {
    	
    	ArrayList<Long> time = new ArrayList<Long>();
    	int trainId=0, packId=0, expPackId=0, rcvd=0;
    	boolean badTrain = false;
        Selector selector = null;
        boolean finished = false;
        
        try {
            selector = Selector.open();
            udpClientSocketChannel.configureBlocking(false);
			tcpSocketChannel.configureBlocking(false);
            
            tcpSocketChannel.register(selector, SelectionKey.OP_READ);
            udpClientSocketChannel.register(selector, SelectionKey.OP_READ);
            
            Long start = System.currentTimeMillis();
            
            do {
            	selector.select(1000);
                Iterator it = selector.selectedKeys().iterator();
                
                if (System.currentTimeMillis() - start > 30000) {
                	System.out.println("\nSender did not respond for 30 seconds...");
                	System.exit(0);
                }
                	
                
                //process each key at a time
                while (it.hasNext()) {

                    SelectionKey selectedKey = (SelectionKey)it.next();
	            	//System.out.println(selectedKey.toString() + " " + selectedKey.isValid() + " " + selectedKey.isReadable());

	                    ByteBuffer inBuffer = ByteBuffer.allocate(max_pkt_sz);
	
	                    if (selectedKey.channel() == udpClientSocketChannel) {
		                	it.remove();
	                    	//System.out.println("UDP!");
		                	
	                        ((DatagramChannel) selectedKey.channel()).receive(inBuffer);
	                        inBuffer.flip();
	                        
	                        int inSize = inBuffer.remaining();
	                        byte[] retVal = new byte[inSize];
	                        inBuffer.get(retVal); /*pos 0, contains trainId*/

	                        trainId = Integer.valueOf(retVal[0]).intValue();
	                        packId = Integer.valueOf(retVal[1]).intValue();
	                        //System.out.println("trainId: " + trainId + " packId: " + packId);
	                        //System.out.flush();
	                        
	                        if ((trainId == expTrainId) && (packId == expPackId)) {
	                        	rcvd++;
	                        	time.add(System.nanoTime()/1000);
	                        	expPackId++;
	                        }
	                        else{
	                        	badTrain = true;
	                        }
	                    }
	
	                    if (selectedKey.channel() == tcpSocketChannel) {
		                	it.remove();
	                    	//System.out.println("TCP!");
	                    	
		                	Long retVal;
	                    	retVal = RcvCtrMesg(tcpSocketChannel, 60*1000);
                    		if ( retVal == -1) {
	            	    		System.out.println("pathload_snd did not respond for 60 sec");
	            	    		System.exit(1);
                    		}
                    		
                    		
	            	        //if ( (((retVal & CTR_CODE) >> 30) == 1) && ((retVal & 0x00ffffff) == FINISHED_TRAIN ) ) {
                    		if ( (((retVal & CTR_CODE) >> 30) == 1) && ((retVal & 0x00ffffff) == FINISHED_TRAIN ) ) {
	            	        	finished = true;
	            	        	break;
	            	        }  
	                    }

	            } //while
                
                if (finished)
                	break;

                
            } while (true);
            
        } catch (IOException e) {
        	e.printStackTrace();
        }
        
    	//System.out.println("trainLen: " + trainLen + " rcvd "  + rcvd + " badTrain: " + badTrain);        
        if ( rcvd != trainLen+1 )
        	badTrain=true;
       
    	Object object[] = new Object[2];
    	object[1] = time;
    	
    	object[0] = badTrain;
    	return object;
        
        
        //return badTrain; 	
    }
    
    
    private static Object[] CheckIntrCoalescence( ArrayList<Long> time, long len, int burst) {

    	//System.out.println("\nInterrupt Coalescence: burst=" + burst + ", len=" + len);
    	//System.out.println(time);
  
    	long minGap;
    	int b2b=0, tmp=0;
    	
    	if (MIN_TIME_INTERVAL > 3*rcv_latency)
    		minGap = MIN_TIME_INTERVAL;
    	else
    		minGap = 3*rcv_latency;
    	    	
    	for (int i=1; i<len; i++) {
    		if ((time.get(i)-time.get(i-1)) <= minGap) {
    			//System.out.println(time.get(i)-time.get(i-1) + " " + minGap + " " + time.get(i) + " " + time.get(i-1));
    			b2b++;
    			tmp	++;
    		}
    		else
    			if (tmp >= 3) {
    				burst++;
    				tmp=0;
    			}
    	}
    	
    	//System.out.println("Interrupt Coalescence: burst=" + burst + ", b2b =" + b2b + ", len=" + len);
    	
    	Object object[] = new Object[2];
    	object[1] = burst;
    	
    	if (b2b > 0.6*len)
    		object[0] = true;
    	else
    		object[0] =  false;
    	
    	return object;
    }
    

    private static void terminateGracefully( Long expStartTime) {
    	Integer ctrCode;
    	Long expEndTime;
    	double min=0,max=0 ;

    	ctrCode = TERMINATE | CTR_CODE;
    	SendCtrMesg(tcpSocketChannel,ctrCode.toString());
    	expEndTime = System.nanoTime();

        //strncpy(buff, ctime(&(exp_end_time.tv_sec)), 24);
        //buff[24] = '\0';
      
       if (DEBUG)
    	   System.out.println("\n\t*****  RESULT *****\n");
      
       if ( minRateFlag != 0) {
    	  if (DEBUG) {
    		  System.out.printf("Avail-bw < minimum sending rate.\n");
    		  System.out.printf("Increase MAX_TIME_INTERVAL in pathload_rcv.h from 200000 usec to a higher value.\n");
    	  }
       }
       else if ( (maxRateFlag != 0) && !interruptCoalescence ) {
    		  if (DEBUG) {
    			  System.out.printf("Avail-bw > maximum sending rate.\n");
    			  if (tr_min != 0)
    				  System.out.printf("Avail-bw > %.2f (Mbps)\n", (double) tr_min);
    		  }
       }
       else if (badFleetCs && !interruptCoalescence) {
			  
    		if (DEBUG) 
				  System.out.printf("Measurement terminated due to frequent CS @ sender/receiver.\n");

			if ( ((tr_min != 0) && (tr_max != 0)) || ( (greyMin != 0) && (greyMax != 0)) ) {
	        	  if ( (greyMin != 0) && (greyMax != 0) ) {
	        	  	min = greyMin ; 
	        	  	max = greyMax ;
	        	  }
	        	  else {
	        		  min = tr_min ;
	        		  max = tr_max ;
	        	  }
			  }
     
        	  if (DEBUG) {
        		  System.out.printf("Available bandwidth range : %.2f - %.2f (Mbps)\n", min, max);
        		  System.out.printf("Measurements finished at %s \n", new Date(expEndTime/1000/1000));
		          System.out.printf("Measurement latency is %.2f sec \n", (double) (expEndTime - expStartTime)/1000000000);
		          
        	  }
    	}
        
    	else {
    		  
    		  if ( !interruptCoalescence && ((converged_gmx_rmx_tm && converged_gmn_rmn_tm) || converged_rmn_rmx_tm )) {
	
	        		  if (DEBUG)
	        			  System.out.printf("Actual probing rate != desired probing rate.\n");
	
	        		  if ( converged_rmn_rmx_tm ) {
	        			  min = tr_min; 
	        			  max = tr_max;
	        		  }
	        		  else {
	        			  min = greyMin ; 
	        			  max = greyMax ;
	        		  }
	        	}
    	  		else if ( !interruptCoalescence && converged_rmn_rmx ) {

        			  if (DEBUG)
        				  System.out.printf("User specified bandwidth resolution achieved\n");

        			  min = tr_min ; 
        			  max = tr_max ;
        		}
	        	else if ( !interruptCoalescence && converged_gmn_rmn && converged_gmx_rmx ) {

    				  if (DEBUG)
    					  System.out.printf("Exiting due to grey bw resolution\n");

    				  min = greyMin ; 
    				  max = greyMax;
        		}
	        	else {
    				  min = tr_min ; 
    				  max = tr_max;
        		}
	        	  

		        if (DEBUG) {
		          if ( lowerBound ) {
		            System.out.printf("Receiver NIC has interrupt coalescence enabled\n");
		            System.out.printf("Available bandwidth is greater than %.2f (Mbps)\n", min);
		          }
		          else
		            System.out.printf("Available bandwidth range : %.2f - %.2f (Mbps)\n", min, max);

		          System.out.printf("Measurements finished at %s \n", new Date(expEndTime/1000/1000));
		          System.out.printf("Measurement latency is %.2f sec \n", (double) (expEndTime - expStartTime)/1000000000);
		          //fprintf(pathload_fp,"Measurements finished at %s \n",  buff);
		          //fprintf(pathload_fp,"Measurement latency is %.2f sec \n", time_to_us_delta(exp_start_time, exp_end_time) / 1000000);

		        }
    	}
       
        System.exit(0);
       
    }
    
    


    /*
        dpending upon trend in fleet :-
        - update the state variables.
        - decide the next fleet rate
        return -1 when converged
    */
    private static int rateAdjustment(Integer flag)
    {
      int retVal = 0 ;
      if( flag == INCREASING ) {
        if ( maxRateFlag != 0)
        	maxRateFlag= (long) 0;
        
        if ( greyMax >= tr )
        	greyMax = greyMin = (long) 0 ;
        
        tr_max = tr ;
        if (!converged_gmx_rmx_tm) {
          if ( !converged() ) 
            radj_increasing() ;
          else
        	  retVal = -1; 
        }
        else {
          expFlag = 0 ;
          if ( !converged() )
            radj_notrend() ;
        }
      }

      else if ( flag == NOTREND ) {
    	  //System.out.println("NOTREND: greyMin: " + greyMin + " greyMax: " + greyMax + " tr: " + tr + " tr_min: " + tr_min);
    	  if ( greyMin < tr )
    		  greyMin = (long) 0;
    	  if ( greyMax < tr )
    		  greyMax = greyMin = (long) 0;
    	  if ( tr > tr_min )
    		  tr_min =  tr ;
    	  if ( !converged_gmn_rmn_tm && !converged() ) 
    		  radj_notrend() ;
    	  else
    		  retVal = -1 ;
      }
      
      else if ( flag == GREY ) {
    	  //System.out.println("GREY: greyMin: " + greyMin + " greyMax: " + greyMax + " tr: " + tr + " tr_min: " + tr_min);
    	  if ( (greyMax == 0) && (greyMin == 0) )
    		  greyMax = greyMin = tr ;
    	  if (tr==greyMax || tr>greyMax ) {
    		  greyMax = tr ;
          
	    	  if ( !converged_gmx_rmx_tm ) {
	    		  if ( !converged() )
	    			  radj_greymax() ;
	    		  else
	    		  	  retVal = -1 ;
	          }
	          else {
	
	            expFlag = 0 ;
	            if ( !converged() )
	              radj_notrend() ;
	            else
	              retVal=-1;
	          }
    	  }
        
    	  else if ( (tr < greyMin) || (greyMin == 0) ) {
    		greyMin = tr ;
    		if ( !converged() )
    			radj_greymin() ;
    		else
    		    retVal = -1 ;
        }
      }

      if (DEBUG)
      {
        System.out.printf("  Rmin-Rmax             :: %.2f-%.2fMbps\n", (double) tr_min, (double) tr_max);
        System.out.printf("  Gmin-Gmax             :: %.2f-%.2fMbps\n", (double) greyMin, (double) greyMax);
      }

      if ( retVal == -1 )
    	  return -1 ;
      if ( tr >= maxRate )
    	  maxRateFlag++ ;
      if ( maxRateFlag > 1 )
         return -1 ;
      if ( minRateFlag > 1 ) 
         return -1 ;
      
      transmissionRate = (long) Math.rint(1000000 * tr ) ;
      return 0 ;
    }
    
    
    
    /*
    test if Rmax and Rmin range is smaller than
    user specified bw resolution
    or
    if Gmin and Gmax range is smaller than grey
    bw resolution.
     */
	static protected boolean converged()
	{
	  boolean retVal= false;
	  if ( (converged_gmx_rmx_tm && converged_gmn_rmn_tm ) || converged_rmn_rmx_tm )
			  retVal=true;
	  
	  else if ( tr_max != 0 && tr_max != tr_min ) {
		  
	    if ( tr_max - tr_min <= BWresol )
	    {
	      converged_rmn_rmx = true;
	      retVal=true;
	    }
	    if( tr_max - greyMax <= greyBWresolution() &&
	              greyMin - tr_min <= greyBWresolution() )
	    {
	      converged_gmn_rmn = true;
	      converged_gmx_rmx = true;
	      retVal=true;
	    }
	  }
	  return retVal ;
	}
    
    
	/*
	  returns : trend in fleet or -1 if more than 50% of stream were discarded 
	*/
	protected static int aggregateTrendResult()
	{
	  int total=0 ,iCnt = 0, nCnt = 0;
	  int numDscrdStrm=0;
	  int i=0;
	  //ArrayList<Integer> pctTrend = new ArrayList<Integer>();
	  //ArrayList<Integer> pdtTrend = new ArrayList<Integer>() ;
	  Map<Integer, Integer> pdtTrend = new TreeMap<Integer, Integer>();
	  Map<Integer, Integer> pctTrend = new TreeMap<Integer, Integer>();
	  
	  Object[] retObj;
	  
	  if (DEBUG)
	    System.out.printf("  PCT metric/stream[%2d] :: ",trendIDX); 
 
	  for (i=0; i < trendIDX;i++ ) {
		  if (DEBUG)
	      System.out.printf("%3.2f:",pctMetric.get(i));
	  }
	  
	  if (DEBUG) {
	    System.out.printf("\n");
	    System.out.printf("  PDT metric/stream[%2d] :: ",trendIDX);
	  }
	   
	  for (i=0; i < trendIDX;i++ ) {
		  if (DEBUG)
			  System.out.printf("%3.2f:",pdtMetric.get(i));
	  }
	  
	  if (DEBUG) {
	    System.out.printf("\n");
	    System.out.printf("  PCT Trend/stream [%2d] :: ",trendIDX);
	  }

	  retObj = getPctTrend(pctMetric, pctTrend, trendIDX);
	  pctTrend = (Map<Integer, Integer>) retObj[0];

	  if (DEBUG)
	    System.out.printf("  PDT Trend/stream [%2d] :: ",trendIDX); 
	   
	  retObj = getPdtTrend(pdtMetric,pdtTrend,trendIDX);
	  pdtTrend = (Map<Integer, Integer>) retObj[0];
	  
	  //System.out.println("pctTrend: " + pctTrend);
	  //System.out.println("pdtTrend: " + pdtTrend);

	  
	  if (DEBUG)
	    System.out.printf("  Trend per stream [%2d] :: ",trendIDX); 
	   
	  for (i=0; i < trendIDX;i++ ) {
		  
		if (!pctTrend.containsKey(i) || !pdtTrend.containsKey(i)) {
			System.out.println("MARIO: aggreateTrendResult(): ONE PCT line is missing?");
			continue;
		}

	    if ( pctTrend.get(i) == DISCARD || pdtTrend.get(i) == DISCARD ) {
	       if (DEBUG)
	         System.out.printf("d");
	       
	       numDscrdStrm++ ;
	    }
	    else if ( pctTrend.get(i) == INCR &&  pdtTrend.get(i) == INCR ) {
	       if (DEBUG)
	         System.out.printf("I");
	       
	       iCnt++;
	    }
	    else if ( pctTrend.get(i) == NOTR && pdtTrend.get(i) == NOTR ) {
	       if (DEBUG)
	         System.out.printf("N");
	       
	       nCnt++;
	    }
	    else if ( pctTrend.get(i) == INCR && pdtTrend.get(i) == UNCL ) {
	       if (DEBUG)
	         System.out.printf("I");
	       
	       iCnt++;
	    }
	    else if ( pctTrend.get(i) == NOTR && pdtTrend.get(i) == UNCL ) {
	       if (DEBUG)
	         System.out.printf("N");
	      
	       nCnt++;
	    }
	    else if ( pdtTrend.get(i) == INCR && pctTrend.get(i) == UNCL ) {
	       if (DEBUG)
	         System.out.printf("I");
	       
	       iCnt++;
	    }
	    else if ( pdtTrend.get(i) == NOTR && pctTrend.get(i) == UNCL ) {
	       if (DEBUG)
	         System.out.printf("N");
	       
	       nCnt++ ;
	    }
	    else {
	       if (DEBUG)
	         System.out.printf("U");
	    }
	    total++ ;
	  }
	  if (DEBUG) 
		  System.out.printf("\n"); 
	   

	  /* check whether number of usable streams is 
	     atleast 50% of requested number of streams */
	  total-=numDscrdStrm ;
	  if ( (total < numStream/2) && !slow && !interruptCoalescence)
	  {
	    badFleetCs = true ;
	    retryFleetCntCs++  ;
	    return -1 ;
	  }
	  else {
	    badFleetCs = false ;
	    retryFleetCntCs=0;
	  }

	  if( (double)iCnt/(total) >= AGGREGATE_THRESHOLD ) {
	    if (DEBUG)
	      System.out.printf("  Aggregate trend       :: INCREASING\n");
	    
	    return INCREASING ;
	  }
	  if( (double)nCnt/(total) >= AGGREGATE_THRESHOLD ) {
	    if (DEBUG)
	      System.out.printf("  Aggregate trend       :: NO TREND\n");
	    
	    return NOTREND ;
	  }
	  else 
	  {
	    if (DEBUG)
	      System.out.printf("  Aggregate trend       :: GREY\n");
	   
	    return GREY ;
	  }
	}
	
	protected static void getSendingRate() {
		
		 timeInterval = sndTimeInterval/num;
		 curReqRate = tr ;
		 curActualRate = (long) (28 + cur_pkt_sz) * (long) 8. / (long) timeInterval ;
	
		 if( !equal(curReqRate,curActualRate) ) { 
		 
			   if( (greyMax == 0) && (greyMin == 0) ) {
				   
				    if( (tr_min!=0) && (tr_max!=0) && (less_than(curActualRate,tr_min)||equal(curActualRate,tr_min)))
				        converged_rmn_rmx_tm = true;
				      if( (tr_min!=0) && (tr_max!=0) && (less_than(tr_max,curActualRate)||equal(tr_max,curActualRate)))
				        converged_rmn_rmx_tm = true;
			   }
			   
		   
			   else if ( curReqRate < tr_max && curReqRate > greyMax )
			   {
			     if( !(less_than(curActualRate,tr_max)&&grtr_than(curActualRate,greyMax)) )
			       converged_gmx_rmx_tm = true;
			   }
			   else if ( curReqRate < greyMin && curReqRate > tr_min )
			   {
			     if( !(less_than(curActualRate,greyMin) && grtr_than(curActualRate,tr_min)) )
			        converged_gmn_rmn_tm = true;
			   }
			   
		  }
	
		  tr = curActualRate ;
		  transmissionRate = (long) Math.rint(1000000 * tr) ;
		  if(DEBUG)
		    System.out.printf("  Fleet Parameter(act)  :: R=%.2fMbps, L=%dB, K=%dpackets, T=%dfusec\n", (double) curActualRate, cur_pkt_sz , streamLen, timeInterval);
		  
		  sndTimeInterval= (long) 0;
		  num=0;
	}

	static Object[] getPctTrend(Map<Integer, Double> pctMetric, Map<Integer, Integer> pctTrend, int pctResultCnt ) {

		 //System.out.println("getPctTrend: " + pctMetric2);
		 int i ;
		 for (i=0; i < pctResultCnt;i++ ) {
			   pctTrend.put(i, UNCL);
			   if ( pctMetric.get(i) == -1  ) {
			     if (DEBUG)
			       System.out.printf("d");
			     
			     pctTrend.put(i, DISCARD);
			   }
			   
			   else if ( pctMetric.get(i) > 1.1 * PCT_THRESHOLD ) {
			     if (DEBUG)
			       System.out.printf("I");
		
			     pctTrend.put(i, INCR);
			   }
			   
			   else if ( pctMetric.get(i) < .9 * PCT_THRESHOLD ) {
			     if (DEBUG)
			       System.out.printf("N");
			     
			     pctTrend.put(i, NOTR);
			   }
			   
			   else if(pctMetric.get(i) <= PCT_THRESHOLD*1.1 && pctMetric.get(i) >= PCT_THRESHOLD*.9 ) {
			     if (DEBUG)
			       System.out.printf("U");
			     
			     pctTrend.put(i, UNCL);
			   }
		 }
		 if (DEBUG)
			 System.out.printf("\n");
		 
		 
	     Object object[] = new Object[1];
	     object[0] = pctTrend;
	    	
	     return object;
		 //return pctTrend;
		 
	}
	

	static Object[] getPdtTrend(Map<Integer, Double> pdtMetric, Map<Integer, Integer> pdtTrend, int pdtResultCnt ) {

		int i ;
		for (i=0; i <pdtResultCnt;i++ ) {
		    if ( pdtMetric.get(i) == 2  ) {
		    	if (DEBUG)
		    		System.out.printf("d");
		        
		    	pdtTrend.put(i, DISCARD) ;
		    }
		    else if ( pdtMetric.get(i) > 1.1 * PDT_THRESHOLD ) {
		    	if (DEBUG)
		    		System.out.printf("I");
		        
		    	pdtTrend.put(i, INCR) ;
		    }
		    else if ( pdtMetric.get(i) < .9 * PDT_THRESHOLD ) {
		    	if (DEBUG)
		    		System.out.printf("N");
		        
		    	pdtTrend.put(i, NOTR) ;
		    }
		    else if ( pdtMetric.get(i) <= PDT_THRESHOLD*1.1 && pdtMetric.get(i) >= PDT_THRESHOLD*.9 ) {
		    	if (DEBUG)
		    		System.out.printf("U");

		    	pdtMetric.put(i, (double) UNCL) ;
		    }
		 }

		if (DEBUG)
			System.out.printf("\n");
		
		
	     Object object[] = new Object[1];
	     object[0] = pdtTrend;
	    	
	     return object; 
		
		
		}
		
	

	/* 
	   Receive N streams .
	   After each stream, compute the loss rate.
	   Mark a stream "lossy" , if losss rate in 
	   that stream is more than a threshold.
	*/
	static long RecvFleet()
	{

		icFlag = false;
		int streamCnt = 0; /* 0->n*/
		int pktLost = 0;
		long firstTime;
		long currentTime;
		double pktLossRate;
		int expPktId = 0;
		int pktRcvd = 0;
		int numBursts;
		int lossyStream = 0;
		int totalPktRcvd = 0;
		boolean abortFleet = false;
		Map<Integer, Long> arrvTv = new TreeMap<Integer, Long>();
		Map<Integer, Long> sndTv = new TreeMap<Integer, Long>();
		Map<Integer, Long> owd = new TreeMap<Integer, Long>();
		Map<Integer, Long> owdfortd = new TreeMap<Integer, Long>();

		int numSubstream, j;
		ArrayList<Long> substream = new ArrayList<Long>(MAX_STREAM_LEN);
		ArrayList<Integer> numSndrCS = new ArrayList<Integer>(20);
		ArrayList<Integer> numRcvrCS = new ArrayList<Integer>(20);
		long high,low,len;
		Map<Integer, Integer>  B2BPktPerStream = new TreeMap<Integer, Integer>();
		int tmpB2B;
		double streamDuration ;
		Long returnVal = (long)-1;
		int return_val = 0;
		trendIDX = 0;

		if (DEBUG) { 
			System.out.printf("Receiving Fleet %d, Rate %.2fMbps\n",expFleetId, (double)tr);
		    System.out.printf("\nReceiving Fleet %d\n",expFleetId);
		    System.out.printf("  Fleet Parameter(req)  :: R=%.2fMbps, L=%dB, K=%dpackets, T=%fusec\n", (double) tr, cur_pkt_sz , streamLen, (double) timeInterval) ;
		    System.out.printf("  Lossrate per stream   :: ");
		}
		
		while (streamCnt < numStream) {
			pktLost = 0;
			firstTime =0;
			
			//System.out.println("streamCnt: "+ streamCnt + " numSream: " + numStream);
			
			arrvTv.clear();
			sndTv.clear();
			owd.clear();
			owdfortd.clear();
			substream.clear();
			
	        try {
				/*Receive K packets of ith stream*/
				Selector selector = Selector.open();
	            udpClientSocketChannel.configureBlocking(false);
				tcpSocketChannel.configureBlocking(false);
	            
	            tcpSocketChannel.register(selector, SelectionKey.OP_READ);
	            udpClientSocketChannel.register(selector, SelectionKey.OP_READ);

				while (true) {
	            	
	            	selector.select(1000);
	                Iterator it = selector.selectedKeys().iterator();
	
	                boolean finished = false;
	                boolean udpFinished = false;

					//process each key at a time
	                while (it.hasNext()) {
	
	    				ByteBuffer inBuffer = ByteBuffer.allocate(cur_pkt_sz);
	                    SelectionKey selectedKey = (SelectionKey)it.next();
	
	                    if (selectedKey.channel() == udpClientSocketChannel) {
		                	it.remove();
		                	
		                	//System.out.println("UDP!");
							currentTime = System.nanoTime();
	                        ((DatagramChannel) selectedKey.channel()).receive(inBuffer);
	                        inBuffer.flip();
	                        
	                        int inSize = inBuffer.remaining();
	                        
	                        byte[] retVal = new byte[inSize];
	                        inBuffer.get(retVal); /*pos 0, contains trainId*/			
	
//===========                            
    			        	Long t1 = (long)0;
	    			        byte[] data = new byte[20];
							for (int i=0; i<20; i++)
								data[i] = retVal[i];
    			        	
    				    	ByteArrayInputStream bis = new ByteArrayInputStream(data); 
    				    	DataInputStream dis = new DataInputStream(bis);
    				    	try {
	    	    					fleetId = dis.readInt();
	    	    					streamId = dis.readInt();
	    	    					pktId = dis.readInt();
	    				    		t1 = dis.readLong();
    						} catch (IOException e1) {
    							e1.printStackTrace();
    						}  
//============	                        
	                        
	    					//System.out.println("fleetId " + fleetId + " streamId " + streamId + " pktId " + pktId);
	    				    //System.out.println("expfleetId " + expFleetId + " streamCnt " + streamCnt + " expPktId " + expPktId + "\n");
	
	    			        if ( fleetId == expFleetId  && streamId == streamCnt && pktId >= expPktId ) {
	    			        	if (firstTime == 0)
	    			        		firstTime = currentTime;

	    						//System.out.println("MARIO: " + t1);
	    					      
	    					    arrvTv.put(pktId, currentTime);
	    			        	sndTv.put(pktId, t1); // time?
	
		    			        if ( pktId > expPktId ) {  /*reordered are considered as lost*/ 
		    			            pktLost += ( pktId - expPktId ) ;
		    			            expPktId  = pktId ;
		    			        }
		    			        ++expPktId;
		    			        ++pktRcvd;
		    			        
	    			        }
	    			        
	                    }
	                    if (selectedKey.channel() == tcpSocketChannel) {
		                	it.remove();
	                    	  //System.out.println("TCP!");
		                	  
		                      returnVal = RcvCtrMesg(tcpSocketChannel, 60*1000);
		                      
		                      if (!udpFinished ) {
		                    	  if ( (((returnVal & CTR_CODE) >> 30) == 1) && ((returnVal & 0x00ffffff) == FINISHED_STREAM ) ) 
		                    	  	udpFinished = true;
		                    	  	//System.out.println("UDP FINISHED!");
		                    	  
		                    	  	
	            	        	
		            	        	while ((returnVal = RcvCtrMesg(tcpSocketChannel, 60*1000)) == (long) -1);
	                			 
		            	        		
	            	        		if ( returnVal == streamCnt ) {
	            	        			finished = true;
	            	        			break ;
	            	        		}
		                    	  
		                     }
	            	        
		                }
	                }
	                
	                if (finished) {
	        	        selector.close();
	                	break;
	                }
				} // end of while(true)
				
	            
	        } catch (IOException e) {
	        	e.printStackTrace();
	        }


        	
	        //System.out.println("Receivded.." + (System.nanoTime() - MARIO)/1000/1000 + " msec");
	        //System.out.println("Before processing" + (System.nanoTime() - MARIO)/1000/1000 + " msec");
	        //System.out.println("I'm OUT!");
	        
	        //System.out.println("MARIO: " + arrvTv.size());
	        
	        for (j=0; j < streamLen; j++) {
	        	;
	        	if (arrvTv.containsKey(j)) {
	        		owd.put(j, (arrvTv.get(j) - sndTv.get(j))/1000);
	        		//System.out.println(arrvTv.get(j) + " " + sndTv.get(j) + " " + owd.get(j));
	        	}
	        	else {
	        		owd.put(j, (long) 0);
	        		arrvTv.put(j, (long) 0);
	        		sndTv.put(j, (long) 0);
	        	}
	        }
	        
	        totalPktRcvd += pktRcvd;
	        pktLost +=  streamLen  - expPktId ;
		    pktLossRate = pktLost * 100. / streamLen ;
		    if (DEBUG)
		    	System.out.printf(":%.1f",pktLossRate ) ;

		    expPktId = 0 ;
		    streamCnt++ ;
	    	
		    numBursts=0;
		    if ( interruptCoalescence ) {
		    	//icFlag=CheckIntrCoalescence(arrvTv,pktRcvd,numBursts);
				Object object[] = CheckIntrCoalescence(new ArrayList<Long>(arrvTv.values()),pktRcvd,numBursts);
				numBursts = Integer.parseInt( object[1].toString());
				icFlag = Boolean.parseBoolean(object[0].toString());
		    }

		    if ( pktLossRate < HIGH_LOSS_RATE && pktLossRate >= MEDIUM_LOSS_RATE )
			      lossyStream++ ;
			  
		    if ( pktLossRate >= HIGH_LOSS_RATE || ( streamCnt >= numStream
			         && lossyStream*100./streamCnt >= MAX_LOSSY_STREAM_FRACTION )) {

		    	if ( increaseStreamLen != 0 ) {
			        increaseStreamLen=0;
			        lowerBound=true;
			      }

			      if(DEBUG)
			        System.out.printf("\n  Fleet aborted due to high lossrate");
			      
			      abortFleet=true;
			      break;
			    }
		    else
		    {
		        /* analyze trend in stream */
		    	Object[] retObj;
		    	retObj = getSndrTimeInterval(new ArrayList<Long>(sndTv.values()), sndTimeInterval);
		    	num += Integer.parseInt( retObj[0].toString());
		    	sndTimeInterval = Long.parseLong( retObj[1].toString());
		  	    //System.out.println("\nnew_j: " + num + " sndTimeInterval: " + sndTimeInterval);
		    	
		    	retObj = adjustOffsetToZero(owd, streamLen);
		    	owd = (Map<Integer,Long>) retObj[0];

			    retObj = eliminateSndrSideCS(sndTv,substream);
			    numSubstream = Integer.parseInt( retObj[0].toString());
			    substream = (ArrayList<Long>) retObj[1];
			    //System.out.println("numSubstream: " + numSubstream + " streamCnt: " + streamCnt + " numBursts: " + numBursts + " icFlag: " + icFlag);
			    //System.out.println(substream);
			    
			    numSndrCS.add(streamCnt-1, numSubstream);
			    substream.add(numSubstream++, (long)streamLen-2);  // WHY -1 and not -2?
			    low=0;
			    numRcvrCS.add(streamCnt-1, 0);
			    tmpB2B=0;

			    for (j=0;j<numSubstream;j++) {
			        high=substream.get(j); 
			        if ( icFlag ) {
			        	 System.out.println("ICFLAG!!");
				         if ( numBursts < 2 ) {
				            if ( ++repeat_1 == 3) {
				            	repeat_1=0;
				                /*Abort fleet and try to find lower bound*/ 
				            	abortFleet=true;
				            	lowerBound=true;
				            	increaseStreamLen=0;
				            	break ;
				            }
				          }
				          else if ( numBursts <= 5 ) {
				            if ( ++repeat_2 == 3) {
				            	repeat_2=0;
				            	/*Abort fleet and retry with longer stream length */ 
				            	abortFleet=true;
				            	increaseStreamLen=1;
				            	break ;
				            }
				          }
				          else {
				        	  
				        	System.out.println("TOO MANY BURSTS?");
				            increaseStreamLen=0;
				            retObj = eliminateB2BpktIC(new ArrayList<Long>(arrvTv.values()), owd, owdfortd, low, high, numRcvrCS.get(streamCnt-1), tmpB2B);
				            len = Integer.parseInt( retObj[0].toString());
					        owdfortd = (Map<Integer,Long>) retObj[1];
					        numRcvrCS.set(streamCnt-1, Integer.parseInt( retObj[2].toString()));
					        tmpB2B = Integer.parseInt( retObj[2].toString());
				            
				            //for(int p=0;p<len;p++)
				            //  System.out.printf("%d %f\n",p,owdfortd.get(p));
				            
				            pctMetric.put(trendIDX, pairwiseComparisionTest(new ArrayList<Long>(owdfortd.values()), 0, (int) len));
				            pdtMetric.put(trendIDX, pairwiseDiffTest(new ArrayList<Long>(owdfortd.values()) , 0, (int) len));
				            trendIDX+=1;
				          }
			        }
			        else {
			        	
			          retObj = eliminateRcvrSideCS(new ArrayList<Long>(arrvTv.values()), owd, owdfortd, low, high, numRcvrCS.get(streamCnt-1),tmpB2B);
			          len = Integer.parseInt( retObj[0].toString());
			          owdfortd = (Map<Integer,Long>) retObj[1];
			          numRcvrCS.set(streamCnt-1, Integer.parseInt( retObj[2].toString()));
			          tmpB2B = Integer.parseInt( retObj[2].toString());
			          //System.out.println("owfordtd: " + owdfortd);
			          //System.out.println("owd: " + owd);
			          //System.out.println("len: " + len + " numRcvCS: " + numRcvrCS.get(streamCnt-1) + " tmpB2B: " +tmpB2B);
			          //System.out.println("\n");
			          			          
			          if ( len > MIN_STREAM_LEN ) 
			            getTrend(new ArrayList<Long>(owdfortd.values()), (int)len);
			        }
			        low=high+1;
			        
			    }

			    if ( abortFleet )
			    	break;
			   
			    else {
		    	  B2BPktPerStream.put(streamCnt-1, tmpB2B);
		          Integer ctrCode = CONTINUE_STREAM | CTR_CODE;
		          SendCtrMesg(tcpSocketChannel, ctrCode.toString());
			    }
		    }//else
		    
			pktRcvd = 0 ;
			    
		    /*A hack for slow links*/ 
		    streamDuration = streamLen * timeInterval ;
		    if ( streamDuration >= 500000 ) {
		      slow = true;
		      break ;
		    }
		    //System.out.println("\nAfter processing " + (System.nanoTime() - MARIO)/1000/1000 + " msec");
		    
		}  //end of while (stream_cnt < num_stream ). 


	   if ( DEBUG ) 
		  System.out.printf("\n");

	   if ( abortFleet ) {
	     System.out.printf("\tAborting fleet. Stream_cnt %d\n",streamCnt);
	     Integer ctrCode = ABORT_FLEET | CTR_CODE;
	     SendCtrMesg(tcpSocketChannel, ctrCode.toString()) ;
	     return_val = -1;
	   }
	   else
	      printContextswitchInfo(numSndrCS,numRcvrCS,B2BPktPerStream,streamCnt);

	  expFleetId++ ;
	  return return_val;
	  
	}
			
	
	static Object[] getSndrTimeInterval(ArrayList<Long> sndTv, Long sum)
	{
		
	  ArrayList<Long> timeInterval = new ArrayList<Long>();
	  Collections.sort(sndTv);
	  int j=0;
	  
	  for (int k=0; k < sndTv.size()-1; k++) {
		  if (sndTv.get(k) == 0 || sndTv.get(k+1) == 0)
			  continue;
		  else
			  timeInterval.add(j++, (sndTv.get(k+1) - sndTv.get(k))/1000);
	  }
	  
	  /* discard the top 15% as outliers  */
	  Collections.sort(timeInterval);
	  
	  int new_j = (int) (j - Math.rint(j*0.15)); 
	  for (int k=0; k < new_j; k++)
		  sum += timeInterval.get(k);
	  
  	Object object[] = new Object[2];
	object[0] = new_j;
	object[1] = sum;
	
	return object;
	  
	}
	
	
	/* Adjust offset to zero again  */
	static Object[] adjustOffsetToZero(Map<Integer,Long> owd, int len)
	{
		//what is this for??
	
	    long owd_min = 0;
	    int i ; 
	    for (i=0; i< len; i++) {
	        if ( owd_min == 0 && owd.get(i) != 0 ) owd_min=owd.get(i);
	        else if (owd_min != 0 && owd.get(i) != 0 && owd.get(i)<owd_min) owd_min=owd.get(i);
	    }

	    for (i=0; i< len; i++) {
	        if ( owd.get(i) != 0 )
	        	owd.put(i, owd.get(i) - owd_min);
	        
	    }
	    
    	Object object[] = new Object[1];
    	object[0] = owd;
    	
    	return object;
	    
	}
	
	
	/*
	  splits stream iff sender sent packets more than
	  time_interval+1000 usec apart.
	*/
	static Object[] eliminateSndrSideCS (Map<Integer, Long> sndrTimeStamp, ArrayList<Long> splitOWD)
	{
	  int j=0, k=0;
	  double cs_threshold;
	  splitOWD = new ArrayList<Long>();


	  cs_threshold = 2*timeInterval>timeInterval+1000 ? 2*timeInterval : timeInterval+1000;
	  for ( k = 0 ; k < streamLen-1 ; k++ )
	  {
		//long out = (sndrTimeStamp.get(k+1)-sndrTimeStamp.get(k))/1000;
		//System.out.println( out + " " + cs_threshold);
	    if ( sndrTimeStamp.get(k) == 0 || sndrTimeStamp.get(k+1) == 0 )
	       continue;

	    else if ((sndrTimeStamp.get(k+1)-sndrTimeStamp.get(k))/1000 > cs_threshold)
	      splitOWD.add(j++, (long)k);

	  }
	  //return j ;
	  
  	  Object object[] = new Object[2];
	  object[0] = j;
	  object[1] = splitOWD;
	
	  return object;
	  
	}
	

	/* eliminates packets received b2b due to IC */
	static Object[] eliminateB2BpktIC ( ArrayList<Long> rcvrTimeStamp, Map<Integer, Long> owd, Map<Integer, Long> owdfortd, long low, long high, int numRcvrCS, int tmpB2B )
	{
		
	  ArrayList<Long> B2Bpkt = new ArrayList<Long>(MAX_STREAM_LEN);
	  int i,k=0 ;
	  int len=0;
	  long minGap;
	  int tmp=0;

	  minGap = MIN_TIME_INTERVAL > 3*rcv_latency ? MIN_TIME_INTERVAL : 3*rcv_latency ;
	  for ( i = (int) low ; i <= (int) high  ; i++ )
	  {
	    if ( rcvrTimeStamp.get(i) == 0 || rcvrTimeStamp.get(i+1) == 0 )
	      continue ;
	    
	    //fprintf(stderr,"i %d  owd %.2f dispersion %.2f",i, owd[i],rcvr_time_stamp[i+1]- rcvr_time_stamp[i]);
	    if ((rcvrTimeStamp.get(i+1)- rcvrTimeStamp.get(i)) < minGap) {
	      B2Bpkt.add(k++, (long)i);
	      tmp++;
	    }
	    else {
	      if ( tmp >= 3 ) {
	        //fprintf(stderr," j\n");
	        tmp=0;
	        owdfortd.put(len++, owd.get(i));
	      }
	    }
	  }
	  //System.out.println(numRcvrCS);
	  //System.out.println(B2Bpkt);
	  
  	  Object object[] = new Object[4];
	  object[0] = len;
	  object[1] = owdfortd;
	  object[2] = numRcvrCS;
	  object[3] = tmpB2B;
	
	  return object;
	  //return len ;
	  
	}
	
	
	/*
    PCT test to detect increasing trend in stream */
	static double pairwiseComparisionTest (ArrayList<Long> array, int start, int end)
	{
	  int improvement = 0 ,i ;
	  long total ;
	
	  if ( ( end - start  ) >= MIN_PARTITIONED_STREAM_LEN )
	  {
	    for ( i = start ; i < end - 1   ; i++ )
	    {
	      if ( array.get(i) < array.get(i+1) )
	        improvement += 1 ;
	    }
	    total = ( end - start ) ;
	    double out = (double) improvement/ (double) total;
	    //System.out.println("pairwiseComparisonTest: " + improvement + " " + total + " " + out);
	    return out;
	  }
	  else
	    return -1 ;
	}
	
	
	/*
    PDT test to detect increasing trend in stream
	*/
	static double pairwiseDiffTest(ArrayList<Long> array, int start, int end)
	{
	  //System.out.println("pairwiseDiffTest: array: " + array);
	  long y = 0 , y_abs = 0 ;
	  int i ;
	  if ( ( end - start  ) >= MIN_PARTITIONED_STREAM_LEN )
	  {
	    for ( i = start+1 ; i < end    ; i++ )
	    {
	      y += array.get(i) - array.get(i-1);
	      y_abs += Math.abs(array.get(i) - array.get(i-1)) ;
	    }
	    
	    if (y_abs != 0) {
	    	double out = (double) y/ (double) y_abs;
	    	return out;
	    }
	    else
	    	return 0;
	  }
	  else
	    return 2. ;
	}
	
	
	/*
	  discards owd of packets received when
	  receiver was NOT running.
	*/
	static Object[] eliminateRcvrSideCS ( ArrayList<Long> rcvrTimeStamp, Map<Integer, Long> owd, Map<Integer, Long> owdfortd, long low, long high, int numRcvrCS,long tmpB2B) 
	{
	  ArrayList<Long> B2Bpkt = new ArrayList<Long>(MAX_STREAM_LEN) ;
	  int i,k=0 ;
	  int len=0;
	  double minGap;
	  owdfortd = new TreeMap<Integer, Long>();

	  //System.out.println("low " + low + " high " + high);
	  //System.out.println("rcvrTimeStamp.size: " + rcvrTimeStamp.size() + " owd.size " + owd.size());
	  
	  minGap = MIN_TIME_INTERVAL > 1.5*rcv_latency ? MIN_TIME_INTERVAL :2.5*rcv_latency ;

	  for ( i = (int) low ; i <= high  ; i++ )
	  {
	    if ( rcvrTimeStamp.get(i) == 0 || rcvrTimeStamp.get(i+1) == 0 )
	      continue ;
	    else if (((rcvrTimeStamp.get(i+1) - rcvrTimeStamp.get(i))/1000) > minGap)
	      owdfortd.put(len++, owd.get(i));
	    else 
	      B2Bpkt.add(k++, (long) i);
	  }

	  /* go through discarded list and count b2b discards as 1 CS instance */
	  for (i=1;i<k;i++)
	    if ( B2Bpkt.get(i)-B2Bpkt.get(i-1) != 1)
	      numRcvrCS++;
	  tmpB2B += k;
	  
  	  Object object[] = new Object[4];
	  object[0] = len;
	  object[1] = owdfortd;
	  object[2] = numRcvrCS;
	  object[3] = tmpB2B;
	
	  return object;
	  //return len ;
	}
	
	
	static void getTrend(ArrayList<Long> owdfortd, int pktCnt )
	{
	  ArrayList<Long> medianOWD = new ArrayList<Long>(MAX_STREAM_LEN);
	  int medianOWDlen=0;
	  ArrayList<Long> ordered = new ArrayList<Long>();
	  int j, count; 
	  int pktPerMin;
	  
	  pktPerMin = (int) Math.floor(Math.sqrt(pktCnt)); 
	
	  count = 0 ;
	  for ( j=0 ; j<pktCnt; j=j+ pktPerMin )
	  {
	    if ( j+pktPerMin >= pktCnt )
	      count = pktCnt - j ;
	    else
	      count = pktPerMin;
	    
	    List<Long> owdfortdSubstr = owdfortd.subList(j, j+count);
	    Collections.sort(owdfortdSubstr);
	    ordered = new ArrayList<Long>(owdfortdSubstr);
	    
	    //System.out.println("j: " + j + " count: " + count + " pktCnt: " + pktCnt + " pktPerMin: " + pktPerMin);
	    //System.out.println(ordered);
	    //Collections.sort(ordered);
	    
	    if ( count % 2 == 0 )
	       medianOWD.add(medianOWDlen++, ( ordered.get((int)(count*.5) -1) + ordered.get((int)(count*0.5)) ) /2 );
	    else
	       medianOWD.add(medianOWDlen++, ordered.get((int)(count*0.5)) );
	  }
	  
	  pctMetric.put(trendIDX, pairwiseComparisionTest(medianOWD, 0, medianOWDlen));
	  pdtMetric.put(trendIDX, pairwiseDiffTest(medianOWD, 0, medianOWDlen));
	  //System.out.println("pctMetric: " + pctMetric);
	  //System.out.println("pdtMetric: " + pdtMetric);
	  
	  trendIDX+=1;
	}
	
	
	
	static void printContextswitchInfo(ArrayList<Integer> numSndrCS, ArrayList<Integer> numRcvrCS, Map<Integer, Integer> discard, int streamCnt)
	{
	    int j;

	    if (DEBUG)
	      System.out.printf("  # of CS @ sndr        :: ");
	    
	    for(j=0;j<streamCnt-1;j++) {
	      if (DEBUG) 
	    	  System.out.printf(":%2d",numSndrCS.get(j));
	    }
	    if ( DEBUG ) { 
	    	System.out.printf("\n");
	      	System.out.printf("  # of CS @ rcvr        :: ");
	    }
	    
	    for(j=0;j<streamCnt-1;j++) {
	      if (DEBUG) 
	    	  System.out.printf(":%2d",numRcvrCS.get(j));
	    }
	    
	    if ( DEBUG ) {
	    	System.out.printf("\n");
	      	System.out.printf("  # of DS @ rcvr        :: ");
	    }

	    for(j=0;j<streamCnt-1;j++) {
	      if (DEBUG) 
	    	  System.out.printf(":%2d",discard.get(j));
	    }
	     
	    if ( DEBUG ) 
	    	System.out.printf("\n");
	}
	

	/*
	   if a approx-equal b, return 1
	   else 0
	*/
	static boolean equal(double a , double b)
	{
	  double maxdiff ;
	  double greater = a<b?a:b;
	  if ( greater < 500 ) 
		  maxdiff = 2.5;
	  else maxdiff = 5 ;
	  if ( Math.abs( a - b ) / b <= .02  && Math.abs(a-b) < maxdiff )
	    return true ;
	  else
	    return false;
	}
	
	static boolean less_than(double a, double b)
	{
	  if ( !equal(a,b) && a < b)
	    return true;
	  else 
	    return false;
	}
	
	static boolean grtr_than(double a, double b)
	{
	  if ( !equal(a,b) && a > b)
	    return true;
	  else 
	    return false;
	}
	
	
}
    		  
