package edu.northwestern.pathload;

import java.util.*;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.*;
import java.nio.ByteBuffer;
import java.nio.channels.DatagramChannel;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import edu.northwestern.dasu.Main;
import edu.northwestern.dasu.util.PluginInterface;
import static edu.northwestern.pathload.NetworkPrimitives.*;

public class PathloadSender extends Thread {

	protected static ServerSocketChannel tcpServerSocketChannel;
	protected static SocketChannel tcpSocketChannel;
	protected static DatagramChannel udpClientChannel;
	protected static InetAddress rcvUdpAddress = null;

	protected static int fleetId = 0;
	protected static int minSleepInterval = 0; /* usec */
	protected static int minTimerIntr = 0; /* usec */
	protected static Long gettimeofday_latency; /* usec */

	private static PathloadSender self;
	private boolean isActive;
	private PluginInterface pi;
	private static final boolean DEBUG = true;

	public PathloadSender() {
		super("PathloadManager");
		this.pi = Main.getPluginInterface();
		self = this;
		isActive = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Thread#start()
	 */
	@Override
	public synchronized void start() {
		// TODO Auto-generated method stub
		super.start();
	}
    
	/**
	 * The static interface to the OessExperiment.
	 * 
	 * @return
	 */
	public synchronized static PathloadSender getInstance() {
		if (self == null) {
			self = new PathloadSender();
			self.setDaemon(true);
			return self;
		} else {
			return self;
		}
	}
    
	public void setActive(boolean isActive) {
		this.isActive = isActive;
		if (!isActive) {
			self = null;
		}
	}
		
	
	private static ServerSocketChannel openControlChannel() {

		ServerSocketChannel ssChannel = null;

		try {
			ssChannel = ServerSocketChannel.open();
			ssChannel.configureBlocking(true);
			ssChannel.socket().setReuseAddress(true);
			ssChannel.socket().bind(new InetSocketAddress(TCPSND_PORT));

			ServerSocket tcpServerSocket = ssChannel.socket();
			if (DEBUG) {
				System.out.println("DEBUG:: Address: "
						+ tcpServerSocket.getInetAddress() + ", Port: "
						+ tcpServerSocket.getLocalPort() + ", AddressReuse?: "
						+ tcpServerSocket.getReuseAddress() + ", BufferSize: "
						+ tcpServerSocket.getLocalPort() + "\n");
			}
		} catch (IOException e) {
			System.err.println("Could not bind to TCP port: " + TCPSND_PORT);
			// System.exit(-1);
		}
		return ssChannel;
	}

	/* Data stream: UDP channel */
	private static DatagramChannel openDataChannel() {

		DatagramChannel dChannel = null;

		try {
			dChannel = DatagramChannel.open();
			dChannel.socket().setSendBufferSize(UDP_BUFFER_SZ);
			dChannel.socket().setReceiveBufferSize(UDP_BUFFER_SZ);

			System.out.print("UDP buffer established!, BufferSz: "
					+ dChannel.socket().getSendBufferSize() + "\n");

			dChannel.socket().setReuseAddress(true);

		} catch (SocketException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return dChannel;
	}

	public void run() {

		numStream = NUM_STREAM;
		gettimeofday_latency = getTimeOfDayLatency();

		/* Control stream: TCP connection */
		tcpServerSocketChannel = openControlChannel();

		/* Data stream: UDP socket */
		udpClientChannel = openDataChannel();

		do {
			if (!quiet) {
				System.out
						.println("Waiting for receiver to establish control stream => ");
			}

			/*
			 * Wait until receiver attempts to connect, starting new measurement
			 * cycle
			 */
			try {

				do {
					if (tcpServerSocketChannel.isOpen()) {
						tcpServerSocketChannel.close();
						tcpServerSocketChannel = openControlChannel();
					}
					if (udpClientChannel.isOpen()) {
						udpClientChannel.disconnect();
						udpClientChannel.close();
						udpClientChannel = openDataChannel();
					}

					if (!udpClientChannel.isOpen()
							|| !tcpServerSocketChannel.socket().isBound()) {
						System.out
								.println("Network problem... retrying in 30 secs...");
						Thread.sleep(30 * 1000);
					}
				} while (!udpClientChannel.isOpen()
						|| !tcpServerSocketChannel.socket().isBound());

				tcpSocketChannel = tcpServerSocketChannel.accept();
				tcpSocketChannel.socket().setTcpNoDelay(true);

				if (!quiet)
					System.out.println("OK");

				/* CHANGE */// rcvUdpAddress =
							// tcpServerSocketChannel.socket().getInetAddress();
				// rcvUdpAddress = InetAddress.getLocalHost();
				InetSocketAddress m = (InetSocketAddress) tcpSocketChannel
						.socket().getRemoteSocketAddress();
				if (m.getAddress().toString().equals("/127.0.0.1"))
					rcvUdpAddress = InetAddress.getLocalHost();
				else
					rcvUdpAddress = m.getAddress();

				if (!quiet) {
					java.util.Date localtm = new java.util.Date();
					System.out.println("Receiver " + rcvUdpAddress + " "
							+ " starts measurements on " + localtm);
				}

				/* Connect UDP socket */
				udpClientChannel.connect(new InetSocketAddress(rcvUdpAddress,
						UDPRCV_PORT));
				udpClientChannel.socket().setReuseAddress(true);

				// Assume Ethernet sized MTU for sender and receiver (1472)
				Long send_time = SendLatency();
				send_time = send_time / 1000; /* convert to microseconds */

				/* tell receiver our send latency */
				System.out.println("Send latency: " + send_time);
				SendCtrMesg(tcpSocketChannel, send_time.toString());

				/* wait for receiver to start ADR measurement */
				long retVal = RcvCtrMesg(tcpSocketChannel, 5 * 1000);

				if (retVal == -1) {
					System.out.println("error, receiver did not respond");
					break;
				}

				if ((((retVal & CTR_CODE) >> 30) == 1)
						&& ((retVal & 0x00ffffff) == SEND_TRAIN)) {
					if (!quiet) {
						System.out
								.println("Estimating ADR to initialize rate adjustment algorithm => ");
					}
					if (sendTrain() == -1)
						continue;
					if (!quiet)
						System.out.println("Done");
				}
				fleetId = 0;
				boolean done = false;

				/* Start avail-bw measurement */
				while (!done) {

					if ((retVal = RcvCtrMesg(tcpSocketChannel, 5 * 1000)) < 0)
						break;

					if ((((retVal & CTR_CODE) >> 30) == 1)
							&& ((retVal & 0x00ffffff) == TERMINATE)) {
						if (!quiet)
							System.out.println("Terminating current run.\n");

						done = true;
					} else {

						transmissionRate = retVal;
						// System.out.println("MARIO: transmissionRate: " +
						// transmissionRate);

						if ((retVal = RcvCtrMesg(tcpSocketChannel, 60 * 1000)) <= 0)
							break;
						cur_pkt_sz = (int) retVal;
						// System.out.println("MARIO: cur_pkt_sz: " +
						// cur_pkt_sz);

						//tcpSocketChannel.configureBlocking(false);
						if ((retVal = RcvCtrMesg(tcpSocketChannel, 60 * 1000)) <= 0)
							break;
						streamLen = (int) retVal;
						// System.out.println("MARIO: streamLen: " + streamLen);

						if ((retVal = RcvCtrMesg(tcpSocketChannel, 60 * 1000)) <= 0)
							break;
						timeInterval = retVal;
						// System.out.println("MARIO: timeInterval: " +
						// timeInterval);

						if ((retVal = RcvCtrMesg(tcpSocketChannel, 60 * 1000)) <= 0)
							break;

						/* ret_val = SEND_FLEET */
						Integer ctrCode = RECV_FLEET | CTR_CODE;
						// System.out.println("MARIO: SENDFLEET!");

						SendCtrMesg(tcpSocketChannel, ctrCode.toString());

						if (SendFleet() == -1)
							break;

						if (!quiet)
							System.out.println();

						fleetId++;
					}
				}

			} catch (IOException e) {
				System.err.println("TCP.accept() / UDP.connect() Error!");
			} catch (InterruptedException e) {
				System.err.println("Thread sleep problem!");
			}

		} while (true);

	}

	// pathload_snd_func

	/*
	 * Send fleet for avail-bw estimation. pkt_id and fleet_id start with 0.
	 */
	public static int SendFleet() {

		int sleepTime;
		Long t1 = (long) 0, t2 = (long) 0, tmp = (long) 0;
		Integer ctrCode;
		int pktCnt = 0;
		int streamCnt = 0;
		int sleepTmUsec;

		int streamDuration;
		Long diff;

		/* Create random payload; does it matter? */
		byte[] packBuf = new byte[cur_pkt_sz];
		for (int i = 0; i < cur_pkt_sz; i++) {
			packBuf[i] = (byte) i;
		}

		if (!quiet)
			System.out.printf("Sending fleet %d ", fleetId);

		while (streamCnt < numStream) {
			if (!quiet)
				System.out.printf("#");

			ByteBuffer buffer = ByteBuffer.allocateDirect(cur_pkt_sz);

			/*
			 * packBuf[0] = (byte) fleetId; packBuf[1] = (byte) streamCnt;
			 */
			t1 = System.nanoTime();

			for (pktCnt = 0; pktCnt < streamLen; pktCnt++) {

				// System.out.printf("SendFleet: Sending Fleetid %d, streamCnt %d, pktCnt %d\n",
				// fleetId, streamCnt, pktCnt);
				// System.out.println("t1: "+t1);
				// packBuf[2] = (byte) pktCnt;

				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				DataOutputStream dos = new DataOutputStream(bos);
				try {
					/**/dos.writeInt(fleetId);
					/**/dos.writeInt(streamCnt);
					/**/dos.writeInt(pktCnt);
					dos.writeLong(t1);
				} catch (IOException e1) {
					e1.printStackTrace();
				}
				byte[] data = bos.toByteArray();

				/*
				 * for (int i=0; i<8; i++){ packBuf[3+i] = data[i]; }
				 */

				for (int i = 0; i < 20; i++) {
					packBuf[i] = data[i];
				}

				buffer.clear();
				buffer.put(packBuf);
				buffer.flip();

				try {

					int numBytesWritten = 0;
					numBytesWritten = udpClientChannel.write(buffer);

					if (numBytesWritten == 0) {
						System.out.println("UDP write error!");
					}

				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return 1;
				}

				tmp = (System.nanoTime() - t1) / 1000; /* usec */

				if (pktCnt < (streamLen - 1)) {

					double tmRemaining = timeInterval - tmp;
					// System.out.println("tmp: " + tmp + " tmRemaining: " +
					// tmRemaining + " timeInterval: " + timeInterval);
					// System.out.println("pktCnt: " + pktCnt + " streamLen: " +
					// streamLen);
					if (tmRemaining > minSleepInterval) {
						sleepTmUsec = tmRemaining
								- (tmRemaining % minTimerIntr) - minTimerIntr < 200 ? 2 * minTimerIntr
								: minTimerIntr;

						sleepTime = sleepTmUsec / 1000; // millis
						// HUH?
						// sleep_time.tv_sec = (int)(sleep_tm_usec / 1000000) ;
						// sleep_time.tv_usec = sleep_tm_usec -
						// sleep_time.tv_sec*1000000 ;
						try {
							Thread.sleep(sleepTime);
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}

					t2 = System.nanoTime();
					diff = gettimeofday_latency > 0 ? gettimeofday_latency - 1 : 0;
					// System.out.println("gettimeofda_latency: " +
					// gettimeofday_latency + " diff: " + diff);
					// System.out.println((t2-t1)/1000);
					while ((t2 - t1) / 1000 < (timeInterval - diff)) {
						t2 = System.nanoTime();
					}

					t1 = t2;
				}
			}

			/*
			 * Wait for 2000 usec and send End of stream message along with
			 * streamid.
			 */

			try {
				// Thread.sleep(2000);
				Thread.sleep(20);
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			// while((t2 - t1) < 8000 ) ;

			ctrCode = FINISHED_STREAM | CTR_CODE;

			SendCtrMesg(tcpSocketChannel, ctrCode.toString());
			Integer s = streamCnt;
			SendCtrMesg(tcpSocketChannel, s.toString());

			/* Wait for continue/cancel message from receiver. */
			long retVal;

			retVal = RcvCtrMesg(tcpSocketChannel, 5 * 1000);

			if ((((retVal & CTR_CODE) >> 30) == 1)
					&& ((retVal & 0x00ffffff) == CONTINUE_STREAM)) {
				streamCnt++;
			} else if ((((retVal & CTR_CODE) >> 30) == 1)
					&& ((retVal & 0x00ffffff) == ABORT_FLEET)) {
				System.out.println("ABORTNG! last pktId " + pktCnt);
				return 0;
			}

			/* inter-stream latency is max (RTT,9*stream_duration) */
			streamDuration = (int) (streamLen * timeInterval);
			if (t2 - t1 < streamDuration * 9) {
				/*
				 * release cpu if inter-stream gap is longer than min_sleep_time
				 */
				if (t2 - t1 - streamDuration * 9 > minSleepInterval) {
					sleepTmUsec = (int) (timeInterval - tmp
							- ((timeInterval - tmp) % minSleepInterval) - minSleepInterval);

					try {
						Thread.sleep(sleepTmUsec);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					t2 = System.nanoTime();
				}

				/* busy wait for the remaining time */
				while (System.nanoTime() - t1 < streamDuration * 9)
					;
			}

			/* A hack for slow links */
			if (streamDuration >= 500000)
				break;
		}

		return 0;
	}

	public static Long SendLatency() {

		long latency = -1;
		DatagramSocket udpRcvSocket = null;
		DatagramSocket udpSndSocket = null;
		InetSocketAddress rcvAdd;

		try {

			InetAddress IPAddress = InetAddress.getLocalHost();
			udpRcvSocket = new DatagramSocket(null);
			rcvAdd = new InetSocketAddress(IPAddress, 9876);
			udpRcvSocket.bind(rcvAdd);
			System.out.println("Bound to UDP port "
					+ udpRcvSocket.getLocalPort() + " at " + IPAddress);

			udpSndSocket = new DatagramSocket(null);

			/* Create random payload; does it matter? */
			byte[] sendData = new byte[max_pkt_sz];
			for (int i = 0; i < max_pkt_sz; i++) {
				sendData[i] = (byte) i;
			}

			DatagramPacket sendPacket = new DatagramPacket(sendData,
					sendData.length, IPAddress, 9876);

			byte[] receiveData = new byte[max_pkt_sz];
			DatagramPacket receivePacket = new DatagramPacket(receiveData,
					receiveData.length);

			long first_time, current_time;
			ArrayList<Long> min_OSdelta = new ArrayList<Long>();
			for (int i = 0; i < 50; i++) {
				first_time = System.nanoTime();
				udpSndSocket.send(sendPacket);
				current_time = System.nanoTime();
				udpRcvSocket.receive(receivePacket);
				min_OSdelta.add(current_time - first_time);
			}

			/* Use median of measured latencies to avoid outliers */
			Collections.sort(min_OSdelta);
			latency = min_OSdelta.get(25);
			
			udpRcvSocket.close();
			System.out.println(min_OSdelta);

		} catch (SocketException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return latency;
	}

	/*
	 * get ADR
	 */
	private static int sendTrain() {

		int trainId = 0;
		int packId;
		Integer ctrCode;
		Long retVal;
		int trainLen = 0;

		/* Create random payload; does it matter? */
		byte[] packBuf = new byte[max_pkt_sz];
		for (int i = 0; i < max_pkt_sz; i++) {
			packBuf[i] = (byte) i;
		}

		ByteBuffer buf = ByteBuffer.allocateDirect(max_pkt_sz);

		while (trainId < MAX_TRAIN) {
			if (trainLen == 5)
				trainLen = 3;
			else
				trainLen = TRAIN_LEN - trainId * 15;

			for (packId = 0; packId <= trainLen; packId++) {
				packBuf[0] = (byte) trainId;
				packBuf[1] = (byte) packId;
				buf.clear();
				buf.put(packBuf);
				buf.flip();

				try {

					int numBytesWritten = 0;
					// if (packId != 10) {
					numBytesWritten = udpClientChannel.write(buf);
					// System.out.println(" trainId: " + trainId + " packId: " +
					// packId);
					// }

					if (numBytesWritten == 0) {
						System.out.println("UDP write error!");
					}

				} catch (IOException e) {
					System.out.println("UDP: Destination port unreachable!!!");
					return 1;
				}
			}
			// System.out.println(" trainId: " + trainId + " packId: " +
			// packId);

			try {
				// System.out.println("Sleep: " + System.currentTimeMillis());
				Thread.sleep(10, 0); // 1000 microseconds = 1 millisecond
										// //MARIO
				// System.out.println("Sleep: " + System.currentTimeMillis());

			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			ctrCode = FINISHED_TRAIN | CTR_CODE;
			SendCtrMesg(tcpSocketChannel, ctrCode.toString());

			retVal = Long.valueOf(RcvCtrMesg(tcpSocketChannel, 5 * 1000))
					.longValue();
			;

			if (retVal == null) {
				System.out.println("error, receiver did not respond");
				break;
			}

			if ((((retVal & CTR_CODE) >> 30) == 1)
					&& ((retVal & 0x00ffffff) == BAD_TRAIN)) {
				if (!quiet) {
					System.out.println("Bad Train");
					trainId++;
					continue;
				}
			} else {
				System.out.println("Good Train");
				return 0;
			}
		}

		return 0;
	}

	/*
	 * Receive a message from the control stream
	 * 
	 * public static String RcvCtrMesg(SocketChannel tcpSocketChannel, int
	 * timeout) {
	 * 
	 * ByteBuffer buf = ByteBuffer.allocateDirect(1024); String ret = null; try
	 * { buf.clear(); tcpSocketChannel.socket().setSoTimeout(timeout); int
	 * numBytesRead = tcpSocketChannel.read(buf);
	 * 
	 * if (numBytesRead == -1) { // No more bytes can be read from the channel
	 * tcpSocketChannel.close(); } else { // To read the bytes, flip the buffer
	 * buf.flip(); }
	 * 
	 * ret = decoder.decode(buf).toString();
	 * 
	 * } catch (SocketTimeoutException e) { buf = null; e.printStackTrace();
	 * 
	 * } catch (IOException e1) { // Connection may have been closed
	 * e1.printStackTrace(); }
	 * 
	 * return ret;
	 * 
	 * }
	 */

	/*
	 * Control stream: TCP connection private static ServerSocket
	 * openControlSocket() throws SocketException {
	 * 
	 * ServerSocket tcpServerSocket = null;
	 * 
	 * try { tcpServerSocket = new ServerSocket(TCPSND_PORT);
	 * tcpServerSocket.setReuseAddress(true);
	 * System.out.print("Server has connected!\n"); if (DEBUG) {
	 * System.out.println("DEBUG:: Address: " + tcpServerSocket.getInetAddress()
	 * + ", Port: " + tcpServerSocket.getLocalPort() + ", AddressReuse?: " +
	 * tcpServerSocket.getReuseAddress() + ", BufferSize: " +
	 * tcpServerSocket.getLocalPort() + "\n"); }
	 * 
	 * } catch (IOException e) {
	 * System.err.println("Could not bind to TCP port: " + TCPSND_PORT);
	 * System.exit(-1); } return tcpServerSocket;
	 * 
	 * }
	 */

	/*
	 * Data stream: UDP socket private static DatagramSocket openDataSocket() {
	 * 
	 * DatagramSocket udpClientSocket = null; try { udpClientSocket = new
	 * DatagramSocket(); int udpBufferSz = UDP_BUFFER_SZ;
	 * udpClientSocket.setSendBufferSize(udpBufferSz);
	 * System.out.print("UDP buffer established!, SendBufferSz: " +
	 * udpClientSocket.getSendBufferSize() + "\n");
	 * 
	 * } catch (IOException e) { //TODO Auto-generated catch block
	 * e.printStackTrace(); }
	 * 
	 * return udpClientSocket;
	 * 
	 * }
	 */

	/*
	 * Send a message through the control stream
	 * 
	 * public static int SendCtrMesg(SocketChannel tcpServerSocketChannel,
	 * String ctr_code) {
	 * 
	 * byte[] ctr_buff = ctr_code.getBytes(); int numBytesWritten = -1; try {
	 * ByteBuffer buf = ByteBuffer.allocateDirect(1024); buf.put(ctr_buff);
	 * buf.flip(); numBytesWritten = tcpServerSocketChannel.write(buf); } catch
	 * (IOException e) { // TODO Auto-generated catch block e.printStackTrace();
	 * } return numBytesWritten; }
	 */

}