package edu.uc.ndt;

/*
Copyright 2003 University of Chicago.  All rights reserved.
The Web100 Network Diagnostic Tool (NDT) is distributed subject to
the following license conditions:
SOFTWARE LICENSE AGREEMENT
Software: Web100 Network Diagnostic Tool (NDT)

1. The "Software", below, refers to the Web100 Network Diagnostic Tool (NDT)
(in either source code, or binary form and accompanying documentation). Each
licensee is addressed as "you" or "Licensee."

2. The copyright holder shown above hereby grants Licensee a royalty-free
nonexclusive license, subject to the limitations stated herein and U.S. Government
license rights.

3. You may modify and make a copy or copies of the Software for use within your
organization, if you meet the following conditions: 
    a. Copies in source code must include the copyright notice and this Software
    License Agreement.
    b. Copies in binary form must include the copyright notice and this Software
    License Agreement in the documentation and/or other materials provided with the copy.

4. You may make a copy, or modify a copy or copies of the Software or any
portion of it, thus forming a work based on the Software, and distribute copies
outside your organization, if you meet all of the following conditions: 
    a. Copies in source code must include the copyright notice and this
    Software License Agreement;
    b. Copies in binary form must include the copyright notice and this
    Software License Agreement in the documentation and/or other materials
    provided with the copy;
    c. Modified copies and works based on the Software must carry prominent
    notices stating that you changed specified portions of the Software.

5. Portions of the Software resulted from work developed under a U.S. Government
contract and are subject to the following license: the Government is granted
for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable
worldwide license in this computer software to reproduce, prepare derivative
works, and perform publicly and display publicly.

6. WARRANTY DISCLAIMER. THE SOFTWARE IS SUPPLIED "AS IS" WITHOUT WARRANTY
OF ANY KIND. THE COPYRIGHT HOLDER, THE UNITED STATES, THE UNITED STATES
DEPARTMENT OF ENERGY, AND THEIR EMPLOYEES: (1) DISCLAIM ANY WARRANTIES,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO ANY IMPLIED WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE OR NON-INFRINGEMENT,
(2) DO NOT ASSUME ANY LEGAL LIABILITY OR RESPONSIBILITY FOR THE ACCURACY,
COMPLETENESS, OR USEFULNESS OF THE SOFTWARE, (3) DO NOT REPRESENT THAT USE
OF THE SOFTWARE WOULD NOT INFRINGE PRIVATELY OWNED RIGHTS, (4) DO NOT WARRANT
THAT THE SOFTWARE WILL FUNCTION UNINTERRUPTED, THAT IT IS ERROR-FREE OR THAT
ANY ERRORS WILL BE CORRECTED.

7. LIMITATION OF LIABILITY. IN NO EVENT WILL THE COPYRIGHT HOLDER, THE
UNITED STATES, THE UNITED STATES DEPARTMENT OF ENERGY, OR THEIR EMPLOYEES:
BE LIABLE FOR ANY INDIRECT, INCIDENTAL, CONSEQUENTIAL, SPECIAL OR PUNITIVE
DAMAGES OF ANY KIND OR NATURE, INCLUDING BUT NOT LIMITED TO LOSS OF PROFITS
OR LOSS OF DATA, FOR ANY REASON WHATSOEVER, WHETHER SUCH LIABILITY IS ASSERTED
ON THE BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE OR STRICT LIABILITY), OR
OTHERWISE, EVEN IF ANY OF SAID PARTIES HAS BEEN WARNED OF THE POSSIBILITY OF
SUCH LOSS OR DAMAGES.
The Software was developed at least in part by the University of Chicago,
as Operator of Argonne National Laboratory (http://miranda.ctd.anl.gov:7123/). 
 */
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.JOptionPane;

import edu.northwestern.dasu.drools.FactNdtProbeResult;

public class Tcpbw100 {
  private static final String VERSION = "3.5.6";
  private static final byte TEST_MID = (1 << 0);
  private static final byte TEST_C2S = (1 << 1);
  private static final byte TEST_S2C = (1 << 2);
  private static final byte TEST_SFW = (1 << 3);
  private static final byte TEST_STATUS = (1 << 4);

  private static final byte SRV_QUEUE     = 1;
  private static final byte MSG_LOGIN     = 2;
  private static final byte TEST_PREPARE  = 3;
  private static final byte TEST_START    = 4;
  private static final byte TEST_MSG      = 5;
  private static final byte TEST_FINALIZE = 6;
  private static final byte MSG_ERROR     = 7;
  private static final byte MSG_RESULTS   = 8;
  private static final byte MSG_LOGOUT    = 9;
  private static final byte MSG_WAITING   = 10;

  private static final int SFW_NOTTESTED  = 0;
  private static final int SFW_NOFIREWALL = 1;
  private static final int SFW_UNKNOWN    = 2;
  private static final int SFW_POSSIBLE   = 3;

  FactNdtProbeResult ret = new FactNdtProbeResult();

  private static final double VIEW_DIFF = 0.1;
  private static final boolean DEBUG = false;

  String[] delays = { "immediate", "1 min", "5 mins", "10 mins", "30 mins", "2 hours", "12 hours", "1 day" };

  boolean Randomize, failed, cancopy;
  URL location;
  String s;
  double t;
  int ECNEnabled, NagleEnabled, MSSSent, MSSRcvd;
  int SACKEnabled, TimestampsEnabled, WinScaleRcvd, WinScaleSent;
  int FastRetran, AckPktsOut, SmoothedRTT, CurrentCwnd, MaxCwnd;
  int SndLimTimeRwin, SndLimTimeCwnd, SndLimTimeSender;
  int SndLimTransRwin, SndLimTransCwnd, SndLimTransSender, MaxSsthresh;
  int SumRTT, CountRTT, CurrentMSS, Timeouts, PktsRetrans;
  int SACKsRcvd, DupAcksIn, MaxRwinRcvd, MaxRwinSent;
  int DataPktsOut, Rcvbuf, Sndbuf, AckPktsIn, DataBytesOut;
  int PktsOut, CongestionSignals, RcvWinScale;
  int pkts, lth=8192, CurrentRTO;
  int c2sData, c2sAck, s2cData, s2cAck;
  // added for mailto url
  protected URL targetURL;
  private String TARGET1 = "U";
  private String TARGET2 = "H";
  double s2cspd, c2sspd, sc2sspd, ss2cspd;
  int ssndqueue;
  double sbytes;
  String errmsg;

  /*************************************************************************
   * Added by Seth Peery
   * Adds public variables to store upstream and downstream speeds in kbps
   */
  public String downstream = "No data";
  public String upstream = "No data";
  /************************************************************************/


  int half_duplex, congestion, bad_cable, mismatch;
  double mylink;
  double loss, estimate, avgrtt, spd, waitsec, timesec, rttsec;
  double order, rwintime, sendtime, cwndtime, rwin, swin, cwin;
  double aspd;

  boolean isApplication = false;
  boolean testInProgress = false;
  String host = null;
  String tmpstr, tmpstr2;
  byte tests = TEST_MID | TEST_C2S | TEST_S2C | TEST_SFW | TEST_STATUS;
  int c2sResult = SFW_NOTTESTED;
  int s2cResult = SFW_NOTTESTED;

  class Message {
    byte type;
    byte[] body;
  }
  
  class Protocol {
    private InputStream _ctlin;
    private OutputStream _ctlout;

    public Protocol(Socket ctlSocket) throws IOException
    {
      _ctlin = ctlSocket.getInputStream();
      _ctlout = ctlSocket.getOutputStream();
    }
    
    public void send_msg(byte type, byte toSend) throws IOException
    {
      byte[] tab = new byte[] { toSend };
      send_msg(type, tab);
    }

    public void send_msg(byte type, byte[] tab) throws IOException
    {
      byte[] header = new byte[3];
      header[0] = type;
      header[1] = (byte) (tab.length >> 8);
      header[2] = (byte) tab.length;
      
      _ctlout.write(header);
      _ctlout.write(tab);
    }

    public int readn(Message msg, int amount) throws IOException
    {
      int read = 0; 
      int tmp;
      msg.body = new byte[amount];
      
      while (read != amount) {
        tmp = _ctlin.read(msg.body, read, amount - read);
        if (tmp <= 0) {
          return read;
        }
        read += tmp;

      }
      return read;
    }
    
    public int recv_msg(Message msg) throws IOException
    {
      int length;
      if (readn(msg, 3) != 3) {
        return 1;
      }
      msg.type = msg.body[0];
      length = ((int) msg.body[1] & 0xFF) << 8;
      length += (int) msg.body[2] & 0xFF; 
      if (readn(msg, length) != length) {
        return 3;
      }
      return 0;
    }

    public void close()
    {
      try {
        _ctlin.close();
        _ctlout.close();
      }
      catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  public boolean test_mid(Protocol ctl) throws IOException
  {
		byte buff[] = new byte[8192];
    Message msg = new Message();
    if ((tests & TEST_MID) == TEST_MID) {
      /* now look for middleboxes (firewalls, NATs, and other boxes that
       * muck with TCP's end-to-end priciples
       */
    	if (DEBUG) System.out.println("Tcpbw100 Middlebox test...");
    	if (DEBUG) System.out.print("Checking for Middleboxes . . . . . . . . . . . . . . . . . .  ");

      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        if (DEBUG) System.out.print(errmsg);
        return true;
      }
      if (msg.type != TEST_PREPARE) {
        errmsg = "Middlebox test: Received wrong type of the message\n";
        if (DEBUG) System.out.print(errmsg);
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
            if (DEBUG) System.out.print(errmsg);
        }
        return true;
      }
      int midport = Integer.parseInt(new String(msg.body));

      Socket in2Socket = null;
      try {
        in2Socket = new Socket(host, midport);
      } catch (UnknownHostException e) {
        System.err.print("Don't know about host: " + host);
        errmsg = "unknown server\n" ;
        if (DEBUG) System.out.print(errmsg);
        return true;
      } catch (IOException e) {
        System.err.print("Couldn't perform middlebox testing to: " + host);
        errmsg = "Server Failed while middlebox testing\n" ;
        if (DEBUG) System.out.print(errmsg);
        return true;
      }

      InputStream srvin2 = in2Socket.getInputStream();
      OutputStream srvout2 = in2Socket.getOutputStream();

      int largewin = 128*1024;

      in2Socket.setSoTimeout(6500);
      int bytes = 0;
      int inlth;
      t = System.currentTimeMillis();

      try {  
        while ((inlth=srvin2.read(buff,0,buff.length)) > 0) {
          bytes += inlth;
          if ((System.currentTimeMillis() - t) > 5500)
            break;
        }
      } 
      catch (IOException e) {}

      t =  System.currentTimeMillis() - t;
      if (DEBUG) System.out.print(bytes + " bytes " + (8.0 * bytes)/t + " kb/s " + t/1000 + " secs");
      s2cspd = ((8.0 * bytes) / 1000) / t;

      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        if (DEBUG) System.out.print(errmsg);
        return true;
      }
      if (msg.type != TEST_MSG) {
        errmsg = "Middlebox test results: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
            if (DEBUG) System.out.print(errmsg);
        }
        return true;
      }
      tmpstr2 = new String(msg.body);

      String tmpstr4 = Double.toString(s2cspd*1000);
      if (DEBUG) System.out.println("Sending '" + tmpstr4 + "' back to server");
      ctl.send_msg(TEST_MSG, tmpstr4.getBytes());

      try {
        tmpstr2 += in2Socket.getInetAddress() + ";";
      } catch (SecurityException e) {
        System.err.println("Unable to obtain Servers IP addresses: using " + host);
        errmsg = "getInetAddress() called failed\n" ;
        if (DEBUG) System.out.print(errmsg);
        tmpstr2 += host + ";";
        if (DEBUG) System.out.print("Unable to obtain remote IP address\n");
      }

      System.err.print("calling in2Socket.getLocalAddress()");
      try {
        tmpstr2 += in2Socket.getLocalAddress() + ";";
      } catch (SecurityException e) {
        System.err.println("Unable to obtain local IP address: using 127.0.0.1");
        errmsg = "getLocalAddress() call failed\n" ;
        if (DEBUG) System.out.print(errmsg);
        tmpstr2 += "127.0.0.1;";
      }

      srvin2.close();
      srvout2.close();
      in2Socket.close();

      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        if (DEBUG) System.out.print(errmsg);
        return true;
      }
      if (msg.type != TEST_FINALIZE) {
        errmsg = "Middlebox test: Received wrong type of the message\n";
        if (DEBUG) System.out.print(errmsg);
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
            if (DEBUG) System.out.print(errmsg);
        }
        return true;
      }
      if (DEBUG) System.out.print("Done\n");
      if (DEBUG) System.out.print("Done\n");

    }
    return false;
  }

  public boolean test_sfw(Protocol ctl) throws IOException
  {
    Message msg = new Message();
    if ((tests & TEST_SFW) == TEST_SFW) {
    	if (DEBUG) System.out.println("Simple firewall test...");
    	if (DEBUG) System.out.println("checking for firewalls . . . . . . . . . . . . . . . . . . .  ");
    	if (DEBUG) System.out.println("checking for firewalls . . . . . . . . . . . . . . . . . . .  ");
      
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_PREPARE) {
        errmsg = "Simple firewall test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }

      String message = new String(msg.body);

      int srvPort, testTime;
      try {
        int k = message.indexOf(" ");
        srvPort = Integer.parseInt(message.substring(0,k));
        testTime = Integer.parseInt(message.substring(k+1));
      }
      catch (Exception e) {
        errmsg = "Simple firewall test: Received improper message\n";
        return true;
      }

      if (DEBUG) System.out.println("SFW: port=" + srvPort);
      if (DEBUG) System.out.println("SFW: testTime=" + testTime);

      ServerSocket srvSocket;
      try {
          SecurityManager security = System.getSecurityManager();
          if (security != null) {
              if (DEBUG) System.out.println("Asking security manager for listen permissions...");
              security.checkListen(0);
          }
          srvSocket = new ServerSocket(0);
      }
      catch (Exception e) {
        e.printStackTrace();
        errmsg = "Simple firewall test: Cannot create listen socket\n";
        return true;
      }

      if (DEBUG) System.out.println("SFW: oport=" + srvSocket.getLocalPort());
      ctl.send_msg(TEST_MSG, Integer.toString(srvSocket.getLocalPort()).getBytes());

      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_START) {
        errmsg = "Simple firewall test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }     

      OsfwWorker osfwTest = new OsfwWorker(srvSocket, testTime);
      new Thread(osfwTest).start();

      Socket sfwSocket = new Socket();
      try {
        sfwSocket.connect(new InetSocketAddress(host, srvPort), testTime * 1000);

        Protocol sfwCtl = new Protocol(sfwSocket);
        sfwCtl.send_msg(TEST_MSG, new String("Simple firewall test").getBytes());
      }
      catch (Exception e) {
        e.printStackTrace();
      }

      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        if (DEBUG) System.out.print(errmsg);
        return true;
      }
      if (msg.type != TEST_MSG) {
        errmsg = "Simple firewall test: Received wrong type of the message\n";
        if (DEBUG) System.out.print(errmsg);
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
            if (DEBUG) System.out.print(errmsg);
        }
        return true;
      }
      c2sResult = Integer.parseInt(new String(msg.body));

      osfwTest.finalize();

      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        if (DEBUG) System.out.print(errmsg);
        return true;
      }
      if (msg.type != TEST_FINALIZE) {
        errmsg = "Simple firewall test: Received wrong type of the message\n";
        if (DEBUG) System.out.print(errmsg);
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
            if (DEBUG) System.out.print(errmsg);
        }
        return true;
      }
      if (DEBUG) System.out.print("Done\n");
      if (DEBUG) System.out.print("Done\n");
    }
    return false;
  }

  public boolean test_c2s(Protocol ctl) throws IOException
  {

	byte buff2[] = new byte[64*1024];
    Message msg = new Message();
    if ((tests & TEST_C2S) == TEST_C2S) {
      if (DEBUG) System.out.print("Tcpbw100 outbound test...");
      if (DEBUG) System.out.print("running 10s outbound test (client-to-server [C2S]) . . . . . ");
      
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_PREPARE) {
        errmsg = "C2S throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }
      int c2sport = Integer.parseInt(new String(msg.body));

      Socket outSocket = null;
      try {
        outSocket = new Socket(host, c2sport);
      } catch (UnknownHostException e) {
        System.err.println("Don't know about host: " + host);
        errmsg = "unknown server\n" ;
        return true;
      } catch (IOException e) {
        System.err.print("Couldn't get 2nd connection to: " + host);
        errmsg = "Server Busy: Please wait 15 seconds for previous test to finish\n" ;
        if (DEBUG) System.out.println(errmsg);
        return true;
      }

      OutputStream out = outSocket.getOutputStream();

      // wait here for signal from server application 
      // This signal tells the client to start pumping out data
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_START) {
        errmsg = "C2S throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }

      Random rng = new Random();
      byte c = '0';
      int i;
      for (i=0; i<lth; i++) {
        if (c == 'z')
          c = '0';
        buff2[i] = c++;
      }
      System.err.println("Send buffer size =" + i);
      outSocket.setSoTimeout(15000); 
      pkts = 0;
      t = System.currentTimeMillis();
      double stop_time = t + 10000; // ten seconds
      do {
        // if (Randomize) rng.nextBytes(buff2);
        try {
          out.write(buff2, 0, buff2.length);
        }
        catch (SocketException e) {
          if (DEBUG) System.out.println(e);
          break;
        }
        pkts++;
      } while (System.currentTimeMillis() < stop_time);

      t =  System.currentTimeMillis() - t;
      if (t == 0) {
        t = 1;
      }
      out.close();
      outSocket.close();
      c2sspd = ((8.0 * pkts * lth) / 1000) / t;
      /* receive the c2sspd from the server */
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_MSG) {
        errmsg = "C2S throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }
      String tmpstr3 = new String(msg.body);
      sc2sspd = Double.parseDouble(tmpstr3) / 1000.0;
      
      if (sc2sspd < 1.0) {
        if (DEBUG) System.out.println(prtdbl(sc2sspd*1000) + "kb/s\n");
      } 
      else {
        if (DEBUG) System.out.println(prtdbl(sc2sspd) + "Mb/s\n");
      }
      
      ret.c2sRate = sc2sspd * 1000/8; /* convert to kB/s */

/*************************************************************************
 * Added by Seth Peery
 * Write string representation of upstream speed to public String upstream
 */
                upstream = ""+(prtdbl(sc2sspd*1000));
/************************************************************************/
      
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_FINALIZE) {
        errmsg = "C2S throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }
    }
    return false;
  }

  public boolean test_s2c(Protocol ctl, Socket ctlSocket) throws IOException
  {
		byte buff[] = new byte[8192];
    Message msg = new Message();
    if ((tests & TEST_S2C) == TEST_S2C) {
      if (DEBUG) System.out.print("Tcpbw100 inbound test...");
      if (DEBUG) System.out.print("running 10s inbound test (server-to-client [S2C]) . . . . . . ");
      
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        if (DEBUG) System.out.print(errmsg);
        return true;
      }
      if (msg.type != TEST_PREPARE) {
        errmsg = "C2S throughput test: Received wrong type of the message\n";
        if (DEBUG) System.out.print(errmsg);
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
            if (DEBUG) System.out.print(errmsg);
        }
        return true;
      }
      int s2cport = Integer.parseInt(new String(msg.body));

      Socket inSocket;
      try {
        inSocket = new Socket(host, s2cport);
      } 
      catch (UnknownHostException e) {
        System.err.println("Don't know about host: " + host);
        errmsg = "unknown server\n" ;
        return true;
      } 
      catch (IOException e) {
        System.err.println("Couldn't get 3rd connection to: " + host);
        errmsg = "Server Failed while receiving data\n" ;
        return true;
      }

      InputStream srvin = inSocket.getInputStream();
      int bytes = 0;
      int inlth;

      // wait here for signal from server application 
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_START) {
        errmsg = "S2C throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }

      inSocket.setSoTimeout(15000);
      t = System.currentTimeMillis();

      try {  
        while ((inlth=srvin.read(buff,0,buff.length)) > 0) {
          bytes += inlth;
          if ((System.currentTimeMillis() - t) > 14500)
            break;
        }
      } 
      catch (IOException e) {}

      t =  System.currentTimeMillis() - t;
      //if (DEBUG) System.out.println(bytes + " bytes " + (8.0 * bytes)/t + " kb/s " + t/1000 + " secs");
      s2cspd = ((8.0 * bytes) / 1000) / t;

      /* receive the s2cspd from the server */
      if (ctl.recv_msg(msg) != 0) {
        errmsg = "Protocol error!\n";
        return true;
      }
      if (msg.type != TEST_MSG) {
        errmsg = "S2C throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
        return true;
      }
      try {
        String tmpstr3 = new String(msg.body);
        int k1 = tmpstr3.indexOf(" ");
        int k2 = tmpstr3.substring(k1+1).indexOf(" ");
        ss2cspd = Double.parseDouble(tmpstr3.substring(0, k1)) / 1000.0;
        ssndqueue = Integer.parseInt(tmpstr3.substring(k1+1).substring(0, k2));
        sbytes = Double.parseDouble(tmpstr3.substring(k1+1).substring(k2+1));
      }
      catch (Exception e) {
        e.printStackTrace();
        errmsg = "S2C throughput test: Received improper message\n";
        return true;
      }

      if (s2cspd < 1.0) {
        if (DEBUG) System.out.println(prtdbl(s2cspd*1000) + "kb/s\n");
      } else {
        if (DEBUG) System.out.println(prtdbl(s2cspd) + "Mb/s\n");
      }
      
      ret.s2cRate = s2cspd * 1000/8; /* convert to kB/s */; 


/*************************************************************************
 * Added by Seth Peery
 * Write string representation of downstream speed to public String upstream
 */
                downstream = ""+(prtdbl(s2cspd*1000));
/************************************************************************/

      srvin.close();
      inSocket.close();

      buff = Double.toString(s2cspd*1000).getBytes();
      String tmpstr4 = new String(buff, 0, buff.length);
      if (DEBUG) System.out.println("Sending '" + tmpstr4 + "' back to server");
      ctl.send_msg(TEST_MSG, buff);

      /* get web100 variables from server */
      tmpstr = "";
      int i = 0;

      // Try setting a 5 second timer here to break out if the read fails.
      ctlSocket.setSoTimeout(5000);
      try {  
        for (;;) {
          if (ctl.recv_msg(msg) != 0) {
            errmsg = "Protocol error!\n";
            return true;
          }
          if (msg.type == TEST_FINALIZE) {
            break;
          }
          if (msg.type != TEST_MSG) {
            errmsg = "S2C throughput test: Received wrong type of the message\n";
        if (msg.type == MSG_ERROR) {
            errmsg += "ERROR MSG: " + new String(msg.body) + "\n";
        }
            return true;
          }
          tmpstr += new String(msg.body);
          i++;
        }
      } catch (IOException e) {}
      ctlSocket.setSoTimeout(0);
    }
    return false;
  }

  public FactNdtProbeResult dottcp( String hostname) throws IOException {
	  
	  ret = new FactNdtProbeResult();
      Socket ctlSocket = null;
      ret.server = hostname;
      host = hostname;

      if (DEBUG) System.out.println("Connecting to '" + host + "' [" + InetAddress.getByName(host) + "] to run test\n");

      int ctlport = 3001;
      int i, wait;

      failed = false;
      try {

//    	  if (preferIPv6.isSelected()) {
//              try {
//                  System.setProperty("java.net.preferIPv6Addresses", "true");
//              }
//              catch (SecurityException e) {
//                  System.err.println("Couldn't set system property. Check your security settings.");
//              }
//          }
//          preferIPv6.setEnabled(false);
    	  
          ctlSocket = new Socket(host, ctlport);   
          ret.server = ctlSocket.getInetAddress().getHostAddress();
      } catch (UnknownHostException e) {
          ret.errorMsg = "unknown server\n" ;
          ret.failed = true;
          return ret;
      } catch (IOException e) {
          ret.errorMsg = "Server process not running: start web100srv process on remote server (" + host + ":" + ctlport + ")\n" ;
          ret.failed = true;
          return ret;
      }
      Protocol ctl = new Protocol(ctlSocket);
      Message msg = new Message();
      
      /* The beginning of the protocol */

      if (ctlSocket.getInetAddress() instanceof Inet6Address) {
    	  if (DEBUG) System.out.println("Connected to: " + host + "  --  Using IPv6 address\n");
      }
      else {
    	  if (DEBUG) System.out.println("Connected to: " + host + "  --  Using IPv4 address\n");
      }

      /* write our test suite request */
      ctl.send_msg(MSG_LOGIN, tests);

      /* read the specially crafted data that kicks off the old clients */
      if (ctl.readn(msg, 13) != 13) {
    	  ret.errorMsg = "Information: The server does not support this command line client\n";
          ret.failed = true;
          return ret;
      }
      
      for (;;) {
          if (ctl.recv_msg(msg) != 0) {
        	  ret.errorMsg = "Protocol error!\n";
              ret.failed = true;
              return ret;
          }
          if (msg.type != SRV_QUEUE) {
        	  ret.errorMsg = "Logging to server: Received wrong type of the message\n";
              ret.failed = true;
              return ret;
          }
          String tmpstr3 = new String(msg.body);
          wait = Integer.parseInt(tmpstr3);
          if (DEBUG) System.out.println("wait flag received = " + wait);

          if (wait == 0) {
              break;
          }

          if (wait == 9988) {
        	  ret.errorMsg = "Server Busy: Too many clients waiting in server queue.  Please try again later.\n";
              ret.failed = true;
              return ret;
          }

          if (wait == 9999) {
        	  ret.errorMsg = "Server Busy: Please wait 60 seconds for the current test to finish.\n";
              ret.failed = true;
              return ret;
          }

	  if (wait == 9990) {  // signal from the server to see if the client is still alive
      	      ctl.send_msg(MSG_WAITING, tests);
	      continue;
	  }

          // Each test should take less than 30 seconds, so tell them 45 sec * number of 
          // tests in the queue.
          wait = (wait * 45);
          if (DEBUG) System.out.println("Another client is currently being served, your test will " +
                  "begin within " + wait + " seconds\n");
      }

      if (ctl.recv_msg(msg) != 0) {
    	  ret.errorMsg = "Protocol error!\n";
          ret.failed = true;
          return ret;
      }
      if (msg.type != MSG_LOGIN) {
    	  ret.errorMsg = "Negotiating NDT version: Received wrong type of the message\n";
          ret.failed = true;
          return ret;
      }

      String vVersion = new String(msg.body);
      if (!vVersion.startsWith("v")) {
    	  ret.errorMsg = "Incompatible version number";
          ret.failed = true;
          return ret;
      }
      if (DEBUG) System.out.println("Server version: " + vVersion.substring(1));

      if (ctl.recv_msg(msg) != 0) {
    	  ret.errorMsg = "Protocol error!\n";
          ret.failed = true;
          return ret;
      }
      if (msg.type != MSG_LOGIN) {
    	  ret.errorMsg = "Negotiating test suite: Received wrong type of the message\n";
          ret.failed = true;
          return ret;
      }
      StringTokenizer tokenizer = new StringTokenizer(new String(msg.body), " ");

      while (tokenizer.hasMoreTokens()) {
          int testId = Integer.parseInt(tokenizer.nextToken());
          switch (testId) {
              case TEST_MID:
                  if (test_mid(ctl)) {
                	  if (DEBUG) System.out.println("Middlebox test FAILED!\n");
                      tests &= (~TEST_MID);
                  }
                  break;
              case TEST_SFW:
                  if (test_sfw(ctl)) {
                	  if (DEBUG) System.out.println("Simple firewall test FAILED!\n");
                      tests &= (~TEST_SFW);
                  }
                  break;
              case TEST_C2S:
            	  if (DEBUG) System.out.println("C2S throughput");
                  if (test_c2s(ctl)) {
                	  if (DEBUG) System.out.println("C2S throughput test FAILED!\n");
                      tests &= (~TEST_C2S);
                      ret.errorMsg = "C2S throughput test FAILED!\n";
                      ret.failed = true;
                  }
                  break;
              case TEST_S2C:
            	  if (DEBUG) System.out.println("S2C throughput");
                  if (test_s2c(ctl, ctlSocket)) {
                	  if (DEBUG) System.out.println("S2C throughput test FAILED!\n");
                      tests &= (~TEST_S2C);
                      ret.errorMsg = "S2C throughput test FAILED!\n";
                      ret.failed = true;
                  }
                  break;
              default:
            	  ret.errorMsg = "Unknown test ID\n";
                  ret.failed = true;
                  return ret;
          }
      }

      if (DEBUG) System.out.println("Receiving results...");
      i = 0;

      try {  
          for (;;) {
              if (ctl.recv_msg(msg) != 0) {
            	  ret.errorMsg = "Protocol error!\n";
                  ret.failed = true;
                  return ret;
              }
              if (msg.type == MSG_LOGOUT) {
                  break;
              }
              if (msg.type != MSG_RESULTS) {
            	  ret.errorMsg = "Tests results: Received wrong type of the message\n";
                  ret.failed = true;
                  return ret;
              }
              tmpstr += new String(msg.body);
              i++;
          }
      } catch (IOException e) {}

      if (i == 0) {
    	  if (DEBUG) System.out.println("Warning! Client time-out while reading data, possible duplex mismatch exists\n");
      }
      System.err.println("Calling InetAddress.getLocalHost() twice");
      try {
    	  if (DEBUG) System.out.println("Client: " + InetAddress.getLocalHost() + "\n");
      } catch (SecurityException e) {
          System.err.println("Unable to obtain local IP address: using 127.0.0.1");
      }

      ctl.close();
      ctlSocket.close();

      try {
          testResults(tmpstr);
      }
      catch (Exception ex) {
    	  if (DEBUG) System.out.println("Error in parsing tests results!\n");
    	  ex.printStackTrace();
    	  ret.errorMsg = "Error in parsing tests results!\n";
    	  ret.failed = true;
    	  return ret;
      }
      if ((tests & TEST_MID) == TEST_MID) {
          middleboxResults(tmpstr2);
      }

      return ret;
      
  }


	class OsfwWorker implements Runnable {
    private ServerSocket srvSocket;
    private int testTime;
    private boolean finalized = false;
    
    OsfwWorker(ServerSocket srvSocket, int testTime)
    {
      this.srvSocket = srvSocket;
      this.testTime = testTime;
    }

    public void finalize()
    {
      while (!finalized) {
        try {
          Thread.currentThread().sleep(1000);
        }
        catch (InterruptedException e) {
          // do nothing.
        }
      }
    }
    
    public void run()
    {
      Message msg = new Message();
      Socket sock = null;

      try {
        srvSocket.setSoTimeout(testTime * 2000);
        try {
          sock = srvSocket.accept();
        }
        catch (Exception e) {
          //e.printStackTrace();
          s2cResult = SFW_POSSIBLE;
          srvSocket.close();
          finalized = true;
          return;
        }
        Protocol sfwCtl = new Protocol(sock);

        if (sfwCtl.recv_msg(msg) != 0) {
          if (DEBUG) System.out.println("Simple firewall test: unrecognized message");
          s2cResult = SFW_UNKNOWN;
          sock.close();
          srvSocket.close();
          finalized = true;
          return;
        }
        if (msg.type != TEST_MSG) {
          s2cResult = SFW_UNKNOWN;
          sock.close();
          srvSocket.close();
          finalized = true;
          return;
        }
        if (! new String(msg.body).equals("Simple firewall test")) {
          if (DEBUG) System.out.println("Simple firewall test: Improper message");
          s2cResult = SFW_UNKNOWN;
          sock.close();
          srvSocket.close();
          finalized = true;
          return;
        }
        s2cResult = SFW_NOFIREWALL;
      }
      catch (IOException ex) {
        s2cResult = SFW_UNKNOWN;
      }
      try {
        sock.close();
        srvSocket.close();
      }
      catch (IOException e) {
        // do nothing
      }
      finalized = true;
    }
  }

  public void showBufferedBytesInfo()
  {
    JOptionPane.showMessageDialog(null,
        "TCP (Transmission Control Protocol) reliably transfers data between two\n" +
        "Internet hosts.  It automatically detects and recovers from errors and\n" +
        "losses.  TCP uses buffers to provide this reliability.  In addition,\n" +
        "switches and routers use buffers to handle cases where multiple input\n" +
        "links send packets to a single output link or link speeds change\n" +
        "(FastEthernet to DSL modem).\n\n" +
        "The NDT server generates and sends 10 seconds of data to the client.  In\n" +
        "some cases the server can generate data faster than it can send packets\n" +
        "into the network (e.g., a 2 GHz CPU sending to a DSL connected client).\n" +
        "When this happens, some packets may remain in the server output queue\n" +
        "when the 10 second timer expires.  TCP will automatically continue to\n" +
        "send these queued packets and the client will continue to accept and\n" +
        "process these incoming packets.  This will result in the client test\n" +
        "running longer than expected.\n\n" +
        "This condition has occurred during this test.  No action is required to\n" +
        "resolve this issue.",
        "Packet queuing",
        JOptionPane.INFORMATION_MESSAGE);
  }

	public void testResults(String tmpstr) {
		
		StringTokenizer tokens;
		int i=0;
		String sysvar, strval;
		int sysval=0, Zero=0;
		double sysval2, j;
		String osName, osArch, osVer, javaVer, javaVendor, client;

		tokens = new StringTokenizer(tmpstr);
		sysvar = null;
		strval = null;
		while(tokens.hasMoreTokens()) {
			sysvar = tokens.nextToken(); 
			strval = tokens.nextToken();
//			if(++i%2 == 1) {
//			}
//			else {
			if (DEBUG) {
				System.out.print("sysvar: |" + sysvar + "| ");
				System.out.print("sysval: |" + sysval + "| ");
				System.out.println(" strval: |" + strval + "|");
				System.out.println(sysvar + " " + strval + "\n");
			}
			
				if (strval.indexOf(".") == -1) {
					sysval = Integer.parseInt(strval);
					save_int_values(sysvar, sysval);
				}
				else {
					sysval2 = Double.valueOf(strval).doubleValue();
					save_dbl_values(sysvar, sysval2);
				}
				
				ret.variablesMap.put(sysvar, Double.valueOf(strval).doubleValue());
//			}
		}

		// Grap some client details from the applet environment
		osName = System.getProperty("os.name");
		osArch = System.getProperty("os.arch");
		osVer = System.getProperty("os.version");
		javaVer = System.getProperty("java.version");
		javaVendor = System.getProperty("java.vendor");

		ret.osName = osName;
		ret.osArch = osArch;
		ret.osVer = osVer;
		ret.javaVer = javaVer;
		
		if (osArch.startsWith("x86") == true) {
			client = "PC";
		}
		else {
			client = "Workstation";
		}

		// Calculate some variables and determine path conditions
		// Note: calculations now done in server and the results are shipped
		//    back to the client for printing.

		if (CountRTT>0) {
			// Now write some messages to the screen
			if (c2sData < 3) {
				if (c2sData < 0) {
					if (DEBUG) System.out.print("Server unable to determine bottleneck link type.\n");
				} 
				else {
					if (DEBUG) System.out.print("Your " + client + " is connected to a ");
					if (c2sData == 1) {
						if (DEBUG) System.out.println("Dial-up Modem\n");
						mylink = .064;
					} 
					else {
						if (DEBUG) System.out.print("Cable/DSL modem\n");
						mylink = 3;
					}
				}
			} 
			else {
				if (DEBUG) System.out.print("The slowest link in the end-to-end path is a ");
				if (c2sData == 3) {
					if (DEBUG) System.out.print("10 Mbps Ethernet subnet\n");
					mylink = 10;
				} 
				else if (c2sData == 4) {
					if (DEBUG) System.out.print("45 Mbps T3/DS3 subnet\n");
					mylink = 45;
				} 
				else if (c2sData == 5) {
					if (DEBUG) System.out.print("100 Mbps ");
					mylink = 100;
					if (half_duplex == 0) {
						if (DEBUG) System.out.print("Full duplex Fast Ethernet subnet\n");
					} 
					else {
						if (DEBUG) System.out.print("Half duplex Fast Ethernet subnet\n");
					}
				} 
				else if (c2sData == 6) {
					if (DEBUG) System.out.print("a 622 Mbps OC-12 subnet\n");
					mylink = 622;
				} 
				else if (c2sData == 7) {
					if (DEBUG) System.out.print("1.0 Gbps Gigabit Ethernet subnet\n");
					mylink = 1000;
				} 
				else if (c2sData == 8) {
					if (DEBUG) System.out.print("2.4 Gbps OC-48 subnet\n");
					mylink = 2400;
				} 
				else if (c2sData == 9) {
					if (DEBUG) System.out.print("10 Gbps 10 Gigabit Ethernet/OC-192 subnet\n");
					mylink = 10000;
				}
			}
			
			ret.mylink = mylink;

			if (mismatch == 1) {
				if (DEBUG) System.out.print("Warning: Old Duplex mismatch condition detected: ");
			}
 			else if (mismatch == 2) {
				if (DEBUG) System.out.print("Alarm: Duplex Mismatch condition detected Switch=Full and Host=half\n ");
			}
 			else if (mismatch == 4) {
				if (DEBUG) System.out.print("Alarm: Possible Duplex Mismatch condition detected Switch=Full and Host=half\n ");
			}
 			else if (mismatch == 3) {
				if (DEBUG) System.out.print("Alarm: Duplex Mismatch condition detected Switch=half and Host=full\n ");
			}
 			else if (mismatch == 5) {
				if (DEBUG) System.out.print("Alarm: Possible Duplex Mismatch condition detected Switch=half and Host=full\n ");
			}
 			else if (mismatch == 7) {
				if (DEBUG) System.out.print("Warning: Possible Duplex Mismatch condition detected Switch=half and Host=full\n ");
			}
			
			ret.mismatch = mismatch;
			
			if (mismatch == 0) {
			    if (bad_cable == 1) {
				if (DEBUG) System.out.println("Alarm: Excessive errors, check network cable(s).\n");
			    }
			    if (congestion == 1) {
				if (DEBUG) System.out.print("Information: Other network traffic is congesting the link\n");
			    }
			    if (((2*rwin)/rttsec) < mylink) {
			        j = (float)((mylink * avgrtt) * 1000) / 8 / 1024;
			        if (j > (float)MaxRwinRcvd) {
				    if (DEBUG) System.out.print("Information: The receive buffer should be " +
				    		prtdbl(j) + " kbytes to maximize throughput\n");
			        }
			    }
			}
     
	      if ((tests & TEST_C2S) == TEST_C2S) {
	        if (sc2sspd < (c2sspd  * (1.0 - VIEW_DIFF))) {
	          // TODO:  distinguish the host buffering from the middleboxes buffering
	          if (DEBUG) System.out.print("[C2S]: Packet queuing detected\n");
	          ret.c2sPacketQueuing = true;
	        }
	      }
	      
	      if ((tests & TEST_S2C) == TEST_S2C) {
	        if (s2cspd < (ss2cspd  * (1.0 - VIEW_DIFF))) {
	          // TODO:  distinguish the host buffering from the middleboxes buffering
	          if (DEBUG) System.out.print("[S2C]: Packet queuing detected\n");
	          ret.s2cPacketQueuing = true;
	        }
	      }

			if (DEBUG) System.out.print("\n\t------  Client System Details  ------\n");
			if (DEBUG) System.out.print("OS data: Name = " + osName + ", Architecture = " + osArch);
			if (DEBUG) System.out.print(", Version = " + osVer + "\n");
			if (DEBUG) System.out.print("Java data: Vendor = " + javaVendor + ", Version = " + javaVer + "\n");
			// if (DEBUG) System.out.println(" java.class.version=" + System.getProperty("java.class.version") + "\n");

			if (DEBUG) System.out.print("\n\t------  Web100 Detailed Analysis  ------\n");
			if (c2sData == -2)
				if (DEBUG) System.out.print("Insufficent data collected to determine link type.\n");
			else if (c2sData == -1)
				if (DEBUG) System.out.print("Interprocess communications failed, unknown link type.\n");
			else if (c2sData == 0)
				if (DEBUG) System.out.print("Link detection algorithm failed due to excessive Round Trip Times.\n");
			else if (c2sData == 1)
				if (DEBUG) System.out.print("Dial-up modem link found.\n");
			else if (c2sData == 2)
				if (DEBUG) System.out.print("Cable modem/DSL/T1 link found.\n");
			else if (c2sData == 3)
				if (DEBUG) System.out.print("10 Mbps Ethernet link found.\n");
			else if (c2sData == 4)
				if (DEBUG) System.out.print("45 Mbps T3/DS3 link found.\n");
			else if (c2sData == 5)
				if (DEBUG) System.out.print("100 Mbps FastEthernet link found.\n");
			else if (c2sData == 6)
				if (DEBUG) System.out.print("622 Mbps OC-12 link found.\n");
			else if (c2sData == 7)
				if (DEBUG) System.out.print("1 Gbps GigabitEthernet link found.\n");
			else if (c2sData == 8)
				if (DEBUG) System.out.print("2.4 Gbps OC-48 link found.\n");
			else if (c2sData == 9)
				if (DEBUG) System.out.print("10 Gbps 10 GigEthernet/OC-192 link found.\n");

			if (half_duplex == 0)
				if (DEBUG) System.out.print("Link set to Full Duplex mode\n");
			else
				if (DEBUG) System.out.print("Link set to Half Duplex mode\n");

			if (congestion == 0)
				if (DEBUG) System.out.print("No network congestion discovered.\n");
			else
				if (DEBUG) System.out.print("Information: throughput is limited by other network traffic.\n");

			if (bad_cable == 0)
				if (DEBUG) System.out.print("Good network cable(s) found\n");
			else
				if (DEBUG) System.out.print("Warning: excessive network errors, check network cable(s)\n");

			if (mismatch == 0)
				if (DEBUG) System.out.println("Normal duplex operation found.\n");
			else if (mismatch == 1) {
				if (DEBUG) System.out.print("Warning: Old Duplex mismatch condition detected: ");
			}
 			else if (mismatch == 2) {
				if (DEBUG) System.out.print("Alarm: Duplex Mismatch condition detected Switch=Full and Host=half\n ");
			}
 			else if (mismatch == 3) {
				if (DEBUG) System.out.print("Alarm: Duplex Mismatch condition detected Switch=half and Host=full\n ");
			}

			if (DEBUG) System.out.print("\nWeb100 reports the Round trip time = " + prtdbl(avgrtt) + " msec; ");
			ret.avgrtt = avgrtt;

			if (DEBUG) System.out.print("the Packet size = " + CurrentMSS + " Bytes; and \n");
			
			ret.congestion = (congestion==1) ? true : false;
			ret.packetSize = CurrentMSS;
			ret.connectionStalledTimes = Timeouts;
			ret.connectionIdlePercentage = (waitsec/timesec)*100;
			ret.fastRetransmit = FastRetran;
			ret.congestionSignals = CongestionSignals;
			ret.timeouts = Timeouts;
			ret.pktsRetrans = PktsRetrans;
			ret.dupAcksIn = DupAcksIn;
			ret.sACKsRcvd = SACKsRcvd;
			ret.outOfOrderPercentage = order*100;
			ret.packetLoss = loss;
						
			if (PktsRetrans > 0) {
				if (DEBUG) System.out.print("There were " + PktsRetrans + " packets retransmitted");
				if (DEBUG) System.out.print(", " + DupAcksIn + " duplicate acks received");
				if (DEBUG) System.out.print(", and " + SACKsRcvd + " SACK blocks received\n");
				if (Timeouts > 0) {
					if (DEBUG) System.out.print("The connection stalled " + Timeouts + " times due to packet loss\n");
				}
	
				if (DEBUG) System.out.print("The connection was idle " + prtdbl(waitsec) + " seconds (" + 
					prtdbl((waitsec/timesec)*100) + "%) of the time\n");
			} 
			else if (DupAcksIn > 0) {
				if (DEBUG) System.out.print("No packet loss - ");
				if (DEBUG) System.out.print("but packets arrived out-of-order " + prtdbl(order*100) +
				"% of the time\n");
			} 
			else {
				if (DEBUG) System.out.print("No packet loss was observed.\n");
			}

	      if ((tests & TEST_C2S) == TEST_C2S) {
	        if (c2sspd > sc2sspd) {
	          if (sc2sspd < (c2sspd  * (1.0 - VIEW_DIFF))) {
	            if (DEBUG) System.out.print("C2S throughput test: Packet queuing detected: " + prtdbl(100 * (c2sspd - sc2sspd) / c2sspd) + "%\n");
	          }
	          else {
	            if (DEBUG) System.out.print("C2S throughput test: Packet queuing detected: " + prtdbl(100 * (c2sspd - sc2sspd) / c2sspd) + "%\n");
	          }
	        }
	        
	        ret.s2cThroughputPacketQueueingDetectedPercentage = (100 * (c2sspd - sc2sspd) / c2sspd);
	      }
	      
	      if ((tests & TEST_S2C) == TEST_S2C) {
	        if (ss2cspd > s2cspd) {
	          if (s2cspd < (ss2cspd  * (1.0 - VIEW_DIFF))) {
	            if (DEBUG) System.out.print("S2C throughput test: Packet queuing detected: " + prtdbl(100 * (ss2cspd - s2cspd) / ss2cspd) + "%\n");
	          }
	          else {
	            if (DEBUG) System.out.print("S2C throughput test: Packet queuing detected: " + prtdbl(100 * (ss2cspd - s2cspd) / ss2cspd) + "%\n");
	          }
	        }
	        
	        ret.s2cThroughputPacketQueueingDetectedPercentage = (100 * (ss2cspd - s2cspd) / ss2cspd);
	      }
      
			if (rwintime > .015) {
				if (DEBUG) System.out.print("This connection is receiver limited " + prtdbl(rwintime*100) +
				"% of the time.\n");

			// I think there is a bug here, it sometimes tells you to increase the buffer
			// size, but the new size is smaller than the current.

				if (((2*rwin)/rttsec) < mylink) {
					if (DEBUG) System.out.println("  Increasing the the client's receive buffer (" + prtdbl(MaxRwinRcvd/1024) +
					" KB) will improve performance\n");
				}
				
				ret.connectionReceiverLimitedPercentage = rwintime*100;
			}
			if (sendtime > .015) {
				if (DEBUG) System.out.print("This connection is sender limited " + prtdbl(sendtime*100) +
				"% of the time.\n");
				if ((2*(swin/rttsec)) < mylink) {
					if (DEBUG) System.out.print("  Increasing the NDT server's send buffer (" + prtdbl(Sndbuf/2048) +
					" KB) will improve performance\n");
				}
				
				ret.connectionSenderLimitedPercentage = sendtime*1000;
				
			}
			if (cwndtime > .005) {
				if (DEBUG) System.out.print("This connection is network limited " + prtdbl(cwndtime*100) + "% of the time.\n");
				ret.connectionNetworkLimitedPercentage = cwndtime*100;
			}
			
			if ((spd < 4) && (loss > .01)) {
				if (DEBUG) System.out.print("Excessive packet loss is impacting your performance, check the ");
				if (DEBUG) System.out.print("auto-negotiate function on your local PC and network switch\n");
			}
			
			if (DEBUG) System.out.print("\nWeb100 reports TCP negotiated the optional Performance Settings to: \n");
			if (DEBUG) System.out.print("RFC 2018 Selective Acknowledgment: ");
			if(SACKEnabled == Zero) {
				if (DEBUG) System.out.print ("OFF\n");
				ret.RFC2018 = false;
			}
			else {
				if (DEBUG) System.out.print ("ON\n");
				ret.RFC2018 = true;
			}

			if (DEBUG) System.out.print("RFC 896 Nagle Algorithm: ");
			if(NagleEnabled == Zero) {
				if (DEBUG) System.out.print ("OFF\n");
				ret.RFC896 = false;
			}
			else {
				if (DEBUG) System.out.print ("ON\n");
				ret.RFC896 = true;
			}

			if (DEBUG) System.out.print("RFC 3168 Explicit Congestion Notification: ");
			if(ECNEnabled == Zero) {
				if (DEBUG) System.out.print ("OFF\n");
				ret.RFC3168 = false;
			}
			else {
				if (DEBUG) System.out.print ("ON\n");
				ret.RFC3168 = true;
			}

			if (DEBUG) System.out.print("RFC 1323 Time Stamping: ");
			if(TimestampsEnabled == 0) {
				if (DEBUG) System.out.print ("OFF\n");
				ret.RFC1323TimeStamping = false;
			}
			else {
				if (DEBUG) System.out.print ("ON\n");
				ret.RFC1323TimeStamping = true;
			}

			if (DEBUG) System.out.print("RFC 1323 Window Scaling: ");
			if (MaxRwinRcvd < 65535)
			    WinScaleRcvd = 0;
			if((WinScaleRcvd == 0) || (WinScaleRcvd > 20)) {
				if (DEBUG) System.out.print ("OFF\n");
				ret.RFC1323WindowScaling = false;
			}
			else {
				if (DEBUG) System.out.print ("ON; Scaling Factors -  Server=" + WinScaleRcvd + ", Client=" + WinScaleSent + "\n");
				ret.RFC1323WindowScaling = true;
				ret.serverScalingFactor = WinScaleRcvd;
				ret.clientScalingFactor = WinScaleSent;
			}

	      if (DEBUG) System.out.print("\n");
	
	      if ((tests & TEST_SFW) == TEST_SFW) {
	        switch (c2sResult) {
	          case SFW_NOFIREWALL:
	            if (DEBUG) System.out.print("Server '" + host + "' is not behind a firewall. [Connection to the ephemeral port was successful]\n");
	            ret.serverBehindFirewall = false;
	            break;
	          case SFW_POSSIBLE:
	            if (DEBUG) System.out.print("Server '" + host + "' is probably behind a firewall. [Connection to the ephemeral port failed]\n");
	            ret.serverBehindFirewall = true;
	            break;
	          case SFW_UNKNOWN:
	          case SFW_NOTTESTED:
	            break;
	        }
	        switch (s2cResult) {
	          case SFW_NOFIREWALL:
	            if (DEBUG) System.out.print("Client is not behind a firewall. [Connection to the ephemeral port was successful]\n");
	            ret.clientBehindFirewall = false;
	            break;
	          case SFW_POSSIBLE:
	            if (DEBUG) System.out.print("Client is probably behind a firewall. [Connection to the ephemeral port failed]\n");
	            ret.clientBehindFirewall = true;
	            break;
	          case SFW_UNKNOWN:
	          case SFW_NOTTESTED:
	            break;
	        }
	      }

			//if (DEBUG) System.out.println("\nEstimate = " + prtdbl(estimate) + " based on packet size = "
			//	+ (CurrentMSS*8/1024) + "kbits, RTT = " + prtdbl(avgrtt) + "msec, " + "and loss = " + loss + "\n");
			
	      if (DEBUG) System.out.println("\n");

      		ret.theoreticalNetworkLimit = estimate * 1000/8; /* convert to kB/s */
      		ret.serverBuffer = Sndbuf/2048;
      		ret.serverThroughputLimit = swin/rttsec * 1000/8; /* convert to kB/s */
      		ret.clientBuffer = MaxRwinRcvd/1024;
      		ret.clientThroughputLimit = rwin/rttsec * 1000/8; /* convert to kB/s */
      		ret.networkFlowControlThroughputLimit = cwin/rttsec  * 1000/8; /* convert to kB/s */
      		
      			
			if (DEBUG) System.out.print("The theoretical network limit is " + prtdbl(estimate) + " Mbps\n");

			if (DEBUG) System.out.print("The NDT server has a " + prtdbl(Sndbuf/2048) + 
				" KByte buffer which limits the throughput to " + prtdbl(swin/rttsec) + " Mbps\n");

			if (DEBUG) System.out.print("Your PC/Workstation has a " + prtdbl(MaxRwinRcvd/1024) +
				" KByte buffer which limits the throughput to " + prtdbl(rwin/rttsec) + " Mbps\n");

			if (DEBUG) System.out.print("The network based flow control limits the throughput to " +	prtdbl(cwin/rttsec) + " Mbps\n");

			if (DEBUG) System.out.print("\nClient Data reports link is '" + prttxt(c2sData) +
				"', Client Acks report link is '" + prttxt(c2sAck) + "'\n" + 
				"Server Data reports link is '" + prttxt(s2cData) +
				"', Server Acks report link is '" + prttxt(s2cAck) + "'\n");
		}
	}  // testResults()
	
	

	/* this routine decodes the middlebox test results.  The data is returned
	* from the server is a specific order.  This routine pulls the string apart
	* and puts the values into the proper variable.  It then compares the values
	* to known values and writes out the specific results.
	*
	* server data is first
	* order is Server IP; Client IP; CurrentMSS; WinScaleRcvd; WinScaleSent;
	* Client then adds
	* Server IP; Client IP.
	*/

	public void middleboxResults(String tmpstr2) {
		StringTokenizer tokens;
		int k;

		tokens = new StringTokenizer(tmpstr2, ";");
		String ssip = tokens.nextToken();
		String scip = tokens.nextToken();

		int mss = Integer.parseInt(tokens.nextToken());
		int winsrecv = Integer.parseInt(tokens.nextToken());
		int winssent = Integer.parseInt(tokens.nextToken());

		String csip = tokens.nextToken();
		k = csip.indexOf("/");
		csip = csip.substring(k+1);

		String ccip = tokens.nextToken();
		k = ccip.indexOf("/");
		ccip = ccip.substring(k+1);

		if (mss == 1456)
			if (DEBUG) System.out.println("Packet size is preserved End-to-End\n");
		else
			if (DEBUG) System.out.println("Information: Network Middlebox is modifying MSS variable\n");


    boolean preserved = false;
    try {
      preserved = InetAddress.getByName(ssip).equals(InetAddress.getByName(csip));
    }
    catch (UnknownHostException e) {
      preserved = ssip.equals(csip);
    }
		if (preserved) {
			if (DEBUG) System.out.println("Server IP addresses are preserved End-to-End\n");
		}
		else {
			if (DEBUG) System.out.println("Information: Network Address Translation (NAT) box is " +
			"modifying the Server's IP address\n");
			if (DEBUG) System.out.println("\tServer says [" + ssip + "] but Client says [" + csip + "]\n");
		}

		if (ccip.equals("127.0.0.1")) {
			if (DEBUG) System.out.println("Client IP address not found.  For IE users, modify the Java parameters\n");
			if (DEBUG) System.out.println("\tclick Tools - Internet Options - Security - Custom Level, scroll down to\n");
			if (DEBUG) System.out.println("\tMicrosoft VM - Java permissions and click Custom, click Java Custom Settings\n");
			if (DEBUG) System.out.println("\tEdit Permissions - Access to all Network Addresses, click Eanble and save changes\n");
		}
		else {
      try {
        preserved = InetAddress.getByName(scip).equals(InetAddress.getByName(ccip));
      }
      catch (UnknownHostException e) {
        preserved = scip.equals(ccip);
      }
			if (preserved)
				if (DEBUG) System.out.println("Client IP addresses are preserved End-to-End\n");
			else {
				if (DEBUG) System.out.println("Information: Network Address Translation (NAT) box is modifying the Client's IP address\n");
				if (DEBUG) System.out.println("\tServer says [" + scip + "] but Client says [" + ccip + "]\n");
			}
		}
	} // middleboxResults()



	public String prtdbl(double d) {
		String str = null;
		int i;

		if (d == 0) {
			return ("0");
		}
		str = Double.toString(d);
		i = str.indexOf(".");
		i = i + 3;
		if (i > str.length()) {
			i = i - 1;
		}
		if (i > str.length()) {
			i = i - 1;
		}
		return (str.substring(0, i));
	} // prtdbl()



	public String prttxt(int val) {
		String str = null;

		if (val == -1)
			str = "System Fault";
		else if (val == 0)
			str = "RTT";
		else if (val == 1)
			str = "Dial-up";
		else if (val == 2)
			str = "T1";
		else if (val == 3)
			str = "Ethernet";
		else if (val == 4)
			str = "T3";
		else if (val == 5)
			str = "FastE";
		else if (val == 6)
			str = "OC-12";
		else if (val == 7)
			str = "GigE";
		else if (val == 8)
			str = "OC-48";
		else if (val == 9)
			str = "10 Gig";
		return(str);
	} // prttxt()
	
	

	/* This routine saves the specific value into the variable of the
	* same name.  There should probably be an easier way to do this.
	*/
	public void save_dbl_values(String sysvar, double sysval) {
		if(sysvar.equals("bw:")) 
			estimate = sysval;
		else if(sysvar.equals("loss:")) 
			loss = sysval;
		else if(sysvar.equals("avgrtt:")) 
			avgrtt = sysval;
		else if(sysvar.equals("waitsec:")) 
			waitsec = sysval;
		else if(sysvar.equals("timesec:")) 
			timesec = sysval;
		else if(sysvar.equals("order:")) 
			order = sysval;
		else if(sysvar.equals("rwintime:")) 
			rwintime = sysval;
		else if(sysvar.equals("sendtime:")) 
			sendtime = sysval;
		else if(sysvar.equals("cwndtime:")) 
			cwndtime = sysval;
		else if(sysvar.equals("rttsec:")) 
			rttsec = sysval;
		else if(sysvar.equals("rwin:")) 
			rwin = sysval;
		else if(sysvar.equals("swin:")) 
			swin = sysval;
		else if(sysvar.equals("cwin:")) 
			cwin = sysval;
		else if(sysvar.equals("spd:")) 
			spd = sysval;
		else if(sysvar.equals("aspd:")) 
			aspd = sysval;
	} // save_dbl_values()



	public void save_int_values(String sysvar, int sysval) {
		/*  Values saved for interpretation:
		*	SumRTT 
		*	CountRTT
		*	CurrentMSS
		*	Timeouts
		*	PktsRetrans
		*	SACKsRcvd
		*	DupAcksIn
		*	MaxRwinRcvd
		*	MaxRwinSent
		*	Sndbuf
		*	Rcvbuf
		*	DataPktsOut
		*	SndLimTimeRwin
		*	SndLimTimeCwnd
		*	SndLimTimeSender
		*/   
		if(sysvar.equals("MSSSent:")) 
			MSSSent = sysval;
		else if(sysvar.equals("MSSRcvd:")) 
			MSSRcvd = sysval;
		else if(sysvar.equals("ECNEnabled:")) 
			ECNEnabled = sysval;
		else if(sysvar.equals("NagleEnabled:")) 
			NagleEnabled = sysval;
		else if(sysvar.equals("SACKEnabled:")) 
			SACKEnabled = sysval;
		else if(sysvar.equals("TimestampsEnabled:")) 
			TimestampsEnabled = sysval;
		else if(sysvar.equals("WinScaleRcvd:")) 
			WinScaleRcvd = sysval;
		else if(sysvar.equals("WinScaleSent:")) 
			WinScaleSent = sysval;
		else if(sysvar.equals("SumRTT:")) 
			SumRTT = sysval;
		else if(sysvar.equals("CountRTT:")) 
			CountRTT = sysval;
		else if(sysvar.equals("CurMSS:"))
			CurrentMSS = sysval;
		else if(sysvar.equals("Timeouts:")) 
			Timeouts = sysval;
		else if(sysvar.equals("PktsRetrans:")) 
			PktsRetrans = sysval;
		else if(sysvar.equals("SACKsRcvd:")) 
			SACKsRcvd = sysval;
		else if(sysvar.equals("DupAcksIn:")) 
			DupAcksIn = sysval;
		else if(sysvar.equals("MaxRwinRcvd:")) 
			MaxRwinRcvd = sysval;
		else if(sysvar.equals("MaxRwinSent:")) 
			MaxRwinSent = sysval;
		else if(sysvar.equals("Sndbuf:")) 
			Sndbuf = sysval;
		else if(sysvar.equals("X_Rcvbuf:")) 
			Rcvbuf = sysval;
		else if(sysvar.equals("DataPktsOut:")) 
			DataPktsOut = sysval;
		else if(sysvar.equals("FastRetran:")) 
			FastRetran = sysval;
		else if(sysvar.equals("AckPktsOut:")) 
			AckPktsOut = sysval;
		else if(sysvar.equals("SmoothedRTT:")) 
			SmoothedRTT = sysval;
		else if(sysvar.equals("CurCwnd:")) 
			CurrentCwnd = sysval;
		else if(sysvar.equals("MaxCwnd:")) 
			MaxCwnd = sysval;
		else if(sysvar.equals("SndLimTimeRwin:")) 
			SndLimTimeRwin = sysval;
		else if(sysvar.equals("SndLimTimeCwnd:")) 
			SndLimTimeCwnd = sysval;
		else if(sysvar.equals("SndLimTimeSender:")) 
			SndLimTimeSender = sysval;
		else if(sysvar.equals("DataBytesOut:")) 
			DataBytesOut = sysval;
		else if(sysvar.equals("AckPktsIn:")) 
			AckPktsIn = sysval;
		else if(sysvar.equals("SndLimTransRwin:"))
			SndLimTransRwin = sysval;
		else if(sysvar.equals("SndLimTransCwnd:"))
			SndLimTransCwnd = sysval;
		else if(sysvar.equals("SndLimTransSender:"))
			SndLimTransSender = sysval;
		else if(sysvar.equals("MaxSsthresh:"))
			MaxSsthresh = sysval;
		else if(sysvar.equals("CurRTO:"))
			CurrentRTO = sysval;
		else if(sysvar.equals("c2sData:"))
			c2sData = sysval;
		else if(sysvar.equals("c2sAck:"))
			c2sAck = sysval;
		else if(sysvar.equals("s2cData:"))
			s2cData = sysval;
		else if(sysvar.equals("s2cAck:"))
			s2cAck = sysval;
		else if(sysvar.equals("PktsOut:"))
			PktsOut = sysval;
		else if(sysvar.equals("mismatch:"))
			mismatch = sysval;
		else if(sysvar.equals("congestion:"))
			congestion = sysval;
		else if(sysvar.equals("bad_cable:"))
			bad_cable = sysval;
		else if(sysvar.equals("half_duplex:"))
			half_duplex = sysval;
		else if(sysvar.equals("CongestionSignals:"))
			CongestionSignals = sysval;
		else if(sysvar.equals("RcvWinScale:")) {
			if (RcvWinScale > 15)
				RcvWinScale = 0;
			else
				RcvWinScale = sysval;
		}
	}  // save_int_values()



//  public static void main(String[] args)
//  {
//	if (DEBUG) System.out.println("ANL/Internet2 NDT");
//    
//    Tcpbw100 test = new Tcpbw100();
//    try {
//		test.dottcp("ndt.iupui.ord01.measurement-lab.org");
//	} catch (IOException e) {
//		// TODO Auto-generated catch block
//		e.printStackTrace();
//	}
//  }
	
	
} // class: Tcpbw100
