// Flot documentation: http://people.iola.dk/olau/flot/API.txt

var plot_colors = ['#0000C3', '#DBC01C', '#5200BF', '#DB7C1C', '#0068B8'];
// See http://colorschemedesigner.com


function dump(obj) {
    var out = '';
    for (var i in obj) {
        out += i + ": " + obj[i] + "\n";
    }

    alert(out);

    // or, if you wanted to avoid alerts...

    var pre = document.createElement('pre');
    pre.innerHTML = out;
    document.body.appendChild(pre)
}

function plot(obj, key, value) {

    if(value == null) {
        plot_usePlaceholder($(obj), true);
        return;
    }

    // function prototype is set up so it can be called directly 
    // value: [allSeriesData, optionData]
    //
    // allSeriesData: list of series to plot

    var allSeriesData = value[0];
    var options = {};
    if(value.length > 1) {
        options = value[1];
    }

    if(!("legend" in options)) {
        // include the legend?
        options.legend = {};
    }
    if( $(obj).attr("data-legend") != null) {
        options.legend.show = true;
        if( $(obj).attr("data-legend") != "") {
           options.legend.container = $(obj.getAttribute("data-legend"));
        }
    } else {
        options.legend.show = false;
    }
    
    options.colors = plot_colors;

    if("hoverable" in options || "clickable" in options) {
	options.grid = {};
    }
    if("hoverable" in options) {
	options.grid.hoverable = true;
    }
    if("clickable" in options) {
	options.grid.hoverable = true;
    }

    $.plot(obj, allSeriesData, options);

    plot_usePlaceholder($(obj), false);

}

var plot_boxwidth = 0.8;

function barPlot(obj, key, value) {

    if(value == null) {
        plot_usePlaceholder($(obj), true);
        return;
    }

    var series = value[0]; // map from series name to data
    var opts = {};
    if(value.length > 1) {
        opts = value[1];
    }

    var hoverable = "hoverable" in opts && opts.hoverable == true;
    var clickable = "clickable" in opts && opts.clickable == true;
    var horizontal = "horizontal" in opts && opts.horizontal == true;

    // Get all x-values, unless xOrder is specified in opts.
    var xOrder = [];
    if("xOrder" in opts) {
        xOrder = opts["xOrder"];
    }// else {
    //    for 
   // }
    
    if(horizontal) {
        // copy and reverse the xOrder array
        // we want x-values to proceed _down_ the plot
        var tmp = [];
        for(var i in xOrder) {
            tmp.push(xOrder[i]);
        }
        tmp.reverse();
        xOrder = tmp;
    }

    // Options for doing the graphs...

    var newSeries = [];
    var order = 1;
    var ymin = 0;
    var ymax = 0;
    for(var i in series) {
        var name = series[i][0];
        var data = series[i][1];
        var newData = [];
        for(var n in xOrder) {
            if(xOrder[n] in data) {
                var val = data[xOrder[n]];
                newData.push([n, val]);
                if(val < ymin) {ymin = val;}
                if(val > ymax) {ymax = val;}
            } else {
                newData.push([n, null]);
            }
        }
        var newObj = {data:newData, bars:{order:order}};
        if(name != null) {
            newObj.label = name;
        }
        newSeries.push(newObj);
        order = order + 1;
    }

    var barwidth = plot_boxwidth / series.length;

    var ticks = [];
    for(i in xOrder) {
        ticks.push([i, xOrder[i]]);
    }

    var options = {bars: {barWidth: barwidth, show: true},
        xaxis: {ticks: ticks},
        colors: plot_colors
    };

    if(horizontal) {
        options.bars.horizontal = horizontal;
        options.yaxis = {};
        options.yaxis.ticks = options.xaxis.ticks;
        delete options.xaxis.ticks;
        if(order == 2) {
            // there was only 1 series, and will not be centered automatically
            options.bars.align = "center";
        }
        options.yaxis.min = -0.5;
        options.yaxis.max = 0.5 + xOrder.length - 1;
        if(ymin < 0) { ymin *= 1.1; }
        options.xaxis.min = ymin;
        options.xaxis.max = ymax * 1.1;
    }

    if(hoverable || clickable) {
	options.grid = {};
    }
    if(hoverable) {
	options.grid.hoverable = true;
    }
    if(clickable) {
	options.grid.clickable = true;
    }

    options.bars.align = "center";
    plot(obj, key, [newSeries, options]);

    plot_usePlaceholder($(obj), false);

}

function plot_usePlaceholder(jqobj, use) {
    // jqobj is the inner div that will be updated by the plot obj
    var top = jqobj.parents(".plot-top");
    
    top.find(".plot-placeholder").css("display", use ? "block" : "none");
    top.find(".plot-container").css("display", use ? "none" : "block");
}
