// This module does several rewriting tasks so that
// the base HTML code can stay relatively simple.
//
// Rewriting descriptions (by matching JQuery/CSS rule):
// .plot : process internal .xlabel and .ylabel elements so that they are placed correctly. Must be called before .valign
// .valign : cross-browser approach for vertically centering an element

function rewrite(target) {
    rewrite_a_href(target);
    rewrite_plot(target);
}

function rewrite_a_href(target) {
    var baseTarget = "";
    if(target != undefined && target != null) {
        baseTarget = target + " ";
    }
    $(baseTarget + "a").map(function(index, obj) {
            if($(obj).attr("href") == undefined) { return; }
            var href = $(obj).attr("href");
            if(href.indexOf("http://") == -1 || href.indexOf("#") != -1) { return; }
            
            var repl = $("<span class='link' onclick='javascript:dasu_openWebLink(\""+href+"\");'></span>");
            
            var toCopy = ['data-key', 'data-func', 'data-legend', 'style', 'id'];
            for(var i in toCopy) {
                var attr = toCopy[i];
                if($(obj).attr(attr) != undefined) {
                    repl.attr(attr, $(obj).attr(attr));
                }
            }
            if($(obj).attr("class")) {
                var classes = $(obj).attr("class").split(" ");
                for(var i in classes) {
                    repl.addClass(classes[i]);
                }
            }
            
            $(obj).replaceWith(repl);
            var html = $.trim($(obj).html());
            if(!$(obj).hasClass("noicon")) {
                html = html + " <span class='ui-icon ui-icon-newwin' style='display: inline-block;'></span>";
            }
            repl.append(html);
    });
}

var rewrite_plot_ylabel_padding = 5;

function rewrite_rotate_helper(jqobj, height) {
    // thanks http://css3please.com/
    var pfxs = ["-moz-", "-o-", "-webkit-", "-ms-", ""];
    for(var i in pfxs) {
        var pfx = pfxs[i];
        jqobj.css(pfx + "transform", "rotate(270deg) translatex(-" + height + "px)");
        jqobj.css(pfx + "transform-origin", "top left");
    }
    jqobj.css("filter", "progid:DXImageTransform.Microsoft.Matrix(M11=-1.836909530733566e-16, M12=1, M21=-1, M22=-1.836909530733566e-16, sizingMethod='auto expand')");
    jqobj.css("zoom", "1");
}

function rewrite_plot(target) {
    var baseTarget = "";
    if(target != null) {
        baseTarget = target + " ";
    }
    $(baseTarget + ".plot").map(function(index, obj) {
            // grab the plot dimensions
            var width = $(obj).outerWidth();
            var height = $(obj).outerHeight();

            // make a top-level div that will contain the plot, and a placeholder.
            var topLevel = $("<div></div>");
            topLevel.addClass("plot-top");

            // construct the outer container and substitute...
            var outer = $("<div></div>");
            outer.addClass("plot-container");
            outer.css("position", "relative");

            // if a ylabel exists, put it in
            var ylabel = $(obj).children(".ylabel");
            var ylabelWidth = 0;
            var ylabelHeight = 0;
            var isYLabel = false;
            if(ylabel.length != 0) {
                var use = ylabel.eq(0);
                use.css("text-align", "center");
                use.css("width", height + "px");
                ylabelWidth = use.height();
                ylabelHeight = use.children().eq(0).width();
                isYLabel = true;
                ylabel.detach(); // detach all...

                var leftShift = parseInt((ylabelHeight - ylabelWidth) / 2);

                var ylabelOuter = $("<div></div>");
                ylabelOuter.css("position", "absolute");
                ylabelOuter.css("top", "0px");
                ylabelOuter.css("left", "0px");
                ylabelOuter.css("height", height + "px");
                ylabelOuter.css("width", ylabelWidth + "px");
                ylabelOuter.append(use);
                rewrite_rotate_helper(use, height);
                outer.append(ylabelOuter);
            }

            // add ylabel padding
            if(isYLabel) {
                ylabelWidth = ylabelWidth + rewrite_plot_ylabel_padding;
            }


            // add the plot div
            $(obj).css("display", "inline-block");
            $(obj).css("position", "absolute");
            $(obj).css("top", "0px");
            $(obj).css("left", ylabelWidth + "px");
            $(obj).replaceWith(topLevel);
            outer.append(obj);
            topLevel.append(outer);

            // if an xlabel exists, put it in
            var xlabel = $(obj).children(".xlabel");
            var xlabelHeight = 0;
            if(xlabel.length != 0) {
                var use = xlabel.eq(0);
                xlabelHeight = use.height();
                xlabel.detach(); // detach all -- in case there are multiples :)

                var xlabelOuter = $("<div></div>");
                xlabelOuter.css("position", "absolute");
                xlabelOuter.css("top", height + "px");
                xlabelOuter.css("left", ylabelWidth + "px");
                xlabelOuter.css("width", width + "px");
                xlabelOuter.css("text-align", "center");
                xlabelOuter.append(use);
                outer.append(xlabelOuter);
            }
			
			// if a caption exists, put it in
            var captions = $(obj).children(".caption");
            var captionHeight = 0;
            if(captions.length != 0) {
                var use = captions.eq(0);
                captionHeight = use.height();
                captions.detach(); // detach all -- in case there are multiples :)

                var captionOuter = $("<div></div>");
                captionOuter.css("position", "absolute");
                captionOuter.css("top", (height + xlabelHeight) + "px");
                captionOuter.css("left", "0px");
                captionOuter.css("width", (width + ylabelWidth) + "px");
                captionOuter.append(use);
                outer.append(captionOuter);
            }

            outer.height(height + xlabelHeight + captionHeight);
            outer.width(width + ylabelWidth);

            // generate placeholder...
            var placeholders = $(obj).children(".placeholder");
            var isPlaceholder = false;
            if(placeholders.length != 0) {
                var use = placeholders.eq(0);
                isPlaceholder = true;
                placeholders.detach(); // detach all...

                var pl = util_placeholder(width + ylabelWidth, height + xlabelHeight + captionHeight, use);
                pl.addClass("plot-placeholder");

                topLevel.append(pl);

                outer.css("display", "none");
            }

    });
}
