var storage_values = {};
var storage_registry = {};

// Initialize the key/value database, and auto-register all the elements in the page.
function storage_autoRegister(target) {
    var baseTarget = "";
    if(target != null && target != undefined) {
        baseTarget = target + " ";
    }
    $(baseTarget + ".value").map(function(index, obj) {
        var func = null;
        //log("registering " + this.getAttribute("key"));
        if(obj.getAttribute("data-func") != null) {
            func = eval(obj.getAttribute("data-func"));
        }
        var keys = [];
        if($(obj).attr("data-key") != undefined) {
            keys = $(obj).attr("data-key").split(":");
        }
        for(var i in keys) {
            var key = $.trim(keys[i]);
            if(key != "") {
                storage_register(key, obj, func);
            }
        }
    });
}

function storage_init(key) {
    // Initialize data structure for this key.
    // Should not be called directly by the UI or client.
    if(key in storage_values) return;
    storage_values[key] = null;
    storage_registry[key] = [];

    if(window.debug_monitorKey) debug_monitorKey(key);
}

function storage_register(key, obj, func) {
    // Register an element of the dom (obj) as a container for the given key
    // If func is not provided (i.e. null), the element's text will simply be replaced
    // If func is provided, func is responsible for doing the updating. It will be called with the id, key, and value.
    // If obj is null, func will still be called.
    storage_init(key);
    //$(obj).css("border", "1px black solid");
    storage_registry[key].push([obj, func]);
    storage_updateInstances(key);
}

function storage_update(key, value) {
    // When a local function updates this key, store it locally and send the update to the client.
    // Return the old value
    storage_init(key);
    storage_values[key] = value;
    storage_updateInstances(key);

    dasu_update(key, JSON.stringify(value));
}

function storage_updated(key, value) {
    // This is called by the client when a key changes.
    storage_init(key);
    storage_values[key] = value;
    storage_updateInstances(key);
}

function storage_updateAllInstances() {
    for(var key in storage_values) {
        storage_updateInstances(key);
    }
}

var storage_debug_updateKey = null;
var storage_debug_updateObj = null;
var storage_debug_updateFun = null;

function storage_updateInstances(key) {
    storage_debug_updateKey = key;
    // This is the code to actually invoke the updates to the local UI when a key is updated.
    // It should not need to be called on its own.
    storage_init(key);

    var val = storage_values[key];
    for(var arr in storage_registry[key]) {
        var obj = storage_registry[key][arr][0];
        storage_debug_updateObj = obj;
        var func = storage_registry[key][arr][1];
        storage_debug_updateFun = func;
        
        if(func == null) {
            $(obj).text(val);
        } else {
            func(obj, key, val);
        }

        storage_debug_updateObj = null;
        storage_debug_updateFun = null;
    }
    storage_debug_updateKey = null;
}

function storage_get(key) {
    storage_init(key);
    return storage_values[key];
}

function storage_toggle(key) {
    storage_init(key);
    var oldVal = false; // Assume it was not set
    if(key in storage_values && storage_values[key]) {
        oldVal = true;
    }
    var newVal = !oldVal;
    storage_update(key, newVal);
}

function storage_reset() {
    storage_values = {};
    storage_registry = {};
    storage_autoRegister();
}
