// John Otto
//
// Functions for approximating time, and updating those values periodically...

var timeSpans = new Object();
var timeIntervalObject;
var timeOffset = 0;

function time_setRemoteOffset(reportedSystemTime) {
	var now = new Date();
	var dateReplace = reportedSystemTime.replace(/-/g, "/");
	var dateFinal = dateReplace + " GMT";
	var d = new Date(dateFinal);
	
	timeOffset = d.getTime() - now.getTime();
}

function timeStartLoop(millis) {
	timeIntervalObject = window.setInterval(time_updateAll, millis);
}

function timeStopLoop() {
	window.clearInterval(timeIntervalObject);
	timeIntervalObject = null;
}

function time_updateAll() {
	var toDelete = new Array();
	
	for(i in timeSpans) {
		if(timeSpans[i]["span"].parentNode == null) {
			toDelete.push(i);
			continue;
		}
		time_updateSpan(timeSpans[i]);
	}

	for(i in toDelete) {
		delete timeSpans[toDelete[i]];
	}
}

function time_makeSpanString(dateString) {
    var dateReplace = dateString.replace(/-/g, "/");
    var dateFinal = dateReplace + " GMT";
    var d = new Date(dateFinal);
    var dMillis = d.getTime();
    var fixedMillis = dMillis - timeOffset;
	
    return time_makeSpan(fixedMillis);
}

function time_makeSpan(dMillis) {
	var d = new Date(dMillis);
	
	var thisDate = new Object();
	thisDate["string"] = d.toGMTString();
	thisDate["date"] = d;
	
	var span = document.createElement("span");
	
	thisDate["span"] = span;
	thisDate["approxMode"] = 1;
	
	//span.onclick = time_makeToggleClosure(thisDate);
	
	var index = d.toGMTString() + "_" + Math.floor(Math.random()*100000);
	
	timeSpans[index] = thisDate;
	time_updateSpan(thisDate);
	return span;
}

function time_makeToggleClosure(obj) {
	return ( function() {
		var approxMode = obj["approxMode"];
		if(approxMode == 1) {
			obj["approxMode"] = 0;
		} else {
			obj["approxMode"] = 1;
		}
		time_updateSpan(obj);
	} );
}

function time_updateSpan(obj) {
	var text;
	if(obj["approxMode"] == 0) {
		text = obj["date"].toLocaleString();
	} else {
		var now = new Date().getTime();
		var then = obj["date"].getTime();
		var diff = now - then;
		
		var future = diff < 0;
		if(future) {
			diff *= -1;
		}

		var result = time_getApproximation(diff);
		text = result["text"] + " ";
		
		var smallDiff = diff - (result["value"] * time_getUnitMillisecondValue(result["unit"]));

		if(smallDiff != 0) {
			var smallResult = time_getApproximation(smallDiff);
			if(time_isReasonableNextUnit(result["unit"], smallResult["unit"])) {
				text += "and " + smallResult["text"] + " ";
			}
		}
		
		if(future) {
			// update text for event in the future...
		} else {
			// update text for event in the past...
		}
	}

	// Compare the old value to the new value.
	// Only make the change _IF THEY'RE DIFFERENT_.
	var modify = !obj["span"].hasChildNodes() || text != obj["span"].firstChild.nodeValue;
	if(modify) {
		while(obj["span"].hasChildNodes()) {
			obj["span"].removeChild(obj["span"].firstChild);
		}
		obj["span"].appendChild(document.createTextNode(text));
	}
}

function time_getUnitMillisecondValue(unit) {
	var val;
	if(unit == "millisecond") {
		val = 1;
	} else if(unit == "second") {
		val = 1000;
	} else if(unit == "minute") {
		val = 60000;
	} else if(unit == "hour") {
		val = 3600000;
	} else if(unit == "day") {
		val = 86400000;
	} else if(unit == "week") {
		val = 86400000 * 7;
	} else if(unit == "month") {
		val = 86400000 * 30;
	} else /* year */ {
		val = 86400000 * 365;
	}
	return val;
}

function time_isReasonableNextUnit(unit1, unit2) {
	if((unit1 == "year" && unit2 == "month") || (unit1 == "month" && (unit2 == "day" || unit2 == "week")) || (unit1 == "week" && unit2 == "day") || (unit1 == "day" && unit2 == "hour") || (unit1 == "hour" && unit2 == "minute") || (unit1 == "minute" && unit2 == "second")) {
		return true;
	}
	return false;
}

function time_getApproximation(diff) {
	var val;
	var unit;
	var text;
	
	// Get the scale:
	if(diff < 1000) { // smaller than a second
		// millisecond scale
		unit = "millisecond";
		val = diff;
	} else if(diff < 60000) { // smaller than a minute
		// second scale
		unit = "second";
		val = Math.floor(diff / 1000);
	} else if(diff < 3600000) { // smaller than an hour
		// minute scale
		unit = "minute";
		val = Math.floor(diff / 60000);
	} else if(diff < 86400000) { // smaller than a day
		// hour scale
		unit = "hour";
		val = Math.floor(diff / 3600000);
	} else if(diff < 86400000 * 7) { // smaller than a week
		// day scale
		unit = "day";
		val = Math.floor(diff / 86400000);
	} else if(diff <= 86400000 * 30) { // smaller than a month
		// week scale
		unit = "week";
		val = Math.floor(diff / 86400000 / 7);
	} else if(diff <= 86400000 * 365) { // smaller than a year
		// month scale
		unit = "month";
		val = Math.floor(diff / 86400000 / 30);
	} else { // otherwise...
		// year scale
		unit = "year";
		val = Math.floor(diff / 86400000 / 365);
	}
	
	text = val.toString() + " " + unit;
	if(val > 1) {
		text += "s";
	}
	
	var obj = new Object();
	obj["value"] = val;
	obj["unit"] = unit;
	obj["text"] = text;
	
	return obj;
}
