//$("#internet_characteristics").tabs({ selected: 0, show: storage_updateAllInstances });
//$("#internet_web").tabs({ selected: 0, show: storage_updateAllInstances });

function internet_webLatency(obj, key, val) {
    if(val == null) {
        plot_usePlaceholder($(obj), true);
        return;
    }
    
    var toShow = {};
    var order = [];

    // Show local configured servers
    var urls = storage_get("internet.web.sites");
    var data = storage_get("internet.web.siteData");

    if(urls != null && data != null) { 
        for(var i in urls) {
            var url = urls[i];
            if(url in data) {
                var subData = data[url];
                if("ping" in subData && subData["ping"] != -1) {
                    var show = url + " (" + subData["ip"] + ")";
                    toShow[show] = subData["ping"];
                    order.push(show);
                }
            }
        }
    }

    var data = [
        [
            [null, toShow] // series
        ],
        {
            xOrder: order,
            horizontal: true
        }
    ];

    if(order.length == 0) {
        plot_usePlaceholder($(obj), true);
        return;
    }

    plot_usePlaceholder($(obj), false);
    barPlot(obj, key, data);
}

var internetDnsPendingKey = "net.dns.test.pending";
var internetDnsFinishedKey = "net.dns.test.finished";

function internetDnsFinished(obj, key, val) {
    $(obj).children().remove();
    if(val == null) {
        return;
    }

    var xOrder = [];
    var data = [];

    $(obj).append($("<h4>Locally Configured DNS Servers</h4>"));

    var minLocalLat = null;
    var minLocalOrder = null;
    var bestLocalLat = null;
    var bestLocalOrder = null;

    // table of local dns servers
    var tab = $("<table></table>");
    tab.addClass("internetDns");
    tab.append($("<tr class='head'><td>Order</td><td>IP Address</td><td>Responding IP</td><td>Domain</td><td>Latency (ms)</td></tr>"));
    for(var i in val["servers"]) {
        var arr = val["servers"][i];
        var tr = $("<tr></tr>");
        //order
        tr.append($("<td>" + (parseInt(i)+1) + "</td>"));
    
        // ip addresses
        tr.append($("<td>" + arr[0] + "</td>"));
        if(arr[2] == "A") {
            tr.append($("<td>" + arr[1] + "</td>"));
        } else {
            tr.append($("<td></td>"));
        }

        // domain
        var dom = null;
        if(arr[2] == "A" && arr[1] in val["reverse"]) {
            tr.append($("<td>" + val["reverse"][arr[1]] + "</td>"));
        } else if(arr[0] in val["reverse"]) {
            tr.append($("<td>" + val["reverse"][arr[0]] + "</td>"));
        } else {
            tr.append($("<td>&mdash;</td>"));
        }

        // latency
        var lat = null;
        if(arr[2] == "A" && arr[1] in val["results"]) {
            tr.append($("<td>" + val["results"][arr[1]] + "</td>"));
            lat = val["results"][arr[1]];
        } else if(arr[0] in val["results"]) {
            tr.append($("<td>" + val["results"][arr[0]] + "</td>"));
            lat = val["results"][arr[0]];
        } else {
            tr.append($("<td>&mdash;</td>"));
        }
        tab.append(tr);

        if(lat != null) {
            var name = "Configured #" + (parseInt(i)+1);
            xOrder.push(name);
            data[name] = lat;
           
            // see which has the best performance...
            if(minLocalOrder == null) {
                minLocalOrder = parseInt(i);
                minLocalLat = lat;
            }
            if(bestLocalLat == null || lat < bestLocalLat) {
                bestLocalLat = lat;
                bestLocalOrder = parseInt(i);
            }
        }

    }

    $(obj).append(tab);

    var minPublicLat = null;
    var minPublicName = null;
    var minPublicIp= null;

    // See what public dns server latencies we have...
    for(var i in val["publicServers"]) {
        var ip = val["publicServers"][i][0];
        var name = val["publicServers"][i][1];

        if(ip in val["results"]) {
            xOrder.push(name);
            data[name] = val["results"][ip];

            if(minPublicLat == null || val["results"][ip] < minPublicLat) {
                minPublicLat = val["results"][ip];
                minPublicName = name;
                minPublicIp = ip;
            }
        }
    }

    $(obj).append($("<h4>Latency Comparison to Public DNS Services</h4>"));

    // build the DNS latency comparison graph
    var plotcontainer = $("<div id='internet_web_dns'>" + $.trim($("#internet_web_dns_plot").html()) + "</div>");
    plotcontainer.children().eq(0).addClass("plot"); // make it rewritable as a plot
    $(obj).append(plotcontainer);
    rewrite_plot("#internet_web_dns");
    var plotdiv = plotcontainer.find(".plot").eq(0);

    // build up the data structure
    var plotdata = [
        [
            [null, data] // series
        ],
        {
            "xOrder": xOrder,
            "horizontal": true
        }
    ];

    if(xOrder.length == 0) {
        plot_usePlaceholder(plotdiv, true);
        return;
    }

    plot_usePlaceholder(plotdiv, false);
    barPlot(plotdiv.get(), key, plotdata);

    // run some diagnostic checks on the configuration
    tab = $("<table></table>");
    tab.addClass("internetDns");
    tab.append($("<tr class='head'><td>Status</td><td>Description</td></tr>"));

    // do some checks on the configuration
    
    // is there a faster (lower latency) public server than the configured primary?
    if(minLocalLat != null && minPublicLat != null) {
        var tr = $("<tr></tr>");
        if(minLocalLat <= minPublicLat) {
            tr.addClass("good");
            tr.append($("<td>OK</td>"));
            tr.append($("<td>Server #" + (minLocalOrder+1) + " has lower latency than the best tested public DNS servers.</td>"));
        } else {
            tr.addClass("warning");
            tr.append($("<td>Warning</td>"));
            tr.append($("<td>The " + minPublicName + " server has lower latency than Server #" + (minLocalOrder+1) + ".<br> Suggest using " + minPublicIp + " as your primary DNS server.</td>"));
        }
        tab.append(tr);
    }

    // is the configured primary the fastest of all local configured servers?
    if(minLocalLat != null) {
        var tr = $("<tr></tr>");
        if(minLocalOrder == bestLocalOrder) {
            tr.addClass("good");
            tr.append($("<td>OK</td>"));
            tr.append($("<td>Your primary server has the lowest latency.</td>"));
        } else {
            tr.addClass("warning");
            tr.append($("<td>Warning</td>"));
            tr.append($("<td>Server #" + (minLocalOrder + 1) + " has higher latency than Server #" + (bestLocalOrder + 1) + ".<br> Suggest moving Server #" + (bestLocalOrder + 1) + " before Server #" + (minLocalOrder + 1) + ".</td>"));
        }
        tab.append(tr);
    }

    $(obj).prepend(tab);
    $(obj).prepend($("<h4>DNS Configuration Diagnostics</h4>"));
}

function internetDnsStatus(obj, key, val) {
    var pending = storage_get(internetDnsPendingKey);
    var finished = storage_get(internetDnsFinishedKey);

    $(obj).children().remove();

    var running = pending != null;
    if(running) {
        $(obj).append($("<h4>Test running...</h4>"));
    } else if(finished != null) {
        var ch1 = $("<h4>Next test scheduled in </h4>");
        ch1.append(time_makeSpan(finished["scheduled"]));

        $(obj).append(ch1);
    }
}

